Package: nestedLogit
Title: Nested Dichotomy Logistic Regression Models
Version: 0.4.2
Date: 2026-05-27
Authors@R: c(
    person("John", "Fox", , "jfox@mcmaster.ca", role = "aut",
           comment = c(ORCID = "0000-0002-1196-8012")),
    person("Michael", "Friendly", , "friendly@yorku.ca", role = c("aut", "cre"),
           comment = c(ORCID = "0000-0002-3237-0941")),
    person("Achim", "Zeileis", , "Achim.Zeileis@uibk.ac.at", role = "ctb",
           comment = c(ORCID = "0000-0003-0918-3766"))
  )
Description: Provides functions for specifying and fitting nested
    dichotomy logistic regression models for a multi-category response and
    methods for summarising and plotting those models.  Nested dichotomies are
    statistically independent, and hence provide an additive decomposition
    of tests for the overall 'polytomous' response.  When the dichotomies
    make sense substantively, this method can be a simpler alternative to
    the standard 'multinomial' logistic model which compares response
    categories to a reference level.  See: J. Fox (2016), "Applied
    Regression Analysis and Generalized Linear Models", 3rd Ed., ISBN
    1452205663.
License: GPL (>= 2)
URL: https://github.com/friendly/nestedLogit,
        https://friendly.github.io/nestedLogit/
BugReports: https://github.com/friendly/nestedLogit/issues
Depends: R (>= 4.1.0)
Imports: broom, car, dplyr, effects, graphics, grDevices, stats,
        stringr, tibble, scales
Suggests: AER, carData, equatiomatic, DescTools, geomtextpath, ggplot2,
        ggeffects, here, insight, lobstr, knitr, nnet, parameters,
        performance, rmarkdown, see, spelling, testthat, tidyr, MASS,
        VGAM, mlogit, vcd
VignetteBuilder: knitr, rmarkdown
Encoding: UTF-8
Language: en-US
LazyData: TRUE
Config/roxygen2/version: 8.0.0
NeedsCompilation: no
Packaged: 2026-05-28 13:19:08 UTC; friendly
Author: John Fox [aut] (ORCID: <https://orcid.org/0000-0002-1196-8012>),
  Michael Friendly [aut, cre] (ORCID:
    <https://orcid.org/0000-0002-3237-0941>),
  Achim Zeileis [ctb] (ORCID: <https://orcid.org/0000-0003-0918-3766>)
Maintainer: Michael Friendly <friendly@yorku.ca>
Repository: CRAN
Date/Publication: 2026-05-28 14:10:01 UTC
Built: R 4.5.2; ; 2026-05-28 16:22:25 UTC; unix

# Generated by roxygen2: do not edit by hand

S3method(Anova,nestedLogit)
S3method(RSQ,multinom)
S3method(RSQ,nestedLogit)
S3method(RSQ,polr)
S3method(anova,nestedLogit)
S3method(as.character,dichotomies)
S3method(as.data.frame,predictDichotomies)
S3method(as.data.frame,predictNestedLogit)
S3method(as.dichotomies,matrix)
S3method(as.matrix,continuationDichotomies)
S3method(as.matrix,dichotomies)
S3method(as.tree,continuationDichotomies)
S3method(as.tree,dichotomies)
S3method(broom::glance,nestedLogit)
S3method(broom::tidy,nestedLogit)
S3method(car::Anova,nestedLogit)
S3method(car::linearHypothesis,nestedLogit)
S3method(coef,nestedLogit)
S3method(confint,predictNestedLogit)
S3method(createDichotomies,default)
S3method(createDichotomies,dichotomies)
S3method(createDichotomies,list)
S3method(effects::Effect,nestedLogit)
S3method(equatiomatic::extract_eq,nestedLogit)
S3method(fitted,nestedLogit)
S3method(linearHypothesis,nestedLogit)
S3method(logLik,nestedLogit)
S3method(models,nestedLogit)
S3method(plot,nestedLogit)
S3method(predict,nestedLogit)
S3method(print,Anova.nestedLogit)
S3method(print,RSQ.multinom)
S3method(print,RSQ.nestedLogit)
S3method(print,RSQ.polr)
S3method(print,anova.nestedLogit)
S3method(print,dichotomies)
S3method(print,nestedLogit)
S3method(print,nestedLogit_equations)
S3method(print,predictDichotomies)
S3method(print,predictNestedLogit)
S3method(print,summary.nestedLogit)
S3method(stats::logLik,nestedLogit)
S3method(summary,nestedLogit)
S3method(update,nestedLogit)
S3method(vcov,nestedLogit)
export(RSQ)
export(as.dichotomies)
export(as.tree)
export(continuationLogits)
export(dichotomy)
export(logits)
export(models)
export(nestedLogit)
importFrom(broom,glance)
importFrom(broom,tidy)
importFrom(car,Anova)
importFrom(car,linearHypothesis)
importFrom(dplyr,bind_cols)
importFrom(dplyr,select)
importFrom(effects,Effect)
importFrom(grDevices,adjustcolor)
importFrom(grDevices,palette)
importFrom(graphics,arrows)
importFrom(graphics,axis)
importFrom(graphics,box)
importFrom(graphics,matplot)
importFrom(graphics,polygon)
importFrom(graphics,text)
importFrom(graphics,title)
importFrom(scales,hue_pal)
importFrom(stats,AIC)
importFrom(stats,BIC)
importFrom(stats,anova)
importFrom(stats,binomial)
importFrom(stats,coef)
importFrom(stats,confint)
importFrom(stats,fitted)
importFrom(stats,formula)
importFrom(stats,glm)
importFrom(stats,logLik)
importFrom(stats,model.frame)
importFrom(stats,model.matrix)
importFrom(stats,model.response)
importFrom(stats,na.omit)
importFrom(stats,pchisq)
importFrom(stats,predict)
importFrom(stats,qnorm)
importFrom(stats,setNames)
importFrom(stats,update)
importFrom(stringr,str_remove)
importFrom(tibble,rownames_to_column)
importFrom(utils,capture.output)

## ----include = FALSE----------------------------------------------------------
knitr::opts_chunk$set(
  collapse = TRUE,
  message = FALSE,
  warning = FALSE,
  fig.align = "center",
  fig.height = 6,
  fig.width = 7,
  fig.path = "fig/",
  dev = "png",
  comment = "#>"
)

# save some typing
knitr::set_alias(w = "fig.width",
                 h = "fig.height",
                 cap = "fig.cap")

set.seed(47)
.opts <- options(digits = 4)

# packages to be cited in packages.bib
.to.cite <- c("nestedLogit", "ggeffects", "ggplot2")

## ----setup--------------------------------------------------------------------
library(nestedLogit)    # Nested Dichotomy Logistic Regression Models
library(ggeffects)      # Create Tidy Data Frames of Marginal Effects
library(ggplot2)        # Data Visualisations Using the Grammar of Graphics

## ----wlf-model----------------------------------------------------------------
data(Womenlf, package = "carData")
comparisons <- logits(work = dichotomy("not.work", c("parttime", "fulltime")),
                      full = dichotomy("parttime", "fulltime"))

wlf.nested <- nestedLogit(partic ~ hincome + children,
                          dichotomies = comparisons,
                          data = Womenlf)

## ----wlf-predict-response-----------------------------------------------------
wlf.pred <- predict_response(wlf.nested, terms = c("hincome", "children"))
wlf.pred

## ----wlf-ggeffects-plot1------------------------------------------------------
plot(wlf.pred)

## ----wlf-ggeffects-plot2------------------------------------------------------
plot(wlf.pred,
     line_size = 2) +
  labs(title = "Predicted Probabilities of Work by Husband's Income",
       y = "Probability",
       x = "Husband's Income") +
  theme_ggeffects(base_size = 14) +
  theme(legend.position = "top")

## ----wlf-logit-scale----------------------------------------------------------
plot(wlf.pred,
     line_size = 2) +
  scale_y_continuous(
    transform = "logit",
    breaks = c(0.05, 0.10, 0.25, 0.50, 0.75, 0.90, 0.95)
  ) +
  labs(title = "Predicted Probabilities (logit scale)",
       y = "Probability (logit scale)",
       x = "Husband's Income") +
  theme_ggeffects(base_size = 16) +
  theme(legend.position = "top")

## ----wlf-dichotomies----------------------------------------------------------
wlf.pred.dichot <- predict_response(wlf.nested, terms = c("hincome", "children"),
                             submodel = "dichotomies")

plot(wlf.pred.dichot) +
  geom_point() +
  theme(legend.position = "inside",
        legend.position.inside = c(.40, .85))

## ----gators-model-------------------------------------------------------------
data(gators)

# setup the dichotomies
gators.dichots = logits(
     other   = dichotomy("Other", c("Fish", "Invertebrates")),
     fish_inv = dichotomy("Fish", "Invertebrates"))

as.tree(gators.dichots)

# fit the model
gators.nested <- nestedLogit(food ~ length,
                             dichotomies = gators.dichots,
                             data = gators)

## ----gators-ggeffects---------------------------------------------------------
predict_response(gators.nested, terms = "length") |>
  plot(line_size = 2)

## ----gators-plot, echo=-1-----------------------------------------------------
par(mar = c(4, 4, 1, 1) + 0.5)
plot(gators.nested, x.var = "length",
     lty=1, lwd = 4,
     label = TRUE, label.col = "black", cex.lab = 1.3)


## ----write-bib, echo=FALSE----------------------------------------------------
pkgs <- unique(c(.to.cite, .packages()))
knitr::write_bib(pkgs, file = here::here("vignettes", "packages.bib"))

## ----include = FALSE----------------------------------------------------------
options(.opts)

## ----include = FALSE----------------------------------------------------------
knitr::opts_chunk$set(
  collapse = TRUE,
  message = FALSE,
  warning = FALSE,
  fig.align = "center",
  fig.height = 6,
  fig.width = 7,
  fig.path = "fig/",
  dev = "png",
  comment = "#>"
)

# save some typing
knitr::set_alias(w = "fig.width",
                 h = "fig.height",
                 cap = "fig.cap")

# colorize text: use inline as `r colorize(text, color)`
colorize <- function(x, color) {
  if (knitr::is_latex_output()) {
    sprintf("\\textcolor{%s}{%s}", color, x)
  } else if (knitr::is_html_output()) {
    sprintf("<span style='color: %s;'>%s</span>", color,
      x)
  } else x
}


set.seed(47)
.opts <- options(digits = 4)

# packages to be cited here. Code at the end automatically updates packages.bib
.to.cite <- c("nnet", "car", "broom", "ggplot2", "geomtextpath", "ggeffects")

# removed: "equatiomatic" -- now in references.bib

## ----setup--------------------------------------------------------------------
library(nestedLogit)    # Nested Dichotomy Logistic Regression Models
library(knitr)          # A General-Purpose Package for Dynamic Report Generation in R
library(car)            # Companion to Applied Regression
library(nnet)           # Feed-Forward Neural Networks and Multinomial Log-Linear Models
library(broom)          # Convert Statistical Objects into Tidy Tibbles
library(dplyr)          # A Grammar of Data Manipulation
library(effects)        # Effect Displays for Linear, Generalized Linear, and Other Models

## -----------------------------------------------------------------------------
knitr::include_graphics("nested.jpg")

## -----------------------------------------------------------------------------
knitr::include_graphics("nested-psychiatric.png")

## ----order-partic-------------------------------------------------------------
data(Womenlf, package = "carData")
Womenlf$partic <- with(Womenlf, 
                       factor(partic, levels = c("not.work", "parttime", "fulltime")))

## ----response-distribution----------------------------------------------------
xtabs(~ partic, data=Womenlf)

## ----recode-------------------------------------------------------------------
Womenlf <- within(Womenlf, {
  work = ifelse(partic == "not.work", 0, 1)
  full = ifelse(partic == "fulltime",  1,
                ifelse(partic == "parttime", 0, NA))
})

## ----dichots-distributions----------------------------------------------------
xtabs(~ work, data=Womenlf)
xtabs(~ full, data=Womenlf, addNA=TRUE)

## ----xtabs--------------------------------------------------------------------
xtabs(~ partic + work, data=Womenlf)
xtabs(~ partic + full, addNA=TRUE, data=Womenlf)

## ----submodels----------------------------------------------------------------
mod.work <- glm(work ~ hincome + children, family=binomial, data=Womenlf)
mod.full <- glm(full ~ hincome + children, family=binomial, data=Womenlf)

## ----direct-estimates---------------------------------------------------------
mod.work
mod.full

## ----comparisons--------------------------------------------------------------
comparisons <- logits(work=dichotomy("not.work", working=c("parttime", "fulltime")),
                      full=dichotomy("parttime", "fulltime"))

comparisons

## ----coerce-comparisons-------------------------------------------------------
as.matrix(comparisons)

cat(as.character(comparisons))

## ----wlf.nested---------------------------------------------------------------
wlf.nested <- nestedLogit(partic ~ hincome + children, 
                          dichotomies = comparisons,
                          data=Womenlf)

## ----names-etc----------------------------------------------------------------
names(wlf.nested)

names(wlf.nested$models) # equivalent: names(models(wlf.models))

# view the separate models
models(wlf.nested, 1) 

models(wlf.nested, 2)

## ----coef---------------------------------------------------------------------
coef(wlf.nested)

# show as odds ratios
exp(coef(wlf.nested))

## ----Anova--------------------------------------------------------------------
Anova(wlf.nested)

## ----linearHyp----------------------------------------------------------------
linearHypothesis(wlf.nested, c("hincome", "childrenpresent"))

## -----------------------------------------------------------------------------
glance(wlf.nested)   # summarize the sub-models

tidy(wlf.nested)     # summarize the coefficients

## -----------------------------------------------------------------------------
gl <- glance(wlf.nested)
gl |> 
  select(response, deviance, df.residual) |> 
  add_row(response = "Combined", deviance = sum(gl$deviance), df.residual = sum(gl$df.residual)) |>
  mutate(
    `P-value` = pchisq(deviance, df.residual, lower.tail = FALSE),
    `$G^2$/df` = deviance / df.residual) |>
  rename(`$G^2$` = deviance,
         df = df.residual) |>
  knitr::kable(digits = 3)

## ----wlf-rsq------------------------------------------------------------------
RSQ(wlf.nested)

## ----update-------------------------------------------------------------------
wlf.nested.1 <- update(wlf.nested, formula = . ~ . + region)

anova(wlf.nested, wlf.nested.1)

## ----more-models--------------------------------------------------------------
wlf.nested.2 <- update(wlf.nested, formula = . ~ .^2)

anova(wlf.nested, wlf.nested.2)

## -----------------------------------------------------------------------------
wlf.pred <- predict(wlf.nested)
print(wlf.pred, n=5)

## -----------------------------------------------------------------------------
new <- expand.grid(hincome=seq(0, 45, length=4),
                   children=c("absent", "present"))

wlf.new <- predict(wlf.nested, new)

## -----------------------------------------------------------------------------
as.data.frame(wlf.new) 

## ----wlf-plot-----------------------------------------------------------------
op <- par(mfcol=c(1, 2), mar=c(4, 4, 3, 1) + 0.1)
col <- scales::hue_pal()(3)                  # ggplot discrete colors
plot(wlf.nested, "hincome",                  # left panel
     other = list(children="absent"),
     xlab = "Husband's Income", 
     legend.location="top", col = col)
plot(wlf.nested, "hincome",                  # right panel
     other = list(children="present"),
     xlab = "Husband's Income", 
     legend=FALSE, col = col)
par(op)

## ----wlf-effect-plot-1--------------------------------------------------------
plot(predictorEffects(wlf.nested))

## ----wlf-effect-plot-2--------------------------------------------------------
plot(predictorEffects(wlf.nested),
     axes=list(y=list(style="stacked")),
     lines=list(col=scales::hue_pal()(3)))

## ----wlf-ggeff----------------------------------------------------------------
# ggpredict(wlf.nested, c("hincome[all]", "children")) |> plot()

## ----alt-model----------------------------------------------------------------
wlf.nested.alt <- nestedLogit(partic ~ hincome + children,
                              logits(full=dichotomy(nonfulltime=c("not.work", "parttime"), "fulltime"),
                                     part=dichotomy("not.work", "parttime")),
                              data=Womenlf)

## ----alt-anova----------------------------------------------------------------
Anova(wlf.nested.alt)

summary(wlf.nested.alt)

## ----fit-correlations---------------------------------------------------------
fit1 <- predict(wlf.nested)$p
fit2 <- predict(wlf.nested.alt)$p
diag(cor(fit1, fit2))
mean(as.matrix(abs(fit1 - fit2)))
max(abs(fit1 - fit2))

## ----compare-logLik-----------------------------------------------------------
logLik(wlf.nested)
logLik(wlf.nested.alt)

## ----compare-AIC--------------------------------------------------------------
AIC(wlf.nested, wlf.nested.alt)

## ----wlf-alt-plot-------------------------------------------------------------
op <- par(mfcol=c(1, 2), mar=c(4, 4, 3, 1) + 0.1)
col <- scales::hue_pal()(3)
plot(wlf.nested.alt, "hincome",           # left panel
     others = list(children="absent"), 
     xlab="Husband's Income", 
     legend.location="top", col = col)
plot(wlf.nested.alt, "hincome",           # right panel
     others = list(children="present"),
     xlab="Husband's Income", 
     legend=FALSE, col = col)
par(op)

## -----------------------------------------------------------------------------
wlf.multinom <- multinom(partic ~ hincome + children, data = Womenlf)
summary(wlf.multinom)
logLik(wlf.multinom)

## ----check-corr---------------------------------------------------------------
fit3 <- predict(wlf.multinom, type="probs")[, c("not.work", "parttime", "fulltime")]
diag(cor(fit2, fit3))
mean(as.matrix(abs(fit2 - fit3)))
max(abs(fit2 - fit3))

## ----write-bib, echo = FALSE--------------------------------------------------
# write a packages.bib file of the packages (.packages()) that have been used here
pkgs <- unique(c(.to.cite, .packages()))
knitr::write_bib(pkgs, file = here::here("vignettes", "packages.bib"))

## ----include = FALSE----------------------------------------------------------
options(.opts)

## ----setup, include=FALSE-----------------------------------------------------
knitr::opts_chunk$set(
  collapse  = TRUE,
  message   = FALSE,
  warning   = FALSE,
  fig.align = "center",
  fig.height = 5,
  fig.width  = 7,
  fig.path   = "fig/",
  dev        = "png",
  comment    = "#>"
)

# packages to be cited in packages.bib
.to.cite <- c("nestedLogit", "nnet", "car", "ggplot2", "AER", "dplyr",
               "mlogit", "vcd", "MASS")

## ----load-packages------------------------------------------------------------
library(nestedLogit)
library(nnet)       # for: multinom()
library(car)        # for: Anova()
library(ggplot2)

## ----chile-data---------------------------------------------------------------
data("Chile", package = "carData")
str(Chile)

## ----chile-drop---------------------------------------------------------------
colSums(is.na(Chile))

# Drop the 168 rows with missing vote so all models use the same cases
chile <- Chile[!is.na(Chile$vote), ]

# Recode vote with longer labels, ordered by median statusquo (No < Abstain < Undec < Yes)
chile$vote <- factor(chile$vote,
                     levels = c("N", "A", "U", "Y"),
                     labels = c("No", "Abstain", "Undec", "Yes"))
nrow(chile)
table(chile$vote)

## ----chile-eda----------------------------------------------------------------
# statusquo distribution by vote
boxplot(statusquo ~ vote, data = chile,
        xlab = "Vote", ylab = "Attitude toward status quo",
        main  = "Chile 1988: status quo attitude by voting intention")

## ----chile-dichots-eng--------------------------------------------------------
dichots_eng <- logits(
  engage    = dichotomy(engaged    = c("Yes", "No"),
                        disengaged = c("Abstain", "Undec")),
  direction = dichotomy("Yes", "No"),
  disengage = dichotomy("Abstain", "Undec")
)

# print method
dichots_eng

# print as an ASCII tree
as.tree(dichots_eng, response = "vote")

## ----chile-dichots-dir--------------------------------------------------------
dichots_dir <- logits(
  direction  = dichotomy(pro  = c("Yes", "Undec"),
                         anti = c("No", "Abstain")),
  engage_pro = dichotomy("Yes", "Undec"),
  engage_ant = dichotomy("No", "Abstain")
)
dichots_dir

## ----chile-fit----------------------------------------------------------------
mod.formula <- vote ~ statusquo + age + sex + education + income

# Multinomial logit baseline (reference = "No")
chile2 <- within(chile, vote <- relevel(vote, ref = "No"))
chile_multi <- multinom(mod.formula, data = chile2, trace = FALSE)

# Nested logit: engagement-first tree
chile_eng <- nestedLogit(mod.formula, dichotomies = dichots_eng, data = chile)

# Nested logit: direction-first tree
chile_dir <- nestedLogit(mod.formula, dichotomies = dichots_dir, data = chile)

## ----chile-summaries, eval=FALSE----------------------------------------------
# summary(chile_eng)
# summary(chile_dir)

## ----chile-anova--------------------------------------------------------------
Anova(chile_eng)
Anova(chile_dir)

## ----chile-aic----------------------------------------------------------------
data.frame(
  model   = c("Multinomial", "Engagement tree", "Direction tree"),
  logLik  = c(logLik(chile_multi), logLik(chile_eng), logLik(chile_dir)),
  AIC     = c(AIC(chile_multi),    AIC(chile_eng),    AIC(chile_dir))
)

## ----chile-rsq-eng------------------------------------------------------------
RSQ(chile_eng, include = c("AIC", "n"))

## ----chile-rsq-dir------------------------------------------------------------
RSQ(chile_dir, include = c("AIC", "n"))

## ----chile-newdata------------------------------------------------------------
new_chile <- data.frame(
  statusquo = seq(-1.5, 1.5, by = 0.25),
  age       = median(chile$age,    na.rm = TRUE),
  sex       = factor("F",  levels = levels(chile$sex)),
  education = factor("S",  levels = levels(chile$education)),
  income    = median(chile$income, na.rm = TRUE)
)

## ----chile-predict------------------------------------------------------------
pred_eng <- as.data.frame(predict(chile_eng, newdata = new_chile))
pred_dir <- as.data.frame(predict(chile_dir, newdata = new_chile))

## ----chile-plot, fig.height=5, fig.width=9------------------------------------
# as.data.frame() returns long format: one row per (observation × response level)
# with predictor columns, plus 'response', 'p', 'se.p', 'logit', 'se.logit'
pred_long <- rbind(
  transform(pred_eng, model = "Engagement-first tree"),
  transform(pred_dir, model = "Direction-first tree")
)

ggplot(pred_long, aes(x = statusquo, y = p, colour = response, fill = response)) +
  geom_ribbon(aes(ymin = pmax(0, p - 1.96 * se.p),
                  ymax = pmin(1, p + 1.96 * se.p)),
              alpha = 0.15, colour = NA) +
  geom_line(linewidth = 1.2) +
  geom_point(color = "blacK") +
  facet_wrap(~ model) +
  labs(x      = "Attitude toward status quo",
       y      = "Predicted probability",
       colour = "Vote", fill = "Vote",
       title  = "Chile 1988: predicted vote probabilities by tree structure") +
  theme_bw()

## ----chile-plot-logit, fig.height=5, fig.width=6------------------------------
# Logit-scale plot using the new scale= argument
plot(chile_eng, "statusquo",
     others = list(age       = median(chile$age, na.rm = TRUE),
                   sex       = "F",
                   education = "S",
                   income    = median(chile$income, na.rm = TRUE)),
     scale  = "logit",
     xlab   = "Attitude toward status quo",
     main   = "Engagement-first tree")

## ----travel-pkg, include=FALSE------------------------------------------------
have_AER <- requireNamespace("AER",   quietly = TRUE)
have_dplyr <- requireNamespace("dplyr", quietly = TRUE)

## ----travel-eval, eval=have_AER && have_dplyr---------------------------------
library(AER)
library(dplyr)
data("TravelMode", package = "AER")
str(TravelMode)

## ----travel-reshape, eval=have_AER && have_dplyr------------------------------
tm <- TravelMode |>
  filter(choice == "yes") |>
  select(mode, income, size) |>
  mutate(mode = relevel(factor(mode), ref = "car"))

table(tm$mode)

## ----travel-dichots, eval=have_AER && have_dplyr------------------------------
travel_dichots <- logits(
  pvt_pub  = dichotomy("car", public = c("air", "train", "bus")),
  air_grnd = dichotomy("air", ground = c("train", "bus")),
  tr_bus   = dichotomy("train", "bus")
)
travel_dichots

## ----travel-fit, eval=have_AER && have_dplyr----------------------------------
tm_multi  <- multinom(mode ~ income + size, data = tm, trace = FALSE)
tm_nested <- nestedLogit(mode ~ income + size,
                         dichotomies = travel_dichots,
                         data = tm)
summary(tm_nested)
Anova(tm_nested)

## ----travel-predict, eval=have_AER && have_dplyr------------------------------
new_tm <- data.frame(income = seq(10, 60, by = 10), size = 1)

# as.data.frame() returns long format; reshape to wide for comparison
pred_tm_nested_long <- as.data.frame(predict(tm_nested, newdata = new_tm))
nested_wide <- reshape(
  pred_tm_nested_long[, c("income", "size", "response", "p")],
  idvar     = c("income", "size"),
  timevar   = "response",
  direction = "wide"
)
names(nested_wide) <- sub("^p\\.", "", names(nested_wide))

pred_tm_multi <- as.data.frame(predict(tm_multi, newdata = new_tm,
                                       type = "probs"))

cat("Nested logit:\n")
cbind(income = new_tm$income, round(nested_wide[, levels(tm$mode)], 3))

cat("Multinomial logit:\n")
cbind(income = new_tm$income, round(pred_tm_multi[, levels(tm$mode)], 3))

## ----travel-aic, eval=have_AER && have_dplyr----------------------------------
data.frame(
  model  = c("Multinomial", "Nested"),
  logLik = c(logLik(tm_multi), logLik(tm_nested)),
  AIC    = c(AIC(tm_multi),    AIC(tm_nested))
)

## ----fishing-pkg, include=FALSE-----------------------------------------------
have_mlogit <- requireNamespace("mlogit", quietly = TRUE)

## ----fishing-load, eval=have_mlogit-------------------------------------------
data("Fishing", package = "mlogit")
str(Fishing)

## ----fishing-dichots, eval=have_mlogit----------------------------------------
fishing_dichots <- logits(
  shore_vessel = dichotomy(shore  = c("beach", "pier"),
                           vessel = c("boat",  "charter")),
  shore_type   = dichotomy("beach", "pier"),
  vessel_type  = dichotomy("boat",  "charter")
)
fishing_dichots
as.tree(fishing_dichots, response = "mode")

## ----fishing-fit, eval=have_mlogit--------------------------------------------
fish_nested <- nestedLogit(mode ~ income,
                           dichotomies = fishing_dichots,
                           data = Fishing)
summary(fish_nested)
Anova(fish_nested)

## ----fishing-plot, eval=have_mlogit-------------------------------------------
plot(fish_nested, x.var = "income",
     xlab  = "Monthly income (USD)",
     main  = "Fishing mode choice vs. income",
     label=TRUE, label.col="black",
     cex.lab = 1.2)

## ----arthritis-pkg, include=FALSE---------------------------------------------
have_vcd <- requireNamespace("vcd", quietly = TRUE)

## ----arthritis-load, eval=have_vcd--------------------------------------------
data("Arthritis", package = "vcd")
str(Arthritis)
table(Arthritis$Improved)

## ----arthritis-dichots, eval=have_vcd-----------------------------------------
arth_dichots <- continuationLogits(c("None", "Some", "Marked"))
arth_dichots

## ----arthritis-fit, eval=have_vcd---------------------------------------------
arth_nested <- nestedLogit(Improved ~ Treatment + Sex + Age,
                           dichotomies = arth_dichots,
                           data = Arthritis)
summary(arth_nested)
Anova(arth_nested)

## ----arthritis-plot, eval=have_vcd--------------------------------------------
plot(arth_nested, x.var = "Age",
     others = list(Treatment = "Treated", Sex = "Female"),
     xlab   = "Age",
     main   = "Arthritis: Treatment = Treated, Sex = Female")

## ----survey-load--------------------------------------------------------------
data("survey", package = "MASS")
# Drop NAs on Smoke
sv <- survey[!is.na(survey$Smoke), ]
table(sv$Smoke)

## ----smoke-dichots------------------------------------------------------------
smoke_dichots <- continuationLogits(c("Never", "Occas", "Regul", "Heavy"))
smoke_dichots

## ----smoke-fit----------------------------------------------------------------
smoke_nested <- nestedLogit(Smoke ~ Sex + Age + Exer,
                            dichotomies = smoke_dichots,
                            data = sv)
summary(smoke_nested)
Anova(smoke_nested)

## ----smoke-plot---------------------------------------------------------------
plot(smoke_nested, x.var = "Age",
     others = list(Sex = "Female", Exer = "Some"),
     xlab   = "Age",
     main   = "Smoking: Female students, some exercise")

## ----pneumo-pkg, include=FALSE------------------------------------------------
have_VGAM <- requireNamespace("VGAM", quietly = TRUE)

## ----pneumo-show, eval=have_VGAM----------------------------------------------
data("pneumo", package = "VGAM")
pneumo

## ----pneumo-fit, eval=have_VGAM-----------------------------------------------
# pneumo is aggregated; nestedLogit() needs individual-level data or
# frequency weights — requires expansion or a weighted interface.
# Placeholder: would use continuationLogits(c("normal", "mild", "severe"))

## ----session-info-------------------------------------------------------------
sessionInfo()

## ----write-bib, echo=FALSE----------------------------------------------------
pkgs <- unique(c(.to.cite, .packages()))
knitr::write_bib(pkgs, file = here::here("vignettes", "packages.bib"))

## ----include = FALSE----------------------------------------------------------
knitr::opts_chunk$set(
  collapse = TRUE,
  message = FALSE,
  warning = FALSE,
  fig.align = "center",
  fig.height = 6,
  fig.width = 7,
  fig.path = "fig/",
  dev = "png",
  comment = "#>"
)

# save some typing
knitr::set_alias(w = "fig.width",
                 h = "fig.height",
                 cap = "fig.cap")

# colorize text
colorize <- function(x, color) {
  if (knitr::is_latex_output()) {
    sprintf("\\textcolor{%s}{%s}", color, x)
  } else if (knitr::is_html_output()) {
    sprintf("<span style='color: %s;'>%s</span>", color,
      x)
  } else x
}


set.seed(47)
.opts <- options(digits = 4)

# packages to be cited here. Code at the end automatically updates packages.bib
#to.cite <- c("ggplot2", "geomtextpath", "equatiomatic")

## ----setup--------------------------------------------------------------------
library(nestedLogit)    # Nested Dichotomy Logistic Regression Models
library(knitr)          # A General-Purpose Package for Dynamic Report Generation in R
library(dplyr)          # A Grammar of Data Manipulation
library(tidyr)          # Tidy Messy Data
library(ggplot2)        # Create Elegant Data Visualisations Using the Grammar of Graphics
library(geomtextpath)   # Curved Text in 'ggplot2'

## ----wlf-model----------------------------------------------------------------
data(Womenlf, package = "carData")
comparisons <- logits(work=dichotomy("not.work", c("parttime", "fulltime")),
                      full=dichotomy("parttime", "fulltime"))

wlf.nested <- nestedLogit(partic ~ hincome + children,
                          dichotomies = comparisons,
                          data=Womenlf)

## ----pred.nested--------------------------------------------------------------
new <- expand.grid(hincome=seq(0, 45, by = 5),
                   children=c("absent", "present"))

pred.nested <- predict(wlf.nested, newdata = new)
names(pred.nested)

## -----------------------------------------------------------------------------
head(pred.nested[["p"]])

## -----------------------------------------------------------------------------
plotdata <- as.data.frame(pred.nested, newdata=new)
head(plotdata)

## ----wlf-ggplot-p1------------------------------------------------------------
theme_set(theme_bw(base_size = 14))

gg1 <- ggplot(plotdata,
       aes(x=hincome, y=p, color=response)) +
  geom_line(linewidth = 2) +
  geom_point(size = 1.5, shape = 16, color = "black") +
  labs(x="Husband's Income", y= "Probability") +
  facet_wrap(~ children, labeller = label_both) +
  geom_ribbon(aes(ymin=p - se.p,
                  ymax=p + se.p,
                  fill = response), alpha = 0.3) 

gg1

## ----wlf-ggplot-p2------------------------------------------------------------
gg1 + geom_textline(aes(label = response), 
                    hjust = -0.01, vjust=-0.5, size=5) +
  theme(legend.position = "none")

## ----wlf-ggplot-logit---------------------------------------------------------
ggplot(plotdata,
       aes(x=hincome, y=logit, color=response)) +
  geom_line(linewidth = 2) +
  geom_point(size = 1.5, shape = 16, color = "black") +
  labs(x="Husband's Income", y= "Log Odds") +
  facet_wrap(~ children, labeller = label_both) +
  geom_ribbon(aes(ymin=logit - se.logit,
                  ymax=logit + se.logit,
                  fill = response), alpha = 0.3) +
  geom_textline(aes(label = response), 
                hjust = -0.01, vjust=-0.5, size=5) +
  theme(legend.position = "none")

## -----------------------------------------------------------------------------
names(models(wlf.nested))

## -----------------------------------------------------------------------------
pred.dichot <- predict(wlf.nested, newdata = new,
                       model = "dichotomies")
str(pred.dichot)

## -----------------------------------------------------------------------------
plotlogit <- as.data.frame(pred.dichot, newdata = new)
head(plotlogit)

## ----wlf-ggplot-dichot1-------------------------------------------------------
ggplot(plotlogit,
       aes(x=hincome, y=logit, color=response)) +
  geom_line(linewidth = 2) +
  geom_point(size = 1.5, shape = 16, color = "black") +
  labs(x="Husband's Income", y= "Log Odds") +
  facet_wrap(~ children, labeller = label_both) +
  geom_ribbon(aes(ymin=logit - se.logit,
                  ymax=logit + se.logit,
                  fill = response), alpha = 0.3) +
  geom_textline(aes(label = response),
                hjust = -0.01, vjust=-0.5, size=5) +
  theme(legend.position = "none")

## ----wlf-ggplot-dichot2-------------------------------------------------------
ggplot(plotlogit,
       aes(x=hincome, y=logit, color=children)) +
  geom_line(linewidth = 2) +
  geom_point(size = 1.5, shape = 16, color = "black") +
  labs(x="Husband's Income", y= "Log Odds") +
  facet_wrap(~ response, labeller = label_both) +
  geom_ribbon(aes(ymin=logit - se.logit,
                  ymax=logit + se.logit,
                  fill = children), alpha = 0.3) +
  geom_textline(aes(label = children),
                hjust = -0.01, vjust=-0.5, size=5) +
  theme(legend.position = "none")

## ----alt-model----------------------------------------------------------------
wlf.nested.alt <- nestedLogit(partic ~ hincome + children,
                              logits(full=dichotomy(nonfulltime=c("not.work", "parttime"), "fulltime"),
                                     part=dichotomy("not.work", "parttime")),
                              data=Womenlf)

## -----------------------------------------------------------------------------
pred.dichot.alt <- predict(wlf.nested.alt, newdata = new,
                       model = "dichotomies")
plotlogit.alt <- as.data.frame(pred.dichot.alt, newdata = new)
head(plotlogit.alt)

## ----wlf-ggplot-alt1----------------------------------------------------------
ggplot(plotlogit.alt,
       aes(x=hincome, y=logit, color=children)) +
  geom_line(linewidth = 2) +
  geom_point(size = 1.5, shape = 16, color = "black") +
  labs(x="Husband's Income", y= "Log Odds") +
  facet_wrap(~ response, labeller = label_both) +
  geom_ribbon(aes(ymin=logit - se.logit,
                  ymax=logit + se.logit,
                  fill = children), alpha = 0.3) +
  geom_textline(aes(label = children),
                hjust = -0.01, vjust=-0.5, size=5) +
  theme(legend.position = "none")

## ----include = FALSE----------------------------------------------------------
options(.opts)

#' Extract Equations for a `"nestedLogit"` Model
#'
#' @description
#' Provides an [equatiomatic::extract_eq()] method for `"nestedLogit"` objects
#' to render the models as LaTeX equations in documents.
#'
#' It extracts the LaTeX equation for each of the binary logit sub-models
#' comprising a nested logit model, substituting the dichotomy name for the
#' internal `..y` placeholder used by [nestedLogit()].
#'
#' @param model a `"nestedLogit"` object.
#' @param submodel character; name of a single dichotomy (e.g. `"work"`).
#'   When supplied, a single `"equation"` object is returned and renders
#'   directly in a knitr chunk.  When omitted, equations for all dichotomies
#'   are returned as a named list of class `"nestedLogit_equations"`.
#' @param \dots additional arguments passed to [equatiomatic::extract_eq()].
#'   For example, use `use_coefs = TRUE` to display fitted coefficients rather
#'   than symbols; `wrap = TRUE` to split the RHS across multiple lines;
#'   `greek_colors`, `var_colors`, etc. to color the symbols.
#'
#' @details
#' Internally, [nestedLogit()] fits each sub-model on a temporary response
#' column named `..y`, so [equatiomatic::extract_eq()] would render the
#' response as `..y` rather than the dichotomy name.  This method replaces
#' `..y` with the name of each dichotomy.
#'
#' Because `_` is the subscript operator in LaTeX, any underscores in
#' dichotomy names are replaced by `.` in the displayed equation
#' (e.g., a dichotomy named `fish_inv` appears as `fish.inv`).
#' This does not affect the names of the returned list, which retain the
#' original R names.
#'
#' @return When `submodel` is `NULL`, an object of class
#'   `c("nestedLogit_equations", "list")` containing one `"equation"` per
#'   dichotomy, named by the dichotomy.  Individual equations can be extracted
#'   with `$name` or by passing `submodel`.  When `submodel` is a dichotomy
#'   name, the single corresponding `"equation"` object is returned and renders
#'   directly in a knitr chunk.
#'
#' @seealso [equatiomatic::extract_eq()]
#' @exportS3Method equatiomatic::extract_eq nestedLogit
#'
#' @examples
#' data("Womenlf", package = "carData")
#' wlf.nested <- nestedLogit(partic ~ hincome + children,
#'                  dichotomies = logits(work = dichotomy("not.work",
#'                                                        working = c("parttime", "fulltime")),
#'                                       full = dichotomy("parttime", "fulltime")),
#'                  data = Womenlf)
#' if (requireNamespace("equatiomatic", quietly = TRUE)) {
#'   equatiomatic::extract_eq(wlf.nested)
#' }
#'
extract_eq.nestedLogit <- function(model, submodel = NULL, ...) {
  mods <- models(model)
  nms  <- names(mods)
  if (!is.null(submodel) && !submodel %in% nms) {
    stop("submodel '", submodel, "' not found; available: ",
         paste(nms, collapse = ", "))
  }
  eqns <- lapply(seq_along(mods), function(i) {
    eq <- equatiomatic::extract_eq(mods[[i]], ...)
    # The sub-models were fit on a data column named `..y`; equatiomatic usually
    # renders that literally.  Replace it with the dichotomy name.
    # Replace underscores with dots: `_` is the subscript operator in LaTeX so
    # `fish_inv` would render as "fish" subscript "inv".  Dots are safe.
    nm_latex <- gsub("_", ".", nms[i], fixed = TRUE)
    # Case 1: equatiomatic used the `..y` data-column name (the normal case).
    eq_str <- gsub("\\\\operatorname\\{\\.\\.y\\}(\\s*=\\s*\\\\operatorname\\{1\\})?",
                   paste0("\\\\operatorname{", nm_latex, "}"),
                   as.character(eq))
    # Case 2: equatiomatic used the formula LHS directly (e.g. `fish_inv`).
    # Use fixed = TRUE so the literal backslash and braces are matched as-is.
    if (grepl("_", nms[i], fixed = TRUE)) {
      eq_str <- gsub(paste0("\\operatorname{", nms[i], "}"),
                     paste0("\\operatorname{", nm_latex, "}"),
                     eq_str, fixed = TRUE)
    }
    class(eq_str) <- class(eq)
    eq_str
  })
  names(eqns) <- nms
  class(eqns) <- c("nestedLogit_equations", "list")
  if (!is.null(submodel)) eqns[[submodel]] else eqns
}

#' @export
print.nestedLogit_equations <- function(x, ...) {
  for (nm in names(x)) {
    cat("\nDichotomy:", nm, "\n")
    print(x[[nm]])
  }
  invisible(x)
}

#' Predicted Probabilities and Logits for \code{"nestedLogit"} Models
#'
#' @name predict.nestedLogit
#' @aliases predict.nestedLogit fitted.nestedLogit print.predictNestedLogit confint.predictNestedLogit print.predictDichotomies
#'
#' @description The \code{predict} and \code{fitted} methods compute predicted values from a fitted
#' \code{"nestedLogit"} model.
#' The \code{confint} method computes point-wise confidence limits for predicted response-category
#' probabilities or logits.
#'
#' \describe{
#'   \item{\code{predict}, \code{fitted}}{Compute predicted response-category probabilities
#'         (or predicted logits for each binary logit model in the dichotomies) from a fitted
#'         \code{"nestedLogit"} model.}
#'   \item{\code{confint}}{Compute point-wise confidence limits for predicted response-category
#'        probabilities or logits.}
#'   \item{\code{print} methods}{Print predicted probabilities, logits, and their standard errors,
#'         with control over how many rows to display.}
#' }
#'
#' @details
#' The \code{predict} method provides predicted values for two representations of the model.
#' \code{model = "nested"} (the default) gives the fitted probabilities for each of the response categories,
#' along with the corresponding logits and standard errors of each.
#' \code{model = "dichotomies"} gives the fitted log odds for each of the binary logit models in the
#' nested dichotomies.
#'
#' The \code{fitted} method (with no \code{newdata}) is equivalent to \code{predict} applied to the
#' original data used to fit the model.
#'
#' For the \code{confint} method with \code{parm = "prob"}, setting
#' \code{conf.limits.logit = TRUE} (the default) computes confidence limits on the logit scale
#' and back-transforms them to probabilities, which ensures that the limits lie in \eqn{[0, 1]}.
#' Setting \code{conf.limits.logit = FALSE} computes Wald-type confidence intervals directly on the
#' probability scale, which may extend outside \eqn{[0, 1]}.
#'
#' @seealso \code{\link{nestedLogit}}, \code{\link{nestedMethods}},
#'          \code{\link{plot.nestedLogit}},
#'          \code{\link{as.data.frame.predictNestedLogit}}
#'
#' @param object a fitted object of class \code{"nestedLogit"};
#'        for \code{confint}, an object of class \code{"predictNestedLogit"}.
#' @param x an object of class \code{"predictNestedLogit"} or \code{"predictDichotomies"}.
#' @param newdata a data frame containing combinations of values of the predictors
#'        at which fitted probabilities (or other quantities) are to be computed.
#'        If missing, the original data are used.
#' @param model either \code{"nested"} (the default), in which case fitted probabilities
#'        under the nested logit model are returned, or \code{"dichotomies"}, in which case
#'        \code{\link{predict.glm}} is invoked for each binary logit model fit to the nested
#'        dichotomies and a named list of the results is returned.
#' @param n an integer or \code{"all"} to control how many rows are printed for each of the
#'        predicted values.
#' @param parm one of \code{"prob"} or \code{"logit"},
#'        indicating whether to generate confidence intervals for probabilities or logits
#'        of the responses.
#' @param level confidence level, a number between 0 and 1; default is \code{0.95}.
#' @param conf.limits.logit logical; when \code{parm = "prob"}, if \code{TRUE} (the default),
#'        confidence limits are computed on the logit scale and back-transformed to probabilities;
#'        if \code{FALSE}, Wald-type limits are computed directly on the probability scale.
#' @param \dots arguments to be passed down.
#'
#' @return  \itemize{
#'    \item The \code{predict} and \code{fitted} methods return an object of class \code{"predictNestedLogit"}
#'    (when \code{model = "nested"}) or \code{"predictDichotomies"} (when \code{model = "dichotomies"}).
#'
#'    A \code{"predictNestedLogit"} object is a list containing:
#'    \describe{
#'      \item{\code{p}}{a data frame of predicted probabilities, with one column per response category.}
#'      \item{\code{logit}}{a data frame of predicted logits.}
#'      \item{\code{se.p}}{a data frame of standard errors of predicted probabilities.}
#'      \item{\code{se.logit}}{a data frame of standard errors of predicted logits.}
#'      \item{\code{.data}}{the \code{newdata} data frame, if supplied.}
#'    }
#'
#'    A \code{"predictDichotomies"} object is a named list of data frames, one per dichotomy,
#'    each produced by \code{\link{predict.glm}}.
#'
#'    \item The \code{confint} method returns a data frame of point estimates and confidence limits.
#'    \item The various \code{print} methods invisibly return their \code{x} arguments.
#'    }
#'
#' @author John Fox and Michael Friendly
#' @keywords regression
#' @examples
#' # define continuation dichotomies for level of education
#' cont.dichots <- continuationLogits(c("<highschool",
#'                                      "highschool",
#'                                      "college",
#'                                      "graduate"))
#'
#' # fit a nested model for the GSS data
#' m <- nestedLogit(degree ~ parentdeg + year,
#'                  cont.dichots,
#'                  data=GSS)
#'
#' # predicted probabilities for first few cases
#' predict(m)
#'
#' # predicted probabilities at specific values of predictors
#' new <- expand.grid(parentdeg=c("<highschool",  "highschool",
#'                                "college", "graduate"),
#'                    year=c(1972, 2016))
#'
#' fit <- predict(m, newdata=new)
#' cbind(new, fit)
#'
#' # use fitted() -- equivalent to predict() on the original data
#' f <- fitted(m)
#'
#' # predicted logits for each dichotomy
#' pred.dichot <- predict(m, newdata=new, model="dichotomies")
#' pred.dichot
#'
#' # confidence intervals for predicted probabilities
#' fit.ci <- confint(fit)
#' head(fit.ci)
#'
#' # confidence intervals for predicted logits
#' fit.ci.logit <- confint(fit, parm="logit")
#' head(fit.ci.logit)

#' @rdname predict.nestedLogit
#' @importFrom stats predict
#' @export
predict.nestedLogit <- function(object, newdata, model=c("nested", "dichotomies"), ...) {
  model <- match.arg(model)

  if (no.newdata <- missing(newdata))
    newdata <- models(object, 1)$data

  if (model == "nested"){

    ndichot <- length(models(object))
    if (ndichot < 2L)
      stop("there are fewer than 2 nested dichotomies")

    var.fitted <- fitted <- vector(ndichot, mode = "list")
    for (j in seq_along(models(object))) {
      pred <- predict(models(object, j), newdata = newdata, type = "response",
                      se.fit=TRUE)
      p <- cbind(1 - pred$fit, pred$fit)
      attr(p, "columns") <- models(object, j)$dichotomy
      fitted[[j]] <- p
      var.fitted[[j]] <- (pred$se.fit)^2
    }

    response.levels <- unique(unlist(lapply(fitted, function(x) attr(x, "columns"))))
    p <- matrix(1, nrow(newdata), length(response.levels))
    v <- matrix(0, nrow(newdata), length(response.levels))
    colnames(v) <- colnames(p) <- response.levels

    # explanation of indices:

    #  k: indexes the m categories of the response
    #  j: indexes the subset of all m - 1 dichotomy models used for
    #     fitted probabilities for a particular category k
    #     of the response
    #  jp: like j, but also excludes current value of j

    for (k in response.levels) {

      for (j in seq_along(models(object))) {

        deriv <- rep(1, nrow(newdata))

        which.j <- sapply(models(object, j)$dichotomy, function(x) k %in% x)
        if (!any(which.j)) next

        for (jp in seq_along(models(object))){
          which.jp <- sapply(models(object, jp)$dichotomy, function(x) k %in% x)
          if (j == jp || !any(which.jp)) next
          deriv <- deriv * fitted[[jp]][, which.jp]
        }

        p[, k] <- p[, k] * fitted[[j]][, which.j]
        v[, k] <- v[, k] + deriv^2 * var.fitted[[j]]
      }
    }

    logit <- log(p/(1 - p))
    v.logit <- (1/(p*(1 - p)))^2 * v
    rownames(v.logit) <- rownames(v) <- rownames(logit) <- rownames(p) <- rownames(newdata)
    result <- list(p = as.data.frame(p), logit = as.data.frame(logit),
                   se.p = as.data.frame(sqrt(v)), se.logit = as.data.frame(sqrt(v.logit)))
    if (!no.newdata) result$.data <- newdata
    class(result) <- "predictNestedLogit"
    return(result)

  } else {
    result <- lapply(models(object),
                     function(x) as.data.frame(predict(x, newdata=newdata, se.fit=TRUE, ...)))
    attr(result, "model") <- deparse(substitute(object))
    attr(result, "dichotomies") <- names(result)
    if (!no.newdata) result$.data <- newdata
    class(result) <- "predictDichotomies"
    result
  }
}

#' @rdname predict.nestedLogit
#' @export
print.predictNestedLogit <- function(x, n=min(10L, nrow(x$p)), ...){
  if (n == "all") n <- nrow(x$p)
  if (truncate <- nrow(x$p) > n) cat(paste0("\nFirst ", n, " of ", nrow(x$p), " rows:\n"))
  cat("\npredicted response-category probabilties\n")
  print(x$p[1:n, ], ...)
  if (truncate) cat("  . . .\n")
  cat("\npredicted response-category logits\n")
  print(x$logit[1:n, ], ...)
  if (truncate) cat("  . . .\n")
  cat("\nstandard errors of predicted probabilities\n")
  print(x$se.p[1:n, ], ...)
  if (truncate) cat("  . . .\n")
  cat("\nstandard errors of predicted logits\n")
  print(x$se.logit[1:n, ], ...)
  if (truncate) cat("  . . .\n")
  invisible(x)
}

#' @rdname predict.nestedLogit
#' @importFrom stats confint qnorm
#' @export
confint.predictNestedLogit <- function (object, parm=c("prob", "logit"),
                                        level=0.95, conf.limits.logit=TRUE, ...) {
  parm <- match.arg(parm)
  if (parm == "logit"){
    logit <- object$"logit"
    se <- object$"se.logit"
    z <- qnorm(1 - (1 - level)/2)
    lower <- logit - z*se
    upper <- logit + z*se
    result <- cbind(logit, lower, upper)
    cnames.1 <- colnames(logit)
    cnames.2 <- c("logit", round((1 - level)/2, 4),
                  round(1 - (1 - level)/2, 4))
    cnames <- paste0(cnames.1, ".", rep(cnames.2, each=ncol(logit)))
    colnames(result) <- cnames
    return(as.data.frame(result))
  } else {
    if (conf.limits.logit){
      p <- object$"p"
      logit <- object$"logit"
      se <- object$"se.logit"
      z <- qnorm(1 - (1 - level)/2)
      lower <- 1/(1 + exp(-(logit - z*se)))
      upper <- 1/(1 + exp(-(logit + z*se)))
    } else {
      p <- object$"p"
      se <- object$"se.p"
      z <- qnorm(1 - (1 - level)/2)
      lower <- p - z*se
      upper <- p + z*se
    }
    result <- cbind(p, lower, upper)
    cnames.1 <- colnames(p)
    cnames.2 <- c("p", round((1 - level)/2, 4),
                  round(1 - (1 - level)/2, 4))
    cnames <- paste0(cnames.1, ".", rep(cnames.2, each=ncol(p)))
    colnames(result) <- cnames
    return(as.data.frame(result))
  }
}

#' @rdname predict.nestedLogit
#' @export
print.predictDichotomies <- function(x, n=10L, ...){
  cat("\n predictions for binary logit models from nested logit model:",
      attr(x, "model"), "\n")
  nms <- names(x)
  for (i in seq_along(x)){
    if (n == "all") n <- nrow(x[[i]])
    cat("\n dichotomy:", nms[i], "\n")
    print(x[[i]][1:min(n, nrow(x[[i]])), ])
  }
  invisible(x)
}


#' @rdname predict.nestedLogit
#' @importFrom stats fitted
#' @export
fitted.nestedLogit <- function(object, model=c("nested", "dichotomies"), ...){
  predict(object, model=model)
}

#' Extract Binary Logit Models from a \code{nestedLogit} Object
#'
#' @name models
#' @aliases models models.nestedLogit
#'
#' @description
#' Nested logit models represent an overall models for a polytomous response (>2 categories)
#' by a set of binary logit models corresponding to nested dichotomies among the response
#' categories.
#' \code{models} is used to extract \code{"glm"} objects representing binary logit
#' models from a \code{"nestedLogit"} object.
#'
#' @param model a \code{"nestedLogit"} model.
#' @param select a numeric or character vector giving the number(s) or names(s)
#'        of one or more
#'        binary logit models to be extracted from \code{model}; if absent, a list of
#'        all of the binary logits models in \code{model} is returned.
#' @param as.list if \code{TRUE} (the default is \code{FALSE}) and one binary logit
#'        model is selected, return the \code{"glm"} object in a one-element named list;
#'        otherwise a single model is returned directly as a \code{"glm"} object;
#'        when more than one binary
#'        logit model is selected, the corresponding \code{"glm"} objects are \emph{always}
#'        returned as a named list.
#'
#' @return \code{model} returns either a single \code{"glm"} object (see \code{\link{glm}}) or a
#'   list of \code{"glm"} objects, each representing a binary logit model.
#'
#' @examples
#'   data("Womenlf", package = "carData")
#'   comparisons <- logits(work=dichotomy("not.work",
#'                                        working=c("parttime", "fulltime")),
#'                         full=dichotomy("parttime", "fulltime"))
#'   m <- nestedLogit(partic ~ hincome + children,
#'                    dichotomies = comparisons,
#'                    data=Womenlf)
#'
#'   # extract both submodels, as a list
#'   models(m, c("work", "full"))
#'
#'   # extract the binomial logit model for working vs. non-working
#'   m_work <- models(m, "work")
#'
#'   # use that to plot residuals
#'   plot(density(residuals(m_work)))
#'
#'   # or plot that model -- gives the 'regression quartet' for a glm()
#'   op <- par(mfrow = c(2,2))
#'   plot(m_work)
#'   par(op)

#' @export
#'
models <- function(model, select, as.list=FALSE){
  UseMethod("models")
}

#' @rdname models
#' @export
models.nestedLogit <- function(model, select, as.list=FALSE){
  if (missing(select)) return(model$models)
  if (is.numeric(select)){
    model.nos <- seq(along=model$models)
    which <- !(select %in% model.nos)
  } else {
    model.names <- names(model$models)
    which <- !(select %in% model.names)
  }
  if (any(which)){
    stop("the following model", if (sum(which) > 1) "s are " else " is ",
         "not available:\n", paste(select[which], collapse=", "))
  }
  result <- model$models[select]
  if (length(result) > 1 || as.list) return(result) else return(result[[1]])
}

#' @name GSS
#' @docType data
#'
#' @title Data From the U.S. General Social Survey 1972-2016
#'
#' @description This data set is drawn from the U.S. General Social Survey (GSS) for years
#' between 1972 and 2016.
#'
#' @usage data("GSS", package = "nestedLogit")
#'
#' @format A data frame with 44091 rows and 3 columns.
#' \describe{
#'   \item{parentdeg}{A factor representing parents' attained level of education
#'   (highest "degree" obtained), recording
#'   the higher of mother's and father's education, with levels \code{"<highschool"},
#'   \code{"highschool"}, \code{"college"}, and \code{"graduate"}.}
#'   \item{degree}{The respondent's level of education, a factor with the same levels
#'   as \code{parentdeg}.}
#'   \item{year}{The year of the survey, between \code{1972} and
#'   \code{2016}.}
#'   }
#'
#' @source General Social Survey, NORC, The University of Chicago
#' https://www.norc.org/Research/Projects/Pages/general-social-survey.aspx.
#'
#' @seealso \code{\link{nestedLogit}}.
#'
#' @examples
#' round(100*with(GSS, prop.table(table(degree, parentdeg), 2)))
#' m.GSS <- nestedLogit(degree ~ parentdeg*year,
#'                      continuationLogits(c("<highschool",  "highschool",
#'                                            "college", "graduate")),
#'                      data=GSS)
#' car::Anova(m.GSS)
#' summary(m.GSS)
#'
#' # plot fitted probabilities
#' plot(m.GSS, x.var = "year",
#'             others = list(parentdeg = "<highschool"),
#'             lty = 1,
#'             label = TRUE)
#' plot(m.GSS, x.var = "year",
#'             others = list(parentdeg = "graduate"),
#'             lty = 1,
#'             label = TRUE)
"GSS"

#' Broom Related Methods
#'
#' These functions give compact summaries of a \code{"nestedLogit"} object
#' \describe{
#'   \item{\code{glance}}{Construct a single row summaries for the dichotomies \code{"nestedLogit"} model.}
#'   \item{\code{tidy}}{Summarizes the terms in \code{"nestedLogit"} model.}
#' }
#'
#' @name broomMethods
#' @aliases glance.nestedLogit tidy.nestedLogit
#' @param x     an object of class \code{"nestedLogit"}.
#' @param \dots arguments to be passed down.
#' @importFrom broom glance
#' @rdname broomMethods
#' @seealso \code{\link{nestedMethods}}, \code{\link[broom]{glance}}, \code{\link[broom]{tidy}}
#' @exportS3Method broom::glance nestedLogit
#' @return \itemize{
#'    \item \code{glance} returns a \code{tibble} containing one row of fit statistics for each dichotomy,
#'          labeled \code{response}. See \code{\link[broom]{glance}} for details.
#'    \item \code{tidy} returns a \code{tibble} containing coefficient estimates and test statistics for
#'          the combinations of \code{response} and \code{term}. See \code{\link[broom]{tidy}} for details.
#' }
#' @examples
#' data("Womenlf", package = "carData")
#' m <-  nestedLogit(partic ~ hincome + children,
#'                   dichotomies = logits(work=dichotomy("not.work",
#'                                                       working=c("parttime", "fulltime")),
#'                                        full=dichotomy("parttime", "fulltime")),
#'                   data=Womenlf)
#'
#' # one-line summaries
#' broom::glance(m)
#' # coefficients and tests
#' broom::tidy(m)
#'
glance.nestedLogit <- function(x, ...){
  result <- dplyr::bind_rows(lapply(models(x), broom::glance))
  result <- dplyr::bind_cols(response = names(models(x)), result)
  result
}

#' @importFrom broom tidy
#' @rdname broomMethods
#' @exportS3Method broom::tidy nestedLogit
tidy.nestedLogit <- function(x, ...){
  result <- dplyr::bind_rows(lapply(models(x), broom::tidy, ...))
  response <- rep(names(models(x)), each = nrow(result)/length(models(x)))
  result <- dplyr::bind_cols(response = response, result)
  result
}

# DONE: changed default colors to use scales::hue_pal()
# DONE: Allow to plot results on the logit scale, using logit = log(p/1-p), as in the
#       example "Plotting log-odds" in `vignettes/plotting-ggplot.Rmd`
#       Added argument `scale = c("prob", "logit")` where "prob" gives the current behavior
# TODO: Allow to facet the plots with a by= variable, rather than producing separate plots by conditioning as in the womenlf example.
#       This should produce a single plot with separate panels, using `par(mfrow=())`
# DONE: Added `label.col` arg (defaulting to the value of `col` in the call), to control the color of the curve label

#' Plotting Nested Logit Models
#'
#' @description A \code{\link{plot}} method for \code{"nestedLogit"} objects produced by the
#' \code{\link{nestedLogit}} function. Fitted probabilities under the model,
#' or the corresponding logits are plotted
#' for each level of the polytomous response variable, with one of the explanatory variables
#' on the horizontal axis and other explanatory variables fixed to particular values.
#' By default, a 95% pointwise confidence envelope is added to the plot.
#'
#' @aliases plot.nestedLogit
#' @seealso \code{\link{nestedLogit}}, \code{\link[graphics]{matplot}}
#' @param x an object of \code{"nestedLogit"} produced by \code{\link{nestedLogit}}.
#' @param x.var quoted name of the variable to appear on the x-axis; if omitted, the first
#'        predictor in the model is used.
#' @param others a named list of values for the other variables in the model,
#'        that is, other than \code{x.var}; if any other predictor is omitted, it is set
#'        to an arbitrary value---the mean for a numeric predictor or the first level or
#'        value of a factor, character, or logical predictor; only one value may be
#'        specified for each variable in \code{others}.
#' @param n.x.values the number of evenly spaced values of \code{x.var} at which
#'        to evaluate fitted probabilities to be plotted (default \code{100}).
#' @param scale character string; \code{"prob"} (the default) plots fitted probabilities
#'        on the y-axis; \code{"logit"} plots fitted log odds (logits), i.e., \eqn{\log(p/(1-p))}.
#' @param xlab label for the x-axis (defaults to the value of \code{x.var}).
#' @param ylab label for the y-axis (defaults to \code{"Fitted Probability"} when
#'        \code{scale = "prob"} and \code{"Fitted Log Odds"} when \code{scale = "logit"}).
#' @param main main title for the graph (if missing, constructed from the variables and
#'        values in \code{others}).
#' @param cex.main size of main title (see \code{\link{par}}).
#' @param digits.main number of digits to retain when rounding values for the main title.
#' @param font.main font for main title (see \code{\link{par}}).
#' @param pch plotting characters (see \code{\link{par}}).
#' @param lwd line width (see \code{\link{par}}).
#' @param lty line types (see \code{\link{par}}).
#' @param col line colors for the response levels (see \code{\link{par}}).
#' @param legend if \code{TRUE} (the default), add a legend for the
#'        response levels to the graph. Ignored when \code{label = TRUE}.
#' @param legend.inset default \code{0.01} (see \code{\link{legend}}).
#' @param legend.location position of the legend (default \code{"topleft"},
#'        see \code{\link{legend}}).
#' @param legend.bty  the type of box to be drawn around the legend. The allowed values are "o" (the default) and "n".
#' @param conf.level the level for pointwise confidence envelopes around the predicted values;
#' the default is \code{0.95}. If \code{NULL}, the confidence envelopes are suppressed.
#' @param conf.alpha the opacity of the confidence envelopes; the default is \code{0.3}.
#' @param label if \code{TRUE}, label the curves directly instead of using a legend.
#'        Default is \code{FALSE}.
#' @param label.x where to place the label for each curve. Either a single string,
#'        \code{"min"} or \code{"max"} (applied to all curves), or a character vector of
#'        length equal to the number of response categories with each element being
#'        \code{"min"} or \code{"max"}, e.g. \code{label.x = c("min", "max", "max")}.
#'        \code{"min"} places the label at the left end of the curve;
#'        \code{"max"} (the default) places it at the right end.
#' @param label.cex character expansion factor for direct labels; default \code{1.25}.
#' @param label.col colors for direct labels; defaults to \code{col}, so labels match
#'        their curves. Supply a vector of length equal to the number of response
#'        categories to use different colors for the labels.
#' @param \dots arguments to be passed to \code{\link{matplot}}.
#' @author John Fox, Michael Friendly
#' @examples
#' data("Womenlf", package = "carData")
#' m <- nestedLogit(partic ~ hincome + children,
#'                  logits(work=dichotomy("not.work", c("parttime", "fulltime")),
#'                         full=dichotomy("parttime", "fulltime")),
#'                         data=Womenlf)
#' plot(m, legend.location="top")
#'
#' op <- par(mfcol=c(1, 2), mar=c(4, 4, 3, 1) + 0.1)
#' plot(m, "hincome", list(children="absent"),
#'      xlab="Husband's Income", legend=FALSE)
#' plot(m, "hincome", list(children="present"),
#'      xlab="Husband's Income")
#' par(op)
#'
#' # Plot on the logit (log-odds) scale
#' plot(m, "hincome", list(children="absent"), scale = "logit",
#'      xlab = "Husband's Income")
#'
#' # Gators example: direct curve labels instead of a legend
#' data("gators", package = "nestedLogit")
#' gators.nested <- nestedLogit(food ~ length,
#'   logits(d1 = dichotomy("Other", c("Fish", "Invertebrates")),
#'          d2 = dichotomy("Fish", "Invertebrates")),
#'   data = gators)
#'
#' # All labels at the right end (default)
#' plot(gators.nested, x.var = "length", label = TRUE,
#'      xlab = "Alligator length (m)")
#'
#' # Mixed placement: Other and Invertebrates labeled at left, Fish at right
#' # (food levels are: "Other", "Fish", "Invertebrates")
#' plot(gators.nested, x.var = "length", label = TRUE,
#'      label.x = c("min", "max", "min"),
#'      xlab = "Alligator length (m)")
#'
#' @importFrom grDevices palette adjustcolor
#' @importFrom graphics axis box matplot title arrows polygon text
#' @importFrom stats formula
#' @importFrom scales hue_pal
#' @rdname plot.nestedLogit
#' @return NULL Used for its side-effect of producing a plot
#' @export
plot.nestedLogit <- function(x,
                             x.var, others,
                             n.x.values=100L,
                             scale=c("prob", "logit"),
                             xlab=x.var, ylab=NULL,
                             main, cex.main=1,
                             digits.main=getOption("digits") - 2L,
                             font.main=1L,
                             pch=1L:length(response.levels),
                             lwd=3,
                             lty=1L:length(response.levels),
#                             col=palette()[1L:length(response.levels)],
                             col=scales::hue_pal()(length(response.levels)),
                             legend=TRUE, legend.inset=0.01,
                             legend.location="topleft",
                             legend.bty = "n", conf.level=0.95,
                             conf.alpha=0.25,
                             label=FALSE,
                             label.x="max", label.cex=1.25,
                             label.col=col,
                             ...){
  scale <- match.arg(scale)
  if (is.null(ylab)) ylab <- if (scale == "prob") "Fitted Probability" else "Fitted Log Odds"
  data <- as.data.frame(x$data)
  vars <- all.vars(formula(x)[-2L])
  response <- setdiff(all.vars(formula(x)), vars)
  if (missing(x.var)) {
    x.var <- vars[1L]
    message("Note: ", x.var, " will be used for the horizontal axis")
  }
  if (!(x.var %in% vars)) stop (x.var, " is not in the model")
  if (!missing(others)){
    if (any(sapply(others, length) > 1L))
      stop("more than one value specified for one or more variables in others")
    other.x <- names(others)
    check <- other.x %in% vars
    if (any(!check)) stop("not in the model: ", paste(other.x[!check], collapse=", "))
    values <- others
  } else {
    values <- list()
  }
  other.xs <- setdiff(vars, x.var)
  missing.xs <- !(other.xs %in% names(values))
  if (any(missing.xs)){
    missing.xs <- other.xs[missing.xs]
    for (missing.x in missing.xs){
      xx <- data[, missing.x]
      if (is.numeric(xx)){
        values[[missing.x]] <- signif(mean(xx, na.rm=TRUE))
        message("Note: missing predictor ", missing.x, " set to its mean, ",
                values[[missing.x]])
      } else if (is.factor(xx)){
        values[[missing.x]] <- levels(xx)[1L]
        message("Note: missing predictor ", missing.x, " set to its first level, '",
                values[[missing.x]], "'")
      } else {
        values[[missing.x]] <-  (sort(unique(xx)))[1L]
        message("Note: missing predictor ", missing.x, " set to its first value, '",
                values[[missing.x]], "'")
      }
    }
  }
  if (is.numeric(data[, x.var])){
    numeric.x <- TRUE
    range.x <- range(data[, x.var], na.rm=TRUE)
    values[[x.var]] <- seq(range.x[1L], range.x[2L], length=n.x.values)
  } else if (is.factor(data[, x.var])){
    numeric.x <- FALSE
    values[[x.var]] <- levels(data[, x.var])
  } else {
    numeric.x <- FALSE
    values[[x.var]] <- sort(unique(data[, x.var]))
  }
  new <- do.call(expand.grid, values)
  predictions <- predict(x, newdata=new)
  response.levels <- levels(data[[response]])
  lower.upper <- paste0(".", c(round((1 - conf.level)/2, 4L),
                                round(1 - (1 - conf.level)/2, 4L)))
  # suffix for point-estimate columns returned by confint()
  pt_suffix <- if (scale == "logit") ".logit" else ".p"

  if (!is.null(conf.level)){
    ci <- confint(predictions, level=conf.level,
                  parm=if (scale == "logit") "logit" else "prob")
    new <- cbind(new, ci[, paste0(response.levels, pt_suffix)],
                 ci[, paste0(response.levels, lower.upper[1L])],
                 ci[, paste0(response.levels, lower.upper[2L])])
    lower_mat <- as.matrix(new[, paste0(response.levels, lower.upper[1L])])
    upper_mat <- as.matrix(new[, paste0(response.levels, lower.upper[2L])])
    ymin <- min(lower_mat[is.finite(lower_mat)])
    ymax <- max(upper_mat[is.finite(upper_mat)])
  } else {
    new <- cbind(new, if (scale == "logit") predictions$logit else predictions$p)
  }

  # column names for point estimates (depends on scale and whether CIs are computed)
  p_cols <- if (!is.null(conf.level)) paste0(response.levels, pt_suffix) else response.levels

  # validate label.x
  if (label) {
    if (length(label.x) == 1L) label.x <- rep(label.x, length(response.levels))
    if (length(label.x) != length(response.levels))
      stop("label.x must be length 1 or the same length as the number of response categories")
    if (!all(label.x %in% c("min", "max")))
      stop('each element of label.x must be "min" or "max"')
  }

  if (numeric.x){
    matplot(new[, x.var],
            new[, p_cols],
            type="l", lwd=lwd, lty=lty,
            col=col, xlab=xlab, ylab=ylab,
            ylim=if (!is.null(conf.level)) c(ymin, ymax) else NULL, ...)
    if (!is.null(conf.level)){
      for (i in seq_along(response.levels)){
        level <- response.levels[i]
        lower_y <- new[, paste0(level, lower.upper[1L])]
        upper_y <- new[, paste0(level, lower.upper[2L])]
        lower_y[!is.finite(lower_y)] <- ymin
        upper_y[!is.finite(upper_y)] <- ymax
        polygon(c(new[, x.var], rev(new[, x.var])),
                c(lower_y, rev(upper_y)),
                col=adjustcolor(col[i], alpha.f=conf.alpha), border=NA)
      }
    }
    if (label) {
      for (i in seq_along(response.levels)) {
        if (label.x[i] == "max") {
          idx <- nrow(new)
          adj <- c(1, -0.1)    # right edge of text at curve end, slightly above
        } else {               # "min"
          idx <- 1L
          adj <- c(0, -0.1)    # left edge of text at curve start, slightly above
        }
        text(new[idx, x.var], new[idx, p_cols[i]],
             labels = response.levels[i],
             adj = adj, col = label.col[i], cex = label.cex)
      }
    } else if (legend) {
      legend(legend.location, legend=response.levels,
             lty=lty, lwd=lwd,
             col=col, inset=legend.inset,
             bty = legend.bty,
             xpd=TRUE)
    }
  } else {
    n.x.levels <- nrow(new)
    matplot(1L:n.x.levels,
            new[, p_cols],
            type="b", lwd=lwd, lty=lty,
            pch=pch, col=col, xlab=xlab, ylab=ylab, axes=FALSE,
            ylim= if (!is.null(conf.level)) c(ymin, ymax) else NULL,
            ...)
    box()
    axis(2L)
    levels <- new[, x.var]
    axis(1L, at=1L:n.x.levels, labels=new[, x.var])
    if (!is.null(conf.level)){
      for (i in 1L:n.x.levels){
        for (j in seq_along(response.levels)){
          level <- response.levels[j]
          arrows(x0=i, y0=new[i, paste0(level, lower.upper[1L])],
                 y1=new[i, paste0(level, lower.upper[2L])],
                 angle=90, col=j, lwd=3, code=3L, length=0.1)
        }
      }
    }
    if (label) {
      for (i in seq_along(response.levels)) {
        if (label.x[i] == "max") {
          idx <- n.x.levels
          adj <- c(1, -0.1)    # right edge of text at curve end, slightly above
        } else {               # "min"
          idx <- 1L
          adj <- c(0, -0.1)    # left edge of text at curve start, slightly above
        }
        text(idx, new[idx, p_cols[i]],
             labels = response.levels[i],
             adj = adj, col = label.col[i], cex = label.cex)
      }
    } else if (legend) {
      legend(legend.location, legend=response.levels,
             lty=lty, lwd=lwd,
             col=col, pch=pch,
             inset=legend.inset,
             bty = legend.bty,
             xpd=TRUE)
    }
  }
  if (!missing(main)) {
    title(main, cex.main=cex.main, font.main=font.main)
  } else {
    if (length(values) != 0L){
      which.main <- !(x.var == names(values))
      if (any(which.main)){
        values <- lapply(values, function(x) if (is.numeric(x)) signif(x, digits.main) else x)
        main <- paste(paste(names(values[which.main]), "=",
                            as.character(unlist(values[which.main]))),
                      collapse=", ")
        title(main, cex.main=cex.main, font.main=font.main)
      }
    }
  }
  return(invisible(NULL))
}

#' Display the Tree Structure of Nested Dichotomies
#'
#' @description Display the nested structure of a \code{"dichotomies"} or
#' \code{"continuationDichotomies"} object as a 2-D ASCII tree diagram showing
#' how the response categories are split at each level of the nesting.
#'
#' @name as.tree
#' @aliases as.tree as.tree.dichotomies as.tree.continuationDichotomies
#'
#' @param x A \code{"dichotomies"} or \code{"continuationDichotomies"} object.
#' @param response Optional character string giving the name of the response
#'   variable, used as the root label of the tree. If \code{NULL} (default),
#'   the root is labeled \code{"(response)"}.
#' @param lobstr Logical. If \code{FALSE} (default), renders a 2-D ASCII tree
#'   with \code{/} and \code{\\} branch connectors, with each node centered
#'   above its two children. If \code{TRUE}, builds a nested list
#'   representation of the tree and renders it via \code{\link[lobstr]{tree}},
#'   which must be installed.
#' @param \dots additional arguments (currently unused).
#'
#' @details
#' The flat list of dichotomies is reconstructed into a binary tree by matching
#' each dichotomy's domain (the union of its two sides) to the multi-level
#' groups produced by earlier splits.  Branch labels are taken from the named
#' arguments to \code{\link{dichotomy}} when present, and are otherwise
#' generated automatically as \code{\{level1, level2, ...\}}.
#'
#' @importFrom stats setNames
#' @return Invisibly returns \code{x}; called for its side effect of printing.
#'
#' @seealso \code{\link{logits}}, \code{\link{continuationLogits}},
#'   \code{\link{print.dichotomies}}
#'   Other conversions:
#'   \code{\link{as.matrix.dichotomies}}, \code{\link{as.character.dichotomies}}
#'
#' @examples
#' ## Womenlf: named group on one branch
#' comparisons <- logits(work = dichotomy("not.work",
#'                                        working = c("parttime", "fulltime")),
#'                       full = dichotomy("parttime", "fulltime"))
#' as.tree(comparisons, response = "partic")
#'
#' ## GSS: continuation logits for ordered education levels
#' cont <- continuationLogits(c("<highschool", "highschool",
#'                              "college", "graduate"))
#' as.tree(cont, response = "degree")
#'
#' ## gators data: Food choice
#' # create dichotomies
#' gators.dichots <- logits(d1=dichotomy("Other", c("Fish", "Invertebrates")),
#'                          d2=dichotomy("Fish", "Invertebrates"))
#' as.tree(gators.dichots, response = "Food")
#' as.tree(gators.dichots, response = "Food", lobstr = TRUE)
#
#'
#' @export
#' @importFrom stats setNames
as.tree <- function(x, ...) UseMethod("as.tree")

#' @rdname as.tree
#' @export
as.tree.dichotomies <- function(x, response = NULL, lobstr = FALSE, ...) {
  tree       <- .build_dichot_tree(x)
  root_label <- if (!is.null(response)) response else "(response)"

  if (isTRUE(lobstr)) {
    if (!requireNamespace("lobstr", quietly = TRUE))
      stop("Package 'lobstr' is required when lobstr = TRUE. ",
           "Install it with: install.packages('lobstr')")
    nested <- setNames(list(.dichot_tree_to_list(tree)), root_label)
    lobstr::tree(nested)
  } else {
    layout <- .layout_tree(tree, root_label)
    lines  <- trimws(layout$canvas, which = "right")
    cat(paste(lines, collapse = "\n"), "\n")
  }
  invisible(x)
}

#' @rdname as.tree
#' @exportS3Method as.tree continuationDichotomies
as.tree.continuationDichotomies <- function(x, response = NULL,
                                            lobstr = FALSE, ...) {
  NextMethod()
}


# ── internal helpers ──────────────────────────────────────────────────────────

# Reconstruct a binary tree from a flat dichotomies object.
# Each node is a named list with:
#   type        "node" | "leaf"
#   label       (leaf only) character label
#   name        (node only) name of the dichotomy at this split
#   left        (node only) left child
#   left_label  (node only) explicit group label or NULL
#   right       (node only) right child
#   right_label (node only) explicit group label or NULL
.build_dichot_tree <- function(x) {
  all_levels <- attr(x, "levels")

  find_split <- function(levels) {
    for (i in seq_along(x))
      if (setequal(union(x[[i]][[1L]], x[[i]][[2L]]), levels)) return(i)
    NULL
  }

  make_node <- function(levels) {
    if (length(levels) == 1L)
      return(list(type = "leaf", label = levels))

    idx <- find_split(levels)
    if (is.null(idx))
      return(list(type = "leaf", label = paste(levels, collapse = "/")))

    d       <- x[[idx]]
    grp_nms <- names(d)

    list(
      type        = "node",
      name        = names(x)[idx],
      left        = make_node(d[[1L]]),
      left_label  = if (!is.null(grp_nms) && nzchar(grp_nms[1L])) grp_nms[1L] else NULL,
      right       = make_node(d[[2L]]),
      right_label = if (!is.null(grp_nms) && nzchar(grp_nms[2L])) grp_nms[2L] else NULL
    )
  }

  make_node(all_levels)
}

# Collect all leaf labels in a subtree (depth-first, left to right).
.collect_leaf_labels <- function(node) {
  if (node$type == "leaf") return(node$label)
  c(.collect_leaf_labels(node$left), .collect_leaf_labels(node$right))
}

# Auto-generate a branch label when no explicit group name is available.
.auto_label <- function(node) {
  if (node$type == "leaf") return(node$label)
  paste0("{", paste(.collect_leaf_labels(node), collapse = ", "), "}")
}

# Compute a 2-D horizontal layout for a subtree.
#
# Arguments:
#   node  — a tree node from .build_dichot_tree()
#   label — the text label to display at the root of this subtree
#
# Returns a list:
#   canvas   — character vector; all strings have equal width (= $width)
#   root_col — 0-based column of the center of `label` in the canvas
#   width    — total character width of the canvas
#
# Algorithm:
#   Leaves produce a 1-line canvas equal to `label`.
#   Internal nodes lay the two child subtrees side by side, separated by
#   enough space so that:
#     (a) the subtrees do not overlap,
#     (b) the / and \ arms each have room to start,
#     (c) the parent label (centered between the two child centers) fits.
#   The / connector is placed one column right of the left child's center;
#   the \ connector one column left of the right child's center.
.layout_tree <- function(node, label) {
  label_w <- nchar(label)

  if (node$type == "leaf") {
    return(list(
      canvas   = label,
      root_col = label_w %/% 2L,
      width    = label_w
    ))
  }

  left_lbl  <- if (!is.null(node$left_label))  node$left_label  else .auto_label(node$left)
  right_lbl <- if (!is.null(node$right_label)) node$right_label else .auto_label(node$right)

  L <- .layout_tree(node$left,  left_lbl)
  R <- .layout_tree(node$right, right_lbl)

  lc <- L$root_col   # center of left child (0-based, within left canvas)

  # right_start: 0-based column where the right subtree's canvas begins.
  # Constraints:
  #   (1) L$width + 1       — at least 1 space gap between canvases
  #   (2) lc + 3 - R$root_col — so that rc >= lc+3 (room for / space \)
  #   (3) label_w - lc - R$root_col — so that parent label starts at col >= 0
  right_start <- max(
    L$width + 1L,
    lc + 3L - R$root_col,
    label_w - lc - R$root_col
  )

  rc <- right_start + R$root_col          # center of right child (merged)
  pc <- (lc + rc) %/% 2L                  # parent center (merged)
  ls <- pc - label_w %/% 2L               # 0-based start column of label
  le <- ls + label_w                       # 0-based end column (exclusive)

  total_width <- max(right_start + R$width, le)

  # ---- label line -----------------------------------------------------------
  lbl_line <- paste0(strrep(" ", ls),
                     label,
                     strrep(" ", total_width - le))

  # ---- arm line (/ and \) ---------------------------------------------------
  arm_line <- strrep(" ", total_width)
  # "/" at 0-based col lc+1  →  1-based substr position lc+2
  # "\" at 0-based col rc-1  →  1-based substr position rc
  substr(arm_line, lc + 2L, lc + 2L) <- "/"
  substr(arm_line, rc,      rc)        <- "\\"

  # ---- merge child canvases -------------------------------------------------
  depth_L <- length(L$canvas)
  depth_R <- length(R$canvas)
  depth   <- max(depth_L, depth_R)

  L_lines <- c(L$canvas, rep(strrep(" ", L$width), depth - depth_L))
  R_lines <- c(R$canvas, rep(strrep(" ", R$width), depth - depth_R))

  child_lines <- vapply(seq_len(depth), function(i) {
    l_pad <- paste0(L_lines[i], strrep(" ", max(0L, right_start - nchar(L_lines[i]))))
    line  <- paste0(l_pad, R_lines[i])
    paste0(line, strrep(" ", max(0L, total_width - nchar(line))))
  }, character(1L))

  list(
    canvas   = c(lbl_line, arm_line, child_lines),
    root_col = pc,
    width    = total_width
  )
}

# Convert the tree to a nested named list for lobstr::tree().
.dichot_tree_to_list <- function(node) {
  if (node$type == "leaf") return(node$label)

  left_lbl  <- if (!is.null(node$left_label))  node$left_label  else .auto_label(node$left)
  right_lbl <- if (!is.null(node$right_label)) node$right_label else .auto_label(node$right)

  setNames(
    list(.dichot_tree_to_list(node$left),
         .dichot_tree_to_list(node$right)),
    c(left_lbl, right_lbl)
  )
}

#' Convert a Predicted Objects to a data.frame
#'
#' These functions provide simple ways to convert the results of \code{\link{predict.nestedLogit}}
#' to a data frame in a consistent format for plotting and other actions.
#'
#' @param x         a \code{"predictNestedLogit"} object
#' @param row.names row.names for result (for conformity with generic; not currently used)
#' @param optional  logical. If TRUE, setting row names and converting column names
#'        (to syntactic names: see \code{\link[base]{make.names}} is optional
#' @param ...       other arguments (unused)
#'
#' @return \itemize{
#'  \item For \code{predict(\dots, model="nested")} (the default), returns
#'  a data frame containing the values of predictors along with the columns
#'         \code{response}, \code{p}, \code{se.p}, \code{logit}, \code{se.logit}.
#'  \item For \code{predict(\dots, model="dichotomies")}, returns
#'  a data frame containing the values of predictors along with the columns
#'         \code{response}, \code{logit}, and \code{se.logit}.
#'    }
#' @export
#'
#' @examples
#' data("Womenlf", package = "carData")
#' comparisons <- logits(work=dichotomy("not.work", c("parttime", "fulltime")),
#'                      full=dichotomy("parttime", "fulltime"))
#'
#' wlf.nested <- nestedLogit(partic ~ hincome + children,
#'                           dichotomies = comparisons,
#'                           data=Womenlf)
#' # get predicted values for a grid of `hincome` and `children`
#' new <- expand.grid(hincome=seq(0, 45, length=10),
#'                    children=c("absent", "present"))
#'
#' pred.nested <- predict(wlf.nested, new)
#' plotdata <- as.data.frame(pred.nested)
#' str(plotdata)
#'
#' # Predicted logit values for the dichotomies
#' pred.dichot <- predict(wlf.nested, new, model = "dichotomies")
#' plotlogit <- as.data.frame(pred.dichot)
#' str(plotlogit)

as.data.frame.predictNestedLogit <- function(x,
                                             row.names = NULL,
                                             optional = FALSE,
                                             ...){
  resp.names <- colnames(x$p)

  result <- data.frame(
    response = rep(resp.names, nrow(x$p)),
    p        = as.vector(t(x$p)),
    se.p     = as.vector(t(x$se.p)),
    logit    = as.vector(t(x$logit)),
    se.logit = as.vector(t(x$se.logit))
  )
  .data <- x$.data
  if (!is.null(.data)) {
    idx <- rep(seq_len(nrow(.data)), each = length(resp.names))
    result <- cbind(.data[idx, , drop = FALSE], result)
  }

  rownames(result) <- NULL
  result
}

#' @importFrom dplyr select bind_cols
#' @importFrom tibble rownames_to_column
#' @importFrom stringr str_remove
#' @export

as.data.frame.predictDichotomies <- function(x,
                                             row.names = NULL,
                                             optional = FALSE,
                                             ...){
  # quiet no visible binding
  response <- fit <- se.fit <- residual.scale <- NULL
  dichotomies <- attr(x, "dichotomies")
  .data <- x$.data

  result <- do.call(rbind, x[dichotomies]) |>
    dplyr::select(- residual.scale) |>
    tibble::rownames_to_column(var = "response") |>
    dplyr::mutate(response =  stringr::str_remove(response, ".\\d+")) |>
    dplyr::rename(logit = fit,
           se.logit = se.fit)

  if(!is.null(.data)) {
    nlogits <- length(x[dichotomies])
    idx <- rep(seq_len(nrow(.data)), nlogits)
    result <- dplyr::bind_cols(.data[idx,], result)
  }
  result
}


# Example of steps
# plotlogit <- do.call(rbind, pred.dichot) |>
#   select(- residual.scale) |>
#   tibble::rownames_to_column(var = "response") |>
#   mutate(response =  stringr::str_remove(response, ".\\d+")) |>
#   rename(logit = fit,
#          se.logit = se.fit)
# idx <- rep(seq_len(nrow(new)), length(pred.dichot))
# plotlogit <- bind_cols(new[idx,], plotlogit)

#' Pseudo-R² Measures for Nested Logit and Related Models
#'
#' @description
#' Computes pseudo-R² and related fit measures for a `"nestedLogit"` object
#' and related models for a polytomous response. For the `"nestedLogit"` case,
#' the result shows
#' one row per binary logit sub-model (dichotomy) and an additional
#' `"Combined"` row for the overall polytomous model.
#'
#' @details
#' `RSQ` is implemented as an S3 generic with methods for `"nestedLogit"`, as well as
#' `nnet::multinom()`, and `MASS::polr()` objects, which are other methods for
#' modeling a polytomous response variable.
#'
#' In contrast to standard, Gaussian linear models, where \eqn{R^2} has a uniformly simple interpretation as
#' "variance accounted for" by the model, and with different, yet _equivalent_ computational formulas,
#' there is no single commonly accepted measure for logistic regression models for a binary response or
#' a dichotomy among outcomes.
#'
#' The following measures are available via the `which` argument:
#' \describe{
#'   \item{`"McFadden"`}{1 - L/L\eqn{_0}, where L is the fitted model
#'     log-likelihood and L\eqn{_0} that of the null (intercept-only) model
#'     (McFadden, 1979).  Values of 0.1--0.3 indicate a reasonable fit in
#'     logistic regression.}
#'   \item{`"McFaddenAdj"`}{1 - (L - k)/L\eqn{_0}, where k is the number
#'     of non-intercept parameters; penalises model complexity (Hosmer &
#'     Lemeshow, 2000).}
#'   \item{`"CoxSnell"`}{1 - exp(2(L\eqn{_0} - L)/n); bounded strictly
#'     below 1 for discrete outcomes (Cox & Snell, 1989).}
#'   \item{`"Nagelkerke"`}{Cox-Snell divided by its theoretical maximum,
#'     rescaling to \[0, \1] (Nagelkerke, 1991).}
#'   \item{`"Tjur"`}{Mean fitted value for \eqn{y = 1} minus mean fitted
#'     value for \eqn{y = 0}; the coefficient of discrimination (Tjur, 2009).
#'     Per-dichotomy only (`NA` in the Combined row).}
#' }
#'
#' For the **Combined** row the log-likelihood is the sum of the sub-model
#' log-likelihoods (exploiting the independence of the nested dichotomies), and
#' \eqn{n} is `nrow(x$data)` --- the full sample size of the polytomous model ---
#' not the sum of per-dichotomy observation counts, which would double-count
#' observations that appear in more than one sub-model.
#'
#' A wider range of pseudo-R² measures for logistic-type models (`glm`,
#' `polr`, `multinom`, `vglm`) is available in
#' [DescTools::PseudoR2()], including the Efron (1978) and
#' McKelvey & Zavoina (1975) measures not implemented here.
#' For an accessible overview see \url{https://statisticalhorizons.com/r2logistic/}.
#'
#' @param x      a `"nestedLogit"` object.
#' @param which  character vector naming the pseudo-R² measures to compute.
#'   Any subset of `c("McFadden", "McFaddenAdj", "CoxSnell", "Nagelkerke", "Tjur")`,
#'   or `"ALL"` to include all of them.
#'   Default: `c("McFadden", "CoxSnell", "Nagelkerke")`.
#' @param include character vector of additional columns to append to the result.
#'   Any subset of `c("AIC", "BIC", "n")`, where `"n"` adds the
#'   number of observations used for each row, or `"ALL"` to include all of them.
#'   Default: `"AIC"`.
#' @param digits  integer; number of decimal places used when printing
#'   (default `3L`).
#' @param ...   currently unused.
#'
#' @return An object of class `c("RSQ.nestedLogit", "data.frame")` with one
#'   row per dichotomy plus a final `"Combined"` row, and columns
#'   `response` (the sub-model name), the requested pseudo-R² measures,
#'   and any additional statistics requested via `include`.
#'   The `formula`, object name, and `digits` are stored as attributes
#'   and used by the `print` method.
#'
#' @references
#' Cox, D. R., & Snell, E. J. (1989). *The Analysis of Binary Data* (2nd ed.).
#' Chapman and Hall.
#'
#' Efron, B. (1978). Regression and ANOVA with zero-one data: Measures of
#' residual variation. *Journal of the American Statistical Association*,
#' *73*(361), 113--121. \doi{10.2307/2286498}
#'
#' Hosmer, D. W., & Lemeshow, S. (2000). *Applied Logistic Regression*
#' (2nd ed.). Wiley. \doi{10.1002/0471722146}
#'
#' McFadden, D. (1979). Quantitative methods for analysing travel behaviour of
#' individuals: Some recent developments. In D. A. Hensher & P. R. Stopher
#' (Eds.), *Behavioural Travel Modelling* (pp. 279--318). Croom Helm.
#'
#' McKelvey, R. D., & Zavoina, W. (1975). A statistical model for the analysis
#' of ordinal level dependent variables. *Journal of Mathematical Sociology*,
#' *4*(1), 103--120. \doi{10.1080/0022250X.1975.9989847}
#'
#' Nagelkerke, N. J. D. (1991). A note on a general definition of the
#' coefficient of determination. *Biometrika*, *78*(3), 691--692.
#' \doi{10.1093/biomet/78.3.691}
#'
#' Tjur, T. (2009). Coefficients of determination in logistic regression
#' models --- a new proposal: The coefficient of discrimination.
#' *The American Statistician*, *63*(4), 366--372.
#' \doi{10.1198/tast.2009.08210}
#'
#' @seealso [nestedLogit()], [broom::glance()],
#'   [DescTools::PseudoR2()],
#'   [nnet::multinom()], [MASS::polr()]
#' @author Michael Friendly
#' @examples
#' data("Womenlf", package = "carData")
#' wlf.nested <- nestedLogit(partic ~ hincome + children,
#'   logits(work = dichotomy("not.work", c("parttime", "fulltime")),
#'          full = dichotomy("parttime", "fulltime")),
#'   data = Womenlf)
#'
#' # Default: McFadden, CoxSnell, Nagelkerke + AIC
#' RSQ(wlf.nested)
#'
#' # All measures and all extra columns
#' RSQ(wlf.nested, which = "ALL", include = "ALL")
#'
#' # Multinomial logit for comparison
#' if (requireNamespace("nnet", quietly = TRUE)) {
#'   wlf.multi <- nnet::multinom(partic ~ hincome + children, data = Womenlf,
#'                               trace = FALSE)
#'   RSQ(wlf.multi)
#' }
#'
#' # Proportional-odds model for comparison
#' if (requireNamespace("MASS", quietly = TRUE)) {
#'   wlf.polr <- MASS::polr(partic ~ hincome + children, data = Womenlf)
#'   RSQ(wlf.polr)
#' }
#'
#' @importFrom utils capture.output
#' @importFrom stats AIC BIC formula fitted model.frame model.response
#' @export
RSQ <- function(x, ...) UseMethod("RSQ")

#' @rdname RSQ
#' @export
RSQ.nestedLogit <- function(x,
                             which   = c("McFadden", "CoxSnell", "Nagelkerke"),
                             include = "AIC",
                             digits  = 3L,
                             ...) {
  obj_name    <- deparse(substitute(x))
  all_which   <- c("McFadden", "McFaddenAdj", "CoxSnell", "Nagelkerke", "Tjur")
  all_include <- c("AIC", "BIC", "n")
  if (identical(which,   "ALL")) which   <- all_which
  if (identical(include, "ALL")) include <- all_include
  which <- match.arg(which,   choices = all_which,   several.ok = TRUE)
  include <- match.arg(include, choices = all_include, several.ok = TRUE)

  # glance.nestedLogit() gives one row per dichotomy with all ingredients:
  #   null.deviance, df.null, logLik, AIC, BIC, df.residual, nobs, ...
  gl        <- broom::glance(x)
  mod_names <- gl$response

  # --- Per-dichotomy rows ---
  rows <- vector("list", nrow(gl))
  for (i in seq_along(mod_names)) {
    L  <- gl$logLik[i]
    L0 <- -gl$null.deviance[i] / 2
    n  <- gl$nobs[i]
    k  <- gl$df.null[i] - gl$df.residual[i]   # non-intercept parameters
    m  <- models(x, mod_names[i])
    rows[[i]] <- .pseudo_r2(L, L0, n, k, which,
                             fitted_vals = fitted(m),
                             y           = m$y)
  }
  sub_df <- dplyr::bind_rows(rows)

  # --- Combined row ---
  # L_combined = sum of sub-model log-likelihoods (independence of dichotomies)
  # n_combined = full sample size (not sum of per-dichotomy nobs)
  L_combined  <- sum(gl$logLik)
  L0_combined <- sum(-gl$null.deviance / 2)
  n_combined  <- nrow(x$data)
  k_combined  <- sum(gl$df.null - gl$df.residual)

  combined_row <- .pseudo_r2(L_combined, L0_combined, n_combined, k_combined,
                              setdiff(which, "Tjur"))
  if ("Tjur" %in% which) combined_row$Tjur <- NA_real_
  # Restore column order to match sub_df
  combined_row <- combined_row[, names(sub_df), drop = FALSE]

  # --- Assemble ---
  result <- dplyr::bind_rows(sub_df, combined_row)
  result <- dplyr::bind_cols(
    data.frame(response = c(mod_names, "Combined"), stringsAsFactors = FALSE),
    result
  )

  # --- Optional extra columns ---
  if ("AIC" %in% include) result$AIC <- c(gl$AIC,  AIC(x))
  if ("BIC" %in% include) result$BIC <- c(gl$BIC,  BIC(x))
  if ("n"   %in% include) result$n   <- c(gl$nobs, n_combined)

  structure(result,
            class      = c("RSQ.nestedLogit", "data.frame"),
            formula    = formula(x),
            model.name = obj_name,
            which      = which,
            digits     = digits)
}

# Internal helper: compute pseudo-R² values from log-likelihood components.
# Not exported.
.pseudo_r2 <- function(L, L0, n, k, which, fitted_vals = NULL, y = NULL) {
  cs     <- 1 - exp(2 * (L0 - L) / n)
  cs_max <- 1 - exp(2 * L0 / n)
  result <- list()
  if ("McFadden"    %in% which) result$McFadden    <- 1 - L / L0
  if ("McFaddenAdj" %in% which) result$McFaddenAdj <- 1 - (L - k) / L0
  if ("CoxSnell"    %in% which) result$CoxSnell    <- cs
  if ("Nagelkerke"  %in% which) result$Nagelkerke  <- cs / cs_max
  if ("Tjur"        %in% which) {
    result$Tjur <- if (!is.null(fitted_vals) && !is.null(y))
      mean(fitted_vals[y == 1L]) - mean(fitted_vals[y == 0L])
    else NA_real_
  }
  as.data.frame(result)
}

#' @rdname RSQ
#' @export
print.RSQ.nestedLogit <- function(x, digits = attr(x, "digits"), ...) {
  cat(sprintf("Pseudo R\u00b2 measures for nestedLogit model %s:\n",
              attr(x, "model.name")))
  cat(" ", paste(deparse(attr(x, "formula")), collapse = " "), "\n\n")

  n_dichot <- nrow(x) - 1L

  # Round numeric columns before formatting
  out      <- as.data.frame(x)
  num_cols <- setdiff(names(out), "response")
  out[, num_cols] <- lapply(out[, num_cols, drop = FALSE], round, digits)

  # Capture formatted lines so we can insert a separator before Combined
  txt <- utils::capture.output(print(out, row.names = FALSE))
  # txt[1]              = column header line
  # txt[2:(1+n_dichot)] = one line per dichotomy
  # txt[2+n_dichot]     = Combined row
  cat(txt[1L], "\n")
  for (i in seq_len(n_dichot)) cat(txt[1L + i], "\n")
  cat(strrep("-", nchar(txt[1L])), "\n")
  cat(txt[2L + n_dichot], "\n")

  invisible(x)
}

#' @rdname RSQ
#' @importFrom stats logLik
#' @export
RSQ.multinom <- function(x,
                          which   = c("McFadden", "CoxSnell", "Nagelkerke"),
                          include = "AIC",
                          digits  = 3L,
                          ...) {
  obj_name    <- deparse(substitute(x))
  all_which   <- c("McFadden", "McFaddenAdj", "CoxSnell", "Nagelkerke", "Tjur")
  all_include <- c("AIC", "BIC", "n")
  if (identical(which,   "ALL")) which   <- all_which
  if (identical(include, "ALL")) include <- all_include
  which   <- match.arg(which,   choices = all_which,   several.ok = TRUE)
  include <- match.arg(include, choices = all_include, several.ok = TRUE)

  # --- Ingredients ---
  L  <- as.numeric(logLik(x))
  n  <- nrow(model.frame(x))

  # Null log-likelihood: intercept-only model predicts marginal proportions
  y   <- model.response(model.frame(x))
  n_j <- as.integer(table(y))
  n_j <- n_j[n_j > 0L]          # drop empty levels
  L0  <- sum(n_j * log(n_j / n))

  m <- length(n_j)               # number of response categories
  # Non-intercept parameters: total df minus one intercept per non-ref category
  k <- attr(logLik(x), "df") - (m - 1L)

  # Tjur is defined only for binary responses
  result <- .pseudo_r2(L, L0, n, k, setdiff(which, "Tjur"))
  if ("Tjur" %in% which) result$Tjur <- NA_real_
  result <- result[, intersect(all_which, names(result)), drop = FALSE]

  # Response variable name as the row label
  resp_name <- deparse(formula(x)[[2L]])
  result <- dplyr::bind_cols(
    data.frame(response = resp_name, stringsAsFactors = FALSE),
    result
  )

  # --- Optional extra columns ---
  if ("AIC" %in% include) result$AIC <- AIC(x)
  if ("BIC" %in% include) result$BIC <- BIC(x)
  if ("n"   %in% include) result$n   <- n

  structure(result,
            class      = c("RSQ.multinom", "data.frame"),
            formula    = formula(x),
            model.name = obj_name,
            which      = which,
            digits     = digits)
}


#' @rdname RSQ
#' @export
print.RSQ.multinom <- function(x, digits = attr(x, "digits"), ...) {
  cat(sprintf("Pseudo R\u00b2 measures for multinom model %s:\n",
              attr(x, "model.name")))
  cat(" ", paste(deparse(attr(x, "formula")), collapse = " "), "\n\n")

  out      <- as.data.frame(x)
  num_cols <- setdiff(names(out), "response")
  out[, num_cols] <- lapply(out[, num_cols, drop = FALSE], round, digits)
  print(out, row.names = FALSE)

  invisible(x)
}

#' @importFrom stats logLik
#' @rdname RSQ
#' @export
RSQ.polr <- function(x,
                     which   = c("McFadden", "CoxSnell", "Nagelkerke"),
                     include = "AIC",
                     digits  = 3L,
                     ...) {
  obj_name    <- deparse(substitute(x))
  all_which   <- c("McFadden", "McFaddenAdj", "CoxSnell", "Nagelkerke", "Tjur")
  all_include <- c("AIC", "BIC", "n")
  if (identical(which,   "ALL")) which   <- all_which
  if (identical(include, "ALL")) include <- all_include
  which   <- match.arg(which,   choices = all_which,   several.ok = TRUE)
  include <- match.arg(include, choices = all_include, several.ok = TRUE)

  L   <- as.numeric(logLik(x))
  n   <- nrow(model.frame(x))

  # Null log-likelihood: intercept-only model predicts marginal proportions
  y   <- model.response(model.frame(x))
  n_j <- as.integer(table(y))
  n_j <- n_j[n_j > 0L]
  L0  <- sum(n_j * log(n_j / n))

  # Non-intercept parameters: slopes only (thresholds/zeta are not predictors)
  k   <- length(x$coefficients)

  # Tjur is defined only for binary responses
  result <- .pseudo_r2(L, L0, n, k, setdiff(which, "Tjur"))
  if ("Tjur" %in% which) result$Tjur <- NA_real_
  result <- result[, intersect(all_which, names(result)), drop = FALSE]

  resp_name <- deparse(formula(x)[[2L]])
  result <- dplyr::bind_cols(
    data.frame(response = resp_name, stringsAsFactors = FALSE),
    result
  )

  if ("AIC" %in% include) result$AIC <- AIC(x)
  if ("BIC" %in% include) result$BIC <- BIC(x)
  if ("n"   %in% include) result$n   <- n

  structure(result,
            class      = c("RSQ.polr", "data.frame"),
            formula    = formula(x),
            model.name = obj_name,
            which      = which,
            digits     = digits)
}

#' @rdname RSQ
#' @export
print.RSQ.polr <- function(x, digits = attr(x, "digits"), ...) {
  cat(sprintf("Pseudo R\u00b2 measures for polr model %s:\n",
              attr(x, "model.name")))
  cat(" ", paste(deparse(attr(x, "formula")), collapse = " "), "\n\n")

  out      <- as.data.frame(x)
  num_cols <- setdiff(names(out), "response")
  out[, num_cols] <- lapply(out[, num_cols, drop = FALSE], round, digits)
  print(out, row.names = FALSE)

  invisible(x)
}

#' Methods for \code{"nestedLogit"} and Related Objects
#'
#' @name nestedMethods
#' @aliases nestedMethods print.nestedLogit summary.nestedLogit print.summary.nestedLogit update.nestedLogit coef.nestedLogit vcov.nestedLogit print.dichotomies as.dichotomies.matrix as.matrix.continuationDichotomies as.character.dichotomies as.matrix.dichotomies as.dichotomies
#'
#' @description Various methods for processing \code{"nestedLogit"} and related objects.
#' Most of these are the standard methods for a model-fitting function.
#' \describe{
#'   \item{\code{coef}, \code{vcov}}{Return the coefficients and their variance-covariance matrix respectively.}
#'   \item{\code{update}}{Re-fit a \code{"nestedLogit"} model with a change in any of the \code{formula}, \code{dichotomies},
#'        \code{data}, \code{subset}, or \code{contrasts}, arguments.}
#'   \item{\code{summary}}{Summarize a \code{"nestedLogit"} model, giving the summary for each binary logit model in the
#'        nested dichotomies.}
#'   \item{\code{print}}{Print the model or a summary of the model.}
#'   \item{\code{as.matrix}, \code{as.character}, \code{as.dichotomies}}{Coerce
#'    dichotomy-related objects to matrices, character vectors, and dichotomies objects.}
#' }
#'
#' @seealso \code{\link{nestedLogit}}, \code{\link{predict.nestedLogit}},
#'          \code{\link{plot.nestedLogit}},
#'          \code{\link{glance.nestedLogit}}, \code{\link{tidy.nestedLogit}}
#'
#' @param x,object in most cases, an object of class \code{"nestedLogit"}.
#' @param as.matrix if \code{TRUE} (the default for \code{coef}) return coefficients
#'        as a matrix with one column for each nested dichotomy,
#'        or coefficient covariances as a matrix with one row and column for each
#'        combination of dichotomies and coefficients; if \code{FALSE} (the default for
#'        \code{vcov}), return a list of coefficients or coefficient covariances
#'        with one element for each dichotomy.
#' @param formula optional updated model formula.
#' @param dichotomies optional updated dichotomies object.
#' @param data optional updated data argument
#' @param subset optional updated subset argument.
#' @param contrasts optional updated contrasts argument.
#' @param \dots arguments to be passed down.
#'
#' @return  \itemize{
#'    \item The \code{coef} and \code{vcov} methods return either matrices or lists of regression
#'    coefficients and their covariances, respectively.
#'    \item The \code{update} method returns an object of class \code{"nestedLogit"} (see \code{\link{nestedLogit}})
#'    derived from the original nested-logit model.
#'    \item The \code{summary} method returns an object of class \code{"summary.nestedLogit"}, which is
#'    a list of summaries of the \code{\link{glm}} objects that comprise the nested-dichotomies model; the
#'    object is normally printed.
#'    \item The methods for \code{as.matrix}, \code{as.character}, and \code{as.dichotomies} coerce
#'    various objects to matrices, character vectors, and dichotomies objects.
#'    \item The various \code{print} methods invisibly return their \code{x} arguments.
#'    }
#'
#' @author John Fox and Michael Friendly
#' @keywords regression
#' @examples
#' # define continuation dichotomies for level of education
#' cont.dichots <- continuationLogits(c("<highschool",
#'                                      "highschool",
#'                                      "college",
#'                                      "graduate"))
#'
#' # Show dichotomies in various forms
#' print(cont.dichots)
#' as.matrix(cont.dichots)
#' as.character(cont.dichots)
#'
#' # fit a nested model for the GSS data examining education degree in relation to parent & year
#' m <- nestedLogit(degree ~ parentdeg + year,
#'                  cont.dichots,
#'                  data=GSS)
#'
#' coef(m)                             # coefficient estimates
#' sqrt(diag(vcov(m, as.matrix=TRUE))) # standard errors
#' print(m)
#' summary(m)
#'
#' @rdname nestedMethods
#' @export
print.nestedLogit <- function(x, ...) {
  cat("Nested logit models: ")
  print(x$formula)
  if (!is.null(x$subset)) cat("subset: ", x$subset, "\n")
  if ("NULL" != x$contrasts.print) cat("contrasts: ", x$contrasts.print, "\n")
  lapply(models(x), print, ...)
  invisible(x)
}

#' @rdname nestedMethods
#' @export
summary.nestedLogit <- function(object, ...) {
  result <- lapply(models(object), summary, ...)
  for (i in seq_along(result)) {
    result[[i]]$dichotomy <- models(object, i)$dichotomy
  }
  class(result) <- "summary.nestedLogit"
  attr(result, "formula") <- object$formula
  attr(result, "subset") <- object$subset
  attr(result, "contrasts.print") <- object$contrasts.print
  result
}

#' @rdname nestedMethods
#' @exportS3Method print summary.nestedLogit
print.summary.nestedLogit <- function(x, ...) {
  cat("Nested logit models: ")
  print(attr(x, "formula"))
  if (!is.null(subset <- attr(x, "subset"))){
    cat("subset: ", subset, "\n")
  }
  if ("NULL" != (contrasts <- attr(x, "contrasts.print"))){
    cat("contrasts: ", contrasts, "\n")
  }
  cat("\n")
  nms <- names(x)
  for (i in seq(along = x)) {
    cat(composeResponse(nms[i], x[[i]]$dichotomy))
    print(x[[i]], ...)
  }
  invisible(x)
}

#' @rdname nestedMethods
#' @export
print.dichotomies <- function(x, ...) {
  nms <- names(x)
  for (i in seq_along(x)) {
    cat(paste0(
      nms[i],
      ": ",
      names(x[[i]][1L]),
      "{",
      paste(x[[i]][[1L]], collapse = ", "),
      "} vs. ",
      names(x[[i]][2L]),
      "{",
      paste(x[[i]][[2L]], collapse = ", "),
      "}\n"
    ))
  }
  invisible(x)
}


#' @rdname nestedMethods
#' @importFrom stats coef
#' @export
coef.nestedLogit <- function(object, as.matrix=TRUE, ...) {
  result <- if(as.matrix) sapply(models(object), coef, ...)
  else lapply(models(object), coef, ...)
  result
}

#' @rdname nestedMethods
#' @export
vcov.nestedLogit <- function(object, as.matrix=FALSE, ...){
  vcovs <- lapply(models(object), vcov)
  if (!as.matrix) {
    return(vcovs)
  }
  n.models <- length(models(object))
  n.coefs <- nrow(vcovs[[1]])
  nms <- expand.grid(names(coef(models(object, 1L))), names(models(object)))
  nms <- paste0(nms[, 2], ".", nms[, 1])
  vcov <- matrix(0, n.models*n.coefs, n.models*n.coefs)
  rownames(vcov) <- colnames(vcov) <- nms
  for (i in 1:n.models){
    start <- (i - 1)*n.coefs + 1
    vcov[start:(start + n.coefs - 1), start:(start + n.coefs - 1)] <- vcovs[[i]]
  }
  vcov
}

#' @rdname nestedMethods
#' @export
update.nestedLogit <- function(object, formula, dichotomies, data, subset, contrasts,...){
  formula <- if (missing(formula)) {
    object$formula
  } else {
    update(object$formula, formula)
  }
  if (missing(dichotomies)) dichotomies <- object$dichotomies
  if (missing(data)) {
    data.name <- object$data.name
    data <- object$data
  } else {
    data.name <- deparse(substitute(data))
  }
  if (missing(subset)) {
    subset <- object$subset
  }
  if (missing(contrasts)){
    contrasts <- object$contrasts
    if (!is.null(contrasts) && contrasts == "NULL") contrasts <- NULL
  }
  result <- nestedLogit(formula, dichotomies=dichotomies, data=data,
                        subset=subset, contrasts=contrasts)
  result$data.name <- data.name
  for (i in seq_along(models(result))){
   result$models[[i]]$call$data <- as.symbol(data.name)
  }
  if (missing(contrasts)){
    result$contrasts <- object$contrasts
    result$contrasts.print <- object$contrasts.print
  } else {
    result$contrasts <- contrasts
    result$contrasts.print <- deparse(substitute(contrasts))
  }
  result
}

#' @rdname nestedMethods
#' @exportS3Method as.matrix dichotomies
as.matrix.dichotomies <- function(x, ...) {
  levels <- unique(unlist(x))
  responses <- matrix(NA, length(x), length(levels))
  for (i in seq_along(x)) {
    responses[i, levels %in% x[[i]][[1L]]] <- 0L
    responses[i, levels %in% x[[i]][[2L]]] <- 1L
  }
  rownames(responses) <- names(x)
  colnames(responses) <- levels
  responses
}

#' @rdname nestedMethods
#' @exportS3Method as.character dichotomies
as.character.dichotomies <- function(x, ...) {
  nms <- names(x)
  result <- ""
  for (i in seq_along(x)) {
    result <- paste0(result,
                     names(x[i]), " = ",
                     "{",
                     names(x[[i]][1L]),
                     "{",
                     paste(x[[i]][[1L]], collapse = " "),
                     "}} {",
                     names(x[[i]][2L]),
                     "{",
                     paste(x[[i]][[2L]], collapse = " "),
                     "}}; "
    )
  }
  result <- substr(result, 1, nchar(result)-2L)
  result
}


#' @rdname nestedMethods
#' @exportS3Method as.matrix continuationDichotomies
as.matrix.continuationDichotomies <- function(x, ...){
  result <- NextMethod()
  result[, attr(x, "levels")]
}


#'
#' @rdname nestedMethods
#' @export
as.dichotomies <- function(x, ...){
  UseMethod("as.dichotomies")
}
#'
#' @rdname nestedMethods
#' @exportS3Method as.dichotomies matrix
as.dichotomies.matrix <- function(x, ...) {
  nlevels <- ncol(x)
  nlogits <- nrow(x)
  if(is.null(rownames(x))) rownames(x) <- paste0("d_", 1:nlogits)
  if(is.null(colnames(x))) colnames(x) <- LETTERS[1:nlevels]
  levels <- colnames(x)
  dnames <- rownames(x)

  logits <- vector("list", nlogits)
  for(i in 1:nlogits) {
    if (!all(x[i,] %in% c(0, 1, NA)))
      stop("Row ", i, " does not define a dichotomy consisting of 0, 1, NA")
    ones  <- levels[which(x[i,] == 1)]
    zeros <- levels[which(x[i,] == 0)]
    logits[[i]] <- list(zeros, ones)
  }
  names(logits) <- dnames
  class(logits) <- "dichotomies"
  attr(logits, "levels") <- levels

  logits
}


# the following utility function isn't exported:

composeResponse <- function(name, dichotomy){
  paste0(
    "Response ",
    name,
    ": ",
    names(dichotomy[1L]),
    "{",
    paste(dichotomy[[1L]], collapse = ", "),
    "} vs. ",
    names(dichotomy[2L]),
    "{",
    paste(dichotomy[[2L]], collapse = ", "),
    "}"
  )
}

#' Effect Displays for Nested Logit Models
#'
#' @name Effect.nestedLogit
#' @aliases Effect.nestedLogit
#' @description Computes effects (in the sense of the \pkg{effects} package---see, in
#' particular, \code{\link[effects]{Effect}})---for \code{"nestedLogit"} models, which then
#' can be used with other functions in the \pkg{effects} package, for example, 
#' \code{\link[effects]{predictorEffects}} and to produce effect plots.
#' 
#' @seealso  \code{\link[effects]{Effect}}, \code{\link[effects]{plot.effpoly}},
#' \code{\link[effects]{predictorEffects}}
#' 
#' @param focal.predictors a character vector of the names of one or more of 
#' the predictors in the model, for which the effect display should be computed.
#' @param mod a \code{"nestedLogit"} model object.
#' @param confidence.level for point-wise confidence bands around the effects
#' (the default is \code{0.95}).
#' @param fixed.predictors controls the values at which other predictors are fixed;
#' see \code{\link[effects]{Effect}} for details; if \code{NULL} (the default),
#' numeric predictors are set to their means, factors to their distribution in the data.
#' @param ... optional arguments to be passed to the \code{\link[effects]{Effect}} method for
#' binary logit models (fit by the \code{\link{glm}} function).
#' @return an object of class \code{"effpoly"} (see \code{\link[effects]{Effect}}).
#' @author John Fox 
#' @keywords regression
#' @references
#' John Fox and Sanford Weisberg (2019). \emph{An R Companion to Applied Regression},
#' 3rd Edition. Sage, Thousand Oaks, CA.
#' 
#' John Fox, Sanford Weisberg (2018). Visualizing Fit and Lack of Fit in Complex
#' Regression Models with Predictor Effect Plots and Partial Residuals. 
#' \emph{Journal of Statistical Software}, 87(9), 1-27.
#' @examples
#' data("Womenlf", package = "carData")
#' comparisons <- logits(work=dichotomy("not.work",
#'                                      working=c("parttime", "fulltime")),
#'                       full=dichotomy("parttime", "fulltime"))
#' m <- nestedLogit(partic ~ hincome + children,
#'                    dichotomies = comparisons,
#'                    data=Womenlf)
#' peff.women <- effects::predictorEffects(m)
#' plot(peff.women)
#' plot(peff.women, axes=list(y=list(style="stacked")))
#' summary(peff.women)
#' 
#' dichots <- logits(AB_CD = dichotomy(c("A", "B"), c("C", "D")),
#'                   A_B   = dichotomy("A", "B"),
#'                   C_D   = dichotomy("C", "D"))
#' m.health <- nestedLogit(product4 ~ age + gender*household + position_level,
#'                         dichotomies = dichots, data = HealthInsurance)
#' eff.gen.hh <- effects::Effect(c("gender", "household"), m.health,
#'                               xlevels=list(household=0:7))
#' eff.gen.hh
#' plot(eff.gen.hh, axes=list(x=list(rug=FALSE)))
#' plot(eff.gen.hh, axes=list(x=list(rug=FALSE), 
#'                            y=list(style="stacked")))

#' @importFrom effects Effect
#' @importFrom stats model.matrix
#' @exportS3Method effects::Effect nestedLogit
Effect.nestedLogit <- function(focal.predictors, mod, confidence.level=0.95, 
                               fixed.predictors=NULL, ...){
  findEffects <- function(m, eff, fixed.predictors, ...){
    env <- environment()
    mods <- models(m)
    
    biggest <- which.max(sapply(mods, function(x) nrow(model.matrix(x))))
    formula <- mods[[biggest]]$formula
    environment(formula) <- env
    data <- mods[[biggest]]$data
    environment(data) <- env
    names(data)[ncol(data)] <- as.character(formula[[2]])
    mod <- glm(formula, data=data, family=binomial)
    given <- colMeans(model.matrix(mod)[, -1])
    if (!is.null(fixed.predictors) &&
        !is.null(fixed.predictors$given.values)) 
      given[names(fixed.predictors$given.values)] <- fixed.predictors$given.values
    if (!is.null(fixed.predictors)){
      fixed.predictors$given.values <- given
    } else {
      fixed.predictors <- list(given.values=given)
    }
    
    effects <- vector(length(mods), mode="list")
    names(effects) <- names(mods)
    for (i in seq_along(mods)){
      # refit binomial logit models
      formula <- mods[[i]]$formula
      environment(formula) <- env
      data <- mods[[i]]$data
      environment(data) <- env
      names(data)[ncol(data)] <- as.character(formula[[2]])
      mod <- glm(formula, data=data, family=binomial)
      # find effects for binomial logit models
      effects[[i]] <- Effect(eff, mod, fixed.predictors=fixed.predictors, ...)
    }
    effects
  }
  effects <- findEffects(mod, focal.predictors, fixed.predictors, ...)
  logits <- lapply(effects, function(ef) as.vector(ef$fit))
  ps <- lapply(logits, function(x) 1/(1 + exp(-x)))
  ses.lam <- lapply(effects, function(ef) ef$se)
  var.p <- mapply(function(x, y) (exp(x)/(1 + exp(x))^2)^2 * y^2, logits, ses.lam,
                 SIMPLIFY=FALSE)
  names(logits) <- names(ps) <- names(var.p) <- names(ses.lam)
  ndichot <- length(models(mod))
  var.fitted <- fitted <- vector(ndichot, mode = "list")
  for (j in 1:ndichot) {
    p <- cbind(1 - ps[[j]], ps[[j]])
    attr(p, "columns") <- models(mod, j)$dichotomy
    fitted[[j]] <- p
    var.fitted[[j]] <- var.p[[j]]
  }
  
  response.levels <- unique(unlist(lapply(fitted, function(x) attr(x, "columns"))))
  
  # compute fitted effects for nested logit model from
  #  effects for binary logit models
  
  p <- matrix(1, nrow(fitted[[1]]), length(response.levels))
  v <- matrix(0, nrow(fitted[[1]]), length(response.levels))
  colnames(v) <- colnames(p) <- response.levels
  
  for (k in response.levels) {
    
    for (j in 1:ndichot) {
      
      deriv <- rep(1, nrow(fitted[[1]]))
      
      which.j <- sapply(models(mod, j)$dichotomy, function(x) k %in% x)
      if (!any(which.j)) next
      
      for (jp in 1:ndichot){
        which.jp <- sapply(models(mod, jp)$dichotomy, function(x) k %in% x)
        if (j == jp || !any(which.jp)) next
        deriv <- deriv * fitted[[jp]][, which.jp]
      }
      
      p[, k] <- p[, k] * fitted[[j]][, which.j]
      v[, k] <- v[, k] + deriv^2 * var.fitted[[j]]
    }
  }
  
  logit <- log(p/(1 - p)) 
  se.logit <- sqrt((1/(p*(1 - p)))^2 * v)
  z <- qnorm((1 - confidence.level)/2, lower.tail=FALSE)
  lower.logit <- logit - z*se.logit
  upper.logit <- logit + z*se.logit
  lower.prob <- (1/(1 + exp(- lower.logit)))
  upper.prob <- (1/(1 + exp(- upper.logit)))
  
  # fake a multinom "effpoly" object
  
  result <- effects[[1]]
  result[c("prob", "logit", "se.prob", "se.logit", 
           "lower.logit", "upper.logit", "lower.prob", "upper.prob",
           "y.levels", "response", "model")] <-
    list(p, logit, sqrt(v), se.logit, lower.logit, upper.logit,
         lower.prob, upper.prob, response.levels, as.character(formula(mod)[2]),
         "multinom")
  class(result) <- "effpoly"
  result
}

# Nested logit models
# last modified 2023-05-02 by J. Fox
# 2023-04-06 MF: add coef method
# 2023-04-11 MF: add `dichotomies` attribute to result of nestedLogit

#' Binary Logit Models for Nested Dichotomies
#'
#' @description Fit a related set of binary logit models via the \code{\link{glm}}
#' function to nested dichotomies, comprising a model for the polytomy.
#' A polytomous response with \eqn{m} categories can be analyzed using
#' \eqn{m-1} binary logit comparisons.  When these comparisons are nested,
#' the \eqn{m-1} sub-models are statistically independent. Therefore,
#' the likelihood chi-square statistics for the sub-models are additive
#' and give overall tests for a model for the polytomy.
#' This method was introduced by Fienberg (1980),and subsequently illustrated by
#' Fox(2016) and Friendly & Meyer (2016).
#'
#' \code{dichotomy} and \code{logits} are helper functions to construct the dichotomies.
#'
#' @details A \emph{dichotomy} for a categorical variable is a comparison of one subset
#' of levels against another subset. A set of dichotomies is \emph{nested}, if after
#' an initial dichotomy, all subsequent ones are \emph{within} the groups of levels
#' lumped together in earlier ones. Nested dichotomies correspond to a binary tree
#' of the successive divisions.
#'
#' For example, for a 3-level response, a first
#' dichotomy could be \code{ {A}, {B, C}} and then the second one would be
#' just \code{{B}, {C}}. Note that in the second dichotomy, observations
#' with response \code{A} are treated as \code{NA}.
#'
#' The function \code{dichotomy} constructs a \emph{single} dichotomy in the required form,
#' which is a list of length 2 containing two character vectors giving the levels
#' defining the dichotomy. The function \code{logits} is used to create the
#' set of dichotomies for a response factor. Alternatively, the nested dichotomies can be
#' specified more compactly as a nested (i.e., recursive) list with optionally named
#' elements; for example,
#' \code{list(air="plane", ground=list(public=list("train", "bus"), private="car"))}.
#'
#' The function \code{continuationLogits} provides a
#' convenient way to generate all dichotomies for an ordered response.
#' For an ordered response with \eqn{m=4} levels, say, \code{A, B, C, D},
#' considered low to high:
#' The dichotomy first contrasts \code{B, C, D} against \code{A}.
#' The second ignores \code{A} and contrasts \code{C, D} against \code{B}.
#' The second ignores \code{A, B} and contrasts \code{D} against \code{C}.
#'
#'
#' @aliases nestedLogit logits dichotomy continuationLogits
#'
#' @param formula a model formula with the polytomous response on the left-hand side
#'        and the usual linear-model-like specification on the right-hand side.
#' @param dichotomies specification of the logits for the nested dichotomies,
#'        constructed by the \code{logits} and \code{dichotomy} functions,
#'        or \code{continuationLogits}. Alternatively, the \code{dichotomies}
#'        can be specified as a nested (i.e., recursive) list, the elements of
#'        which can be given optional names. See Details.
#' @param data a data frame with the data for the model; unlike in most statistical
#'        modeling functions, the \code{data} argument is required. Cases with \code{NA}s
#'        in any of the variables appearing in the model formula will be removed
#'        with a Note message.
#' @param subset a character string specifying an expression to fit the model
#'        to a subset of the data; the default, \code{NULL}, uses the full data set.
#' @param contrasts an optional list of contrast specification for specific factors in the
#'        model; see \code{\link{lm}} for details.
#' @param \dots for \code{nestedLogit}, optional named arguments to be passed to \code{\link{glm}};
#'        for \code{logits}, definitions of the nested logits---with each named argument specifying
#'        a dichotomy; for \code{dichotomy}, two character vectors giving the levels
#'        defining the dichotomy; the vectors can optionally be named.
#' @param levels for \code{continuationLogits}, a character vector giving the ordered levels for a set of continuation dichotomies,
#'        or the number of levels, in which case the levels will be named \code{"A"}, \code{"B"}, \code{"C"}, etc.
#' @param names an optional character vector of names for the continuation dichotomies; if absent,
#'        names will be generated from the levels.
#' @param prefix a character string (default: \code{"above_"}) used as a prefix to the names of the continuation dichotomies.
#'
#' @return \code{nestedLogit} returns an object of class \code{"nestedLogit"} containing
#' the following elements:
#' \itemize{
#'    \item \code{models}, a named list of (normally) \eqn{m - 1} \code{"glm"} objects,
#'    each a binary logit model for one of the \eqn{m - 1} nested dichotomies representing
#'    the \eqn{m}-level response.
#'    \item \code{formula}, the model formula for the nested logit models.
#'    \item \code{dichotomies}, the \code{"dichotomies"} object defining the nested dichotomies
#'    for the model.
#'    \item \code{data.name}, the name of the data set to which the model is fit, of class \code{"name"}.
#'    \item \code{data}, the data set to which the model is fit.
#'    \item \code{subset}, a character representation of the \code{subset} argument or
#'    \code{"NULL"} if the argument isn't specified.
#'    \item \code{contrasts}, the \code{contrasts} argument or \code{NULL} if the argument
#'    isn't specified.
#'    \item \code{contrasts.print} a character representation of the \code{contrasts} argument or
#'    \code{"NULL"} if the argument isn't specified.
#' }
#'   \code{logits} and \code{continuationLogits} return objects of class \code{"dichotomies"}
#'   and \code{c("continuationDichotomies" "dichotomies")}, respectively, which are two-elements lists,
#'   each element containing a list of two character vectors representing a dichotomy.
#'   \code{dichotomy} returns a list of two character vectors representing a dichotomy.
#'
#' @importFrom stats  anova binomial coef glm model.frame model.response na.omit pchisq predict update
#' @references
#' S. Fienberg (1980). \emph{The Analysis of Cross-Classified Categorical Data},
#'       2nd Edition, MIT Press, Section 6.6.
#'
#' J. Fox (2016), \emph{Applied Linear Regression and Generalized Linear Models}, 3rd Edition, Sage,
#'       Section 14.2.2.
#'
#' J. Fox and S. Weisberg (2011), \emph{An R Companion to Applied Regression}, 2nd Edition, Sage, Section 5.8.
#'
#' M. Friendly and D. Meyers (2016), \emph{Discrete Data Analysis with R}, CRC Press,
#'       Section 8.2.
#' @seealso \code{\link{nestedMethods}}
#' @author John Fox
#' @keywords regression
#' @examples
#' data("Womenlf", package = "carData")
#'
#'   #' Use `logits()` and `dichotomy()` to specify the comparisons of interest
#'   comparisons <- logits(work=dichotomy("not.work",
#'                                        working=c("parttime", "fulltime")),
#'                         full=dichotomy("parttime", "fulltime"))
#'   print(comparisons)
#'
#'   m <- nestedLogit(partic ~ hincome + children,
#'                    dichotomies = comparisons,
#'                    data=Womenlf)
#'   print(summary(m))
#'   print(car::Anova(m))
#'   coef(m)
#'
#'   # equivalent;
#'   nestedLogit(partic ~ hincome + children,
#'               dichotomies = list("not.work",
#'                                  working=list("parttime", "fulltime")),
#'               data=Womenlf)
#'
#'   # get predicted values
#'   new <- expand.grid(hincome=seq(0, 45, length=10),
#'                      children=c("absent", "present"))
#'   pred.nested <- predict(m, new)
#'
#'   # plot
#'   op <- par(mfcol=c(1, 2), mar=c(4, 4, 3, 1) + 0.1)
#'   plot(m, "hincome", list(children="absent"),
#'        xlab="Husband's Income", legend=FALSE)
#'   plot(m, "hincome", list(children="present"),
#'        xlab="Husband's Income")
#'   par(op)
#'
#'
#' @export
nestedLogit <- function(formula, dichotomies, data, subset=NULL,
                        contrasts=NULL, ...) {

  makeDichotomies <- function(y, dichotomies) {
    responses <- matrix(NA, length(y), length(dichotomies))
    for (i in seq_along(dichotomies)) {
      responses[y %in% dichotomies[[i]][[1L]], i] <- 0L
      responses[y %in% dichotomies[[i]][[2L]], i] <- 1L
    }
    colnames(responses) <- names(dichotomies)
    responses
  }

 dichotomies <- createDichotomies(dichotomies)

  data.name <- substitute(data)
  data.save <- data
  if (!is.null(subset)){
    data <- subset(data, eval(parse(text=subset), envir=data))
  }
  n.original <- nrow(data)
  data <- na.omit(data[, all.vars(formula)])
  n.final <- nrow(data)
  if (n.final < n.original) message("Note: ", n.original - n.final,
                                   " cases omitted due to missingness.")

  nested.formula <- formula
  y <- model.response(model.frame(formula, data))
  n.dichot <- length(dichotomies)
  ys <- makeDichotomies(y, dichotomies)
  models <- vector(n.dichot, mode = "list")
  resp.names <- names(dichotomies)
  names(models) <- resp.names
  formula[[2]] <- quote(..y)

  for (i in seq(length = n.dichot)) {
    data$..y <- ys[, i]
    models[[i]] <- glm(formula, family = binomial, data = data, contrasts=contrasts, ...)
    form <- models[[i]]$formula
    form[[2]] <- as.symbol(resp.names[i])
    models[[i]]$formula <- form
    call <- models[[i]]$call
    call$formula <- form
    call$data <- data.name
    models[[i]]$call <- call
    models[[i]]$dichotomy <- dichotomies[[i]]
  }

  result <- list(
    models = models,
    formula = nested.formula,
    dichotomies = dichotomies,
    data = data.save,
    data.name = data.name,
    subset = subset,
    contrasts = contrasts,
    contrasts.print = deparse(substitute(contrasts))
  )
  class(result) <- "nestedLogit"
  result
}

#' @rdname nestedLogit
#' @export
logits <- function(...) {

  checkDichotomies <- function(logits){
    nlevels <- length(unique(unlist(logits)))
    nlogits <- length(logits)
    if (nlogits > (nlevels - 1L))
      stop("too many nested dichotomies (>", nlevels - 1L, ")")
    if (nlogits < (nlevels - 1L))
      message("Note: number of nested dicotomies not equal to ", nlevels - 1L)
    return(invisible(NULL))
  }

  hasParent <- function(dichotomy){
    any(sapply(all, function(each) setequal(union(dichotomy[[1L]], dichotomy[[2L]]), each)))
  }

  allChildren <- function(dichotomies){
    all <- unlist(dichotomies, recursive=FALSE)
    for (levels in all){
      if (length(levels) == 1L) next
      which <- sapply(dichotomies, function(dichotomy) setequal(levels,
                                                                union(dichotomy[[1L]], dichotomy[[2L]])))
      if (!any(which)) message("Note: {", paste(levels, collapse=", "), "} is not subdivided")
    }
  }

  logits <- list(...)
  duplicated <- duplicated(logits)
  if (any(duplicated)){
    stop("the following dichotomies are duplicated: ",
         paste(names(logits)[duplicated], collapse=", "))
  }
  all <- unlist(logits, recursive=FALSE)
  all$all <- levels <- unique(unlist(logits))
  nested <- sapply(logits, function(x) hasParent(x))
  if (any(!nested)) stop("dichotomies aren't nested")
  allChildren(logits)
  nms <- names(logits)
  if (is.null(nms) || any(nms == "") || any(is.na(nms)))
    stop("logits must be named")
  nms2 <- make.names(nms)
  if (!all(nms2 == nms)){
    names(logits) <- nms2
    warning("logit names coerced to valid names")
  }
  class(logits) <- "dichotomies"
  attr(logits, "levels") <- levels                      # MF
  checkDichotomies(logits)
  logits
}

#' @rdname nestedLogit
#' @export
dichotomy <- function(...) {
  logit <- list(...)
  if (length(logit) != 2L)
    stop("dichotomy must define two categories")
  logit
}


#' Construct Continuation Logits for an Ordered Categorical Variable
#'
#' \code{continuationLogits} constructs a set of \eqn{m-1} logit comparisons, called
#' continuation logits,
#' for an ordered response. With \eqn{m=4} levels, say, \code{A, B, C, D},
#' considered low to high:
#' The first contrasts \code{B, C, D} against \code{A}.
#' The second ignores \code{A} and contrasts \code{C, D} against \code{B}.
#' The second ignores \code{A, B} and contrasts \code{D} against \code{C}.
#'
#' @param levels A character vector of set of levels of the variables or a number
#' specifying the numbers of levels (in which case, uppercase letters will be
#' use for the levels).
#' @param names  Names to be assigned to the dichotomies; if absent, names
#' will be generated from the levels.
#'
#' @rdname nestedLogit
#' @export
#'
#' @examples
#'   continuationLogits(c("none", "gradeschool", "highschool", "college"))
#'   continuationLogits(4)
#'
#'
#  TODO:
#  - add prefix = "above_" argument [DONE]
#  - allow reverse = FALSE (default) argument to do continuations in reverse order ???

continuationLogits <- function(levels, names, prefix = "above_"){
  if (is.numeric(levels)){
    levels <- as.integer(levels)
    levels <- LETTERS[1L:levels]
  }
  nlevels <- length(levels)
  if (nlevels < 3L) stop("too few levels: ", levels)
  logits <- vector((nlevels - 1L), mode="list")
  for (i in 1L:(nlevels - 1L)){
    logits[[i]] <- list(levels[i], levels[-(1L:i)])
  }
  if (missing(names)) names <- paste0(prefix, levels[1L:(nlevels - 1L)])
  if (length(names) != nlevels - 1L)
    stop("number of names not equal to ", nlevels - 1L)
  nms <- make.names(names)
  if (!all(names == nms)){
    names <- nms
  }
  names(logits) <- names
  attr(logits, "levels") <- levels
  class(logits) <- c("continuationDichotomies", "dichotomies")
  logits
}


# these functions not exported:

createDichotomies <- function(x){
  UseMethod("createDichotomies")
}

#' @keywords internal
#' @export
createDichotomies.dichotomies <- function(x) x

#' @keywords internal
#' @export
createDichotomies.default <- function(x)
  stop("dichotomies argument must be of class 'dichotomies' or mode 'list'")

#' @keywords internal
#' @export
createDichotomies.list <- function(x){
  helper <- function(dichotomies, x){
    if (length(x) != 2L || !is.list(x)) {
      stop("ill-formed nested list of dichotomies: ",
           paste("(length = ", length(x), "; is.list = ", is.list(x), ")"),
           "\nEach dichotomy must be a list with exactly 2 sets of levels")
    }
    a <- x[[1L]]
    b <- x[[2L]]
    nm.a <- names(x[1L])
    nm.b <- names(x[2L])
    aa <- unlist(a, recursive=TRUE)
    bb <- unlist(b, recursive=TRUE)
    name <- make.names(
      paste0(if (!is.null(nm.a) && nm.a != "") nm.a else paste(aa, collapse="."),
             "_v_",
             if (!is.null(nm.b) && nm.b != "") nm.b else paste(bb, collapse="."))
    )
    aa.bb <- list(aa, bb)
    names(aa.bb) <- c(nm.a, nm.b)
    dichotomies[[name]] <- do.call(dichotomy, aa.bb)
    if (length(a) > 1L) dichotomies <- helper(dichotomies, a)
    if (length(b) > 1L) dichotomies <- helper(dichotomies, b)
    dichotomies
  }
  dichotomies <- helper(list(), x)
  do.call(logits, dichotomies)
}

# test that update.nested() works properly

data(Womenlf, package = "carData")

m1 <- nestedLogit(partic ~ hincome, 
                  logits(work=dichotomy("not.work", c("parttime", "fulltime")),
                         full=dichotomy("parttime", "fulltime")),
                  data=Womenlf)

m2 <- nestedLogit(partic ~ hincome + children, 
                  logits(work=dichotomy("not.work", c("parttime", "fulltime")),
                         full=dichotomy("parttime", "fulltime")),
                  data=Womenlf)

m2a <- update(m1, . ~ . + children)
test_that("update.nested() works correctly, formula arg", {
  expect_equal(coef(m2), coef(m2a))
})

Ontario <- Womenlf[Womenlf$region == "Ontario", ]
m3 <- nestedLogit(partic ~ hincome + children, 
                  logits(work=dichotomy("not.work", c("parttime", "fulltime")),
                         full=dichotomy("parttime", "fulltime")),
                  data=Ontario)
m3a <- update(m2, data=Ontario)
test_that("update.nested() works correctly, data arg", {
  expect_equal(coef(m3), coef(m3a))
})

m4 <- nestedLogit(partic ~ hincome + children, 
                  logits(work=dichotomy("not.work", c("parttime", "fulltime")),
                         full=dichotomy("parttime", "fulltime")),
                  data=Womenlf,
                  contrasts=list(children=contr.sum))
m4a <- update(m2, contrasts=list(children=contr.sum))
test_that("update.nested() works correctly, contrasts arg", {
  expect_equal(coef(m4), coef(m4a))
})

m5 <- nestedLogit(partic ~ hincome + children, 
                  logits(full=dichotomy(c("not.work", "parttime"), "fulltime"),
                         part=dichotomy("not.work", "parttime")),
                  data=Womenlf)
m5a <- update(m2, dichotomies=logits(
                         full=dichotomy(c("not.work", "parttime"), "fulltime"),
                         part=dichotomy("not.work", "parttime")))
test_that("update.nested() works correctly, dichotomies arg", {
  expect_equal(coef(m5), coef(m5a))
})

# test that Effect.nestedLogit() correctly computes effects and their SEs

data(Womenlf, package = "carData")

m <- nestedLogit(partic ~ hincome + children, 
                  logits(work=dichotomy("not.work", c("parttime", "fulltime")),
                         full=dichotomy("parttime", "fulltime")),
                  data=Womenlf)

eff.1 <- Effect("children", m)
new <- eff.1$x
new$hincome <- mean(Womenlf$hincome)
pred.1 <- predict(m, newdata=new)
test_that("Effects are computed correctly for a numeric predictor", {
  expect_equal(as.vector(eff.1$prob), 
               as.vector(as.matrix(pred.1$p)))
})
test_that("Effect SEs are computed correctly for a numeric predictor", {
  expect_equal(as.vector(eff.1$se.prob), 
               as.vector(as.matrix(pred.1$se.p)))
})

Womenlf$kids <- with(Womenlf, ifelse(children == "present", 1, 0))
mm <- update(m, . ~ . - children + kids, data=Womenlf)
eff.2 <- Effect("hincome", m)
new <- eff.2$x
new$kids <- mean(Womenlf$kids)
pred.2 <- predict(mm, newdata=new)
test_that("Effects are computed correctly for a factor predictor", {
  expect_equal(as.vector(eff.2$prob), 
               as.vector(as.matrix(pred.2$p)))
})
test_that("Effect SEs are computed correctly for a factor predictor", {
  expect_equal(as.vector(eff.2$se.prob), 
               as.vector(as.matrix(pred.2$se.p)))
})

# test that predict.nestedLogit() correctly computes SEs of fitted probabilities

m.gss <- nestedLogit(degree ~ parentdeg + year, 
                     continuationLogits(c("<highschool",  "highschool", 
                                          "college", "graduate")),
                     data=GSS)

new <- expand.grid(parentdeg=c("<highschool",  "highschool", 
                               "college", "graduate"),
                   year=seq(1972, 2016, length=5))
pred.gss <- predict(m.gss, new)

GSS$y1 <- with(GSS, ifelse(degree == "<highschool", 0, 1))
GSS$y2 <- with(GSS, ifelse(y1 == 0, NA,
                           ifelse(degree == "highschool", 0, 1)))
GSS$y3 <- with(GSS, ifelse(degree == "college", 0,
                           ifelse(degree == "graduate", 1, NA)))
m1 <- glm(y1 ~ parentdeg + year, data=GSS, family=binomial)
m2 <- glm(y2 ~ parentdeg + year, data=GSS, family=binomial)
m3 <- glm(y3 ~ parentdeg + year, data=GSS, family=binomial)

pr1 <- predict(m1, newdata=new, type="response", se.fit=TRUE)
p1 <- pr1$fit
v1 <- (pr1$se.fit)^2

pr2 <- predict(m2, newdata=new, type="response", se.fit=TRUE)
p2 <- pr2$fit
v2 <- (pr2$se.fit)^2

pr3 <- predict(m3, newdata=new, type="response", se.fit=TRUE)
p3 <- pr3$fit
v3 <- (pr3$se.fit)^2

se.p.a <- sqrt(v1)
se.p.b <- sqrt((1 - p2)^2*v1 + p1^2*v2)
se.p.c <- sqrt((p2*(1 - p3))^2*v1 + (p1*(1 - p3))^2*v2 + (p1*p2)^2*v3)
se.p.d <- sqrt((p2*p3)^2*v1 + (p1*p3)^2*v2 + (p1*p2)^2*v3)

test_that("SEs of fitted probabilities computed correctly", {
  expect_equal(as.vector(cbind(se.p.a, se.p.b, se.p.c, se.p.d)), 
               as.vector(as.matrix(pred.gss$se.p)))
})

# test that nestedLogit()/predict.nestedLogit() correctly compute fitted probabilities

m2 <- nestedLogit(degree ~ parentdeg + year, 
                  continuationLogits(c("<highschool",  "highschool", 
                                       "college", "graduate")),
                  data=GSS)

new <- expand.grid(parentdeg=c("<highschool",  "highschool", 
                               "college", "graduate"),
                   year=1972:2016)
pred.GSS <- as.matrix(predict(m2, new)$p)

p.m2.1 <- predict(models(m2, 1), newdata=new, type="response")
p.m2.2 <- predict(models(m2, 2), newdata=new, type="response")
p.m2.3 <- predict(models(m2, 3), newdata=new, type="response")

test_that("fitted probabilities computed correctly, test 1", {
  expect_equal(1 - p.m2.1, pred.GSS[, "<highschool"])
})

test_that("fitted probabilities computed correctly, test 2", {
  expect_equal(p.m2.1*(1 - p.m2.2), pred.GSS[, "highschool"])
})

test_that("fitted probabilities computed correctly, test 3", {
  expect_equal(p.m2.1*p.m2.2*(1 - p.m2.3), pred.GSS[, "college"])
})

test_that("fitted probabilities computed correctly, test 4", {
  expect_equal(p.m2.1*p.m2.2*p.m2.3, pred.GSS[, "graduate"])
})

#' @name HealthInsurance
#' @docType data
#' 
#' @title Choice of Health Insurance Product
#'
#' @description A company recently introduced a new health insurance provider for 
#' its employees. At the beginning of the year the employees had to choose one of 
#' three (or four) different health plan products from this provider to best suit 
#' their needs.
#' 
#' This dataset was modified from its original source (McNulty, 2022) for the 
#' present purposes by adding a fourth choice, sampled randomly from the original three.
#'
#' @usage data("HealthInsurance", package = "nestedLogit")
#'
#' @format A data frame with 1448 rows and 7 columns.
#' \describe{
#'   \item{product}{Choice among three products, a factor with levels \code{"A"}, \code{"B"}, 
#'   and \code{"C"}.}
#'   \item{product4}{Choice among four products, a factor with levels \code{"A"}, \code{"B"}, 
#'   \code{"C"}, and \code{"D"}.}
#'   \item{age}{The age of the individual, in years.}
#'   \item{household}{The number of people living with the individual in the 
#'   same household.}
#'   \item{position_level}{Position level in the company at the time the choice was made, 
#'   where 1 is is the lowest level and 5 is the highest, a numeric vector.}
#'   \item{gender}{The gender of the individual, a factor with levels \code{"Female"}
#'   and \code{"Male"}.}
#'   \item{absent}{The number of days the individual was absent from work in the year 
#'   prior to the choice,}
#'   }
#'
#' @source Originally taken from McNulty, K. (2022). 
#' \emph{Handbook of Regression Modeling in People Analytics},
#' \url{https://peopleanalytics-regression-book.org/data/health_insurance.csv}.
#' 
#' @seealso \code{\link{nestedLogit}}.
#'
#' @examples
#' lbinary <- logits(AB_CD = dichotomy(c("A", "B"), c("C", "D")),
#'                   A_B   = dichotomy("A", "B"),
#'                   C_D   = dichotomy("C", "D"))
#' as.matrix(lbinary)
#' health.nested <- nestedLogit(product4 ~ age  + gender * household + position_level,
#'                              dichotomies = lbinary, data = HealthInsurance)
#'                              car::Anova(health.nested)
#' coef(health.nested)
"HealthInsurance"

#' @name gators
#' @docType data
#'
#' @title Alligator Food Choice
#'
#' @description Agresti (1996, p. 207) gives this data on 59 alligators sampled
#' from a lake in Florida. It has the length of the alligator in meters and the
#' primary food type found in the alligator's stomach. The food type was classified
#' into three categories: "Fish", "Invertebrates", and "Other".
#'
#' Of interest is whether or not the length of an alligator is associated with the
#' primary food type. Does knowing the length of an alligator give us some indication
#' about its primary food type? If so, how is length associated with the choice of
#' food type?
#'
#' @usage data("gators", package = "nestedLogit")
#'
#' @format A data frame with 59 rows and 2 columns.
#' \describe{
#'   \item{food}{Primary food type found in the alligator's stomach, a factor with
#'   levels \code{"Other"}, \code{"Fish"}, and \code{"Invertebrates"}.}
#'   \item{length}{Length of the alligator in meters, a numeric vector.}
#'   }
#'
#' @source Agresti, A. (1996). \emph{An Introduction to Categorical Data Analysis}. Wiley.
#'
#' @references
#' An example using this from \url{https://data.library.virginia.edu/getting-started-with-multinomial-logit-models/}.
#'
#' @seealso \code{\link{nestedLogit}}.
#'
#' @examples
#' data(gators)
#' table(gators$food)
#' # average length of gators by food
#' with(gators, tapply(length, food, mean))
#'
#' # create dichotomies
#' dichot <- logits(d1=dichotomy("Other", c("Fish", "Invertebrates")),
#'                  d2=dichotomy("Fish", "Invertebrates"))
#'
#' gators.nested <- nestedLogit(food ~ length,
#'                             dichotomies = dichot,
#'                             data = gators)
#' car::Anova(gators.nested)
#'
#' # use the plot method
#' plot(gators.nested, x.var = "length")

NULL

#' Hypothesis-Testing and Related Methods for \code{"nestedLogit"} Objects
#'
#' @name nestedHypotheses
#' @aliases nestedHypotheses Anova.nestedLogit print.Anova.nestedLogit anova.nestedLogit print.anova.nestedLogit logLik.nestedLogit
#'
#' @description Various methods for testing hypotheses about nested logit models.
#' \describe{
#'   \item{\code{Anova}}{Calculates type-II or type-III analysis-of-variance tables for \code{"nestedLogit"} objects;
#'   see \code{\link[car]{Anova}} in the \pkg{car} package.}
#'   \item{\code{anova}}{Computes sequential analysis of variance (or deviance) tables for one or more fitted
#'   \code{"nestedLogit"}  objects; see \code{\link{anova}}.}
#'   \item{\code{linearHypothesis}}{Computes Wald tests for linear hypotheses;
#'   see \code{\link[car]{linearHypothesis}} in the \pkg{car} package.}
#'   \item{\code{logLik}}{Returns the log-likelihood and degrees of freedom for the nested-dichotomies model.
#'   (and through it \code{\link{AIC}} and \code{\link{BIC}} model-comparison statistics).}
#' }
#' @param x,object,object2,mod,model in most cases, an object of class \code{"nestedLogit"}.
#' @param \dots arguments to be passed down. In the case of \code{linearHypothesis},
#'        the second argument is typically the \code{hypothesis.matrix}. See the
#'        Details section of \code{\link[car]{linearHypothesis}}. In the case of \code{anova},
#'        additional sequential \code{"nestedLogit"} models.
#'
#' @return \itemize{
#'    \item The \code{Anova} and \code{anova} methods return objects
#'          of class \code{"Anova.nestedLogit"} and \code{"anova.nestedLogit"}, respectively,
#'          each of which contains a list of \code{"anova"} objects (see \code{\link{anova}})
#'          and is usually printed.
#'    \item The \code{linearHypothesis} method is called for its side effect, printing
#'          the result of linear hypothesis tests, and invisibly returns \code{NULL}.
#'    \item The \code{logLik} method returns an object of class \code{"logLik"} (see \code{\link{logLik}}).
#' }
#' @seealso \code{\link[car]{Anova}}, \code{\link{anova}},
#' \code{\link[car]{linearHypothesis}}, \code{\link{logLik}}, \code{\link{AIC}},
#' \code{\link{BIC}}
#' @author John Fox
#' @keywords regression
#' @examples
#' # define continuation dichotomies for level of education
#' cont.dichots <- continuationLogits(c("<highschool",
#'                                      "highschool",
#'                                      "college",
#'                                      "graduate"))
#' # fit a nested model for the GSS data examining education degree in relation to parent & year
#' m <- nestedLogit(degree ~ parentdeg + year,
#'                  cont.dichots,
#'                  data=GSS)
#'
#' # Anova and anova tests
#' car::Anova(m) # type-II (partial) tests
#'
#' anova(update(m, . ~ . - year), m) # model comparison
#'
#' # Wald test
#' car::linearHypothesis(m, c("parentdeghighschool", "parentdegcollege",
#'                            "parentdeggraduate"))
#'
#' # log-liklihood, AIC, and BIC
#' logLik(m)
#' AIC(m)
#' BIC(m)

#' @rdname nestedHypotheses
#' @importFrom car Anova
#' @exportS3Method car::Anova nestedLogit
#' @export
Anova.nestedLogit <- function(mod, ...) {
  result <- lapply(models(mod), Anova, ...)
  nms <- names(models(mod))
  heading <- attr(result[[1L]], "heading")[1L]
  heading <- sub("Table", "Tables", heading)
  for (i in seq(along = result)) {
    attr(result[[i]], "heading") <- composeResponse(nms[i], models(mod, i)$dichotomy)
  }
  attr(result, "heading") <- heading
  class(result) <- "Anova.nestedLogit"
  result
}

#' @rdname nestedHypotheses
#' @exportS3Method print Anova.nestedLogit
print.Anova.nestedLogit <- function(x, ...) {
  if (length(x) < 2L) {
    return(invisible(print(x[[1L]], ...)))
  }
  cat("\n", attr(x, "heading"), "\n")
  table <- print(x[[1L]], ...)
  for (i in 2L:length(x)) {
    cat("\n\n")
    table <- table + print(x[[i]], ...)
  }
  table[, 3L] <- pchisq(table[, 1L], table[, 2L], lower.tail = FALSE)
  attr(table, "heading") <- "Combined Responses"
  class(table) <- c("anova", "data.frame")
  cat("\n\n")
  print(table)
  invisible(x)
}

#' @rdname nestedHypotheses
#' @importFrom car linearHypothesis
#' @exportS3Method car::linearHypothesis nestedLogit
#' @export
linearHypothesis.nestedLogit <- function(model, ...) {
  nms <- names(models(model))
  h <- car::linearHypothesis(models(model, 1L), ...)
  formula <-  as.character(model$formula)
  heading <- attr(h, "heading")
  heading[length(heading) - 1] <- paste("Model 1: restricted model\nModel 2:",
                                        paste(formula[2], formula[1], formula[3],
                                              collapse = " "))
  for (line in heading){
    cat(paste(line, "\n"))
  }
  attr(h, "heading") <- NULL
  table <- h
  cat(composeResponse(nms[1L], models(model, 1L)$dichotomy), "\n")
  print(h)
  for (i in 2L:length(nms)) {
    cat("\n", composeResponse(nms[i], models(model, i)$dichotomy), "\n", sep="")
    h <- car::linearHypothesis(models(model, i), ...)
    attr(h, "heading") <- NULL
    print(h)
    table <- table + h
  }
  chisq <- table$Chisq[2]
  df <- table$Df[2]
  p <- pchisq(chisq, df, lower.tail=FALSE)
  cat(paste0("\nCombined Responses\nChisq = ", round(chisq, 3), ", Df = ", df,
             ", Pr(>Chisq) = ", format.pval(p)))
  return(invisible(NULL))
}

#' @rdname nestedHypotheses
#' @export
anova.nestedLogit <- function(object, object2, ...){
  if (missing(object2)){
    result <- lapply(models(object), anova, test="LRT")
    heading <- attr(result[[1L]], "heading")[1L]
    heading <- sub("Table", "Tables", heading)
    heading <- sub("\\.\\.y", as.character(object$formula[[2]]), heading)
    heading <- sub("binomial, link: logit", "Nested Logit", heading)
  } else {
    if (!inherits(object2, "nestedLogit"))
      stop(deparse(substitute(object2)), " is not of class 'nestedLogit'")
    result <- mapply(anova, models(object), models(object2), MoreArgs=list(test="LRT"), SIMPLIFY=FALSE)
    heading <- attr(result[[1L]], "heading")
    heading <- sub("Table", "Tables", heading)
    heading <- gsub("\\.\\.y", as.character(object$formula[[2]]), heading)
    heading <- sub("Model 2", " Model 2", heading)
    heading <- c(heading, "\n")
  }
  nms <- names(models(object))
  for (i in seq(along = result)) {
    attr(result[[i]], "heading") <- composeResponse(nms[i], models(object, i)$dichotomy)
  }
  attr(result, "heading") <- heading
  class(result) <- "anova.nestedLogit"
  result
}

#' @rdname nestedHypotheses
#' @exportS3Method print anova.nestedLogit
print.anova.nestedLogit <- function(x, ...) {
  if (length(x) < 2L) {
    return(invisible(print(x[[1L]], ...)))
  }
  cat("\n", attr(x, "heading"), "\n")
  table <- print(x[[1L]], ...)
  for (i in 2L:length(x)) {
    cat("\n\n")
    table <- table + print(x[[i]], ...)
  }
  table[, "Pr(>Chi)"] <- pchisq(table[, "Deviance"], table[, "Df"], lower.tail = FALSE)
  attr(table, "heading") <- "Combined Responses"
  class(table) <- c("anova", "data.frame")
  cat("\n\n")
  print(table)
  invisible(x)
}

# also provides BIC() and AIC():

#' @rdname nestedHypotheses
#' @exportS3Method stats::logLik nestedLogit
#' @export
logLik.nestedLogit <- function(object, ...){
  result <- lapply(models(object), logLik)
  logLik <- sum(sapply(result, I))
  df <- sum(sapply(result, function(x) attr(x, "df")))
  nobs <- max(sapply(result, function(x) attr(x, "nobs")))
  result <- logLik
  attr(result, "df") <- df
  attr(result, "nobs") <- nobs
  class(result) <- "logLik"
  result
}

---
title: "Using ggeffects with nestedLogit models"
author: "Michael Friendly"
date: "`r Sys.Date()`"
package: nestedLogit
output:
  rmarkdown::html_vignette:
  fig_caption: yes
bibliography: ["references.bib", "packages.bib"]
csl: apa.csl
vignette: >
  %\VignetteIndexEntry{Using ggeffects with nestedLogit models}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  message = FALSE,
  warning = FALSE,
  fig.align = "center",
  fig.height = 6,
  fig.width = 7,
  fig.path = "fig/",
  dev = "png",
  comment = "#>"
)

# save some typing
knitr::set_alias(w = "fig.width",
                 h = "fig.height",
                 cap = "fig.cap")

set.seed(47)
.opts <- options(digits = 4)

# packages to be cited in packages.bib
.to.cite <- c("nestedLogit", "ggeffects", "ggplot2")
```

Load the packages we'll use here:
```{r setup}
library(nestedLogit)    # Nested Dichotomy Logistic Regression Models
library(ggeffects)      # Create Tidy Data Frames of Marginal Effects
library(ggplot2)        # Data Visualisations Using the Grammar of Graphics
```

## Overview

The `ggeffects` package [@R-ggeffects; @ggeffects2018] provides a simple and unified interface for
computing and plotting adjusted predictions and marginal effects from a wide
variety of regression models. Its main function, `predict_response()`,
returns a tidy data frame of model predictions that can be plotted directly
with a built-in `plot()` method or further customized with `ggplot2`.

The package now supports `"nestedLogit"` objects, making it easy to visualize predicted
probabilities for each response category across levels of the predictors, without the
manual data wrangling described in `vignette("plotting-ggplot", package = "nestedLogit")`.

**Note**:: `ggeffects` will at some point be superseded by the [`modelbased`](https://easystats.github.io/modelbased/) package from the [`easystats`](https://easystats.github.io/easystats/) project. 


## 👩 Women's labor force participation

We use the standard `Womenlf` example from the main vignette. The response `partic` has
three categories --- not working, working part-time, and working full-time --- modeled
as nested dichotomies against husband's income and presence of young children.

```{r wlf-model}
data(Womenlf, package = "carData")
comparisons <- logits(work = dichotomy("not.work", c("parttime", "fulltime")),
                      full = dichotomy("parttime", "fulltime"))

wlf.nested <- nestedLogit(partic ~ hincome + children,
                          dichotomies = comparisons,
                          data = Womenlf)
```

### Predicted probabilities with `predict_response()`

The simplest way to obtain predicted probabilities and a plot is with
`predict_response()`, specifying the focal predictors in the `terms` argument.
This returns predicted probabilities for each response level, with confidence intervals,
averaged over the non-focal predictors. The default print method displays these ("prettified") for a
small subset of values of the quantitative predictor, `hincome`.

```{r wlf-predict-response}
wlf.pred <- predict_response(wlf.nested, terms = c("hincome", "children"))
wlf.pred
```

The default `plot()` method produces a faceted plot with one panel for each response category
and separate curves for the levels of the other predictor (`children`).

```{r wlf-ggeffects-plot1}
#| out.width = "100%",
#| fig.height = 5,
#| fig.cap = "Predicted probabilities from `predict_response()` with default `ggeffects` plot."
plot(wlf.pred)
```

### Customizing the plot

The `plot()` method returns a `ggplot` object, so it can be further customized with
standard `ggplot2` functions. For example, we can adjust the line size, labels,
theme, and legend position:

```{r wlf-ggeffects-plot2}
#| out.width = "100%",
#| fig.height = 5,
#| fig.cap = "Customized `ggeffects` plot with adjusted labels and theme."
plot(wlf.pred,
     line_size = 2) +
  labs(title = "Predicted Probabilities of Work by Husband's Income",
       y = "Probability",
       x = "Husband's Income") +
  theme_ggeffects(base_size = 14) +
  theme(legend.position = "top")
```

### Plotting on the logit scale

`ggplot2` provides a built-in `"logit"` transformation for axes via
`scale_y_continuous(transform = "logit")`. This displays predicted probabilities
on the logit scale, $\text{logit}(p) = \log(p / (1-p))$, where the axis labels
remain as probabilities but their spacing reflects the logit transformation.
This is useful because the logistic regression model is linear on the logit scale,
so the predicted curves appear as straighter lines.

Since the `plot()` method for `ggeffects` returns a `ggplot` object, we can
simply add this scale transformation. Note that we need to specify breaks
manually, because the automatic break algorithm does not work well with the
logit transformation.

```{r wlf-logit-scale}
#| out.width = "100%",
#| fig.height = 5,
#| fig.cap = "Predicted probabilities on the logit scale."
plot(wlf.pred,
     line_size = 2) +
  scale_y_continuous(
    transform = "logit",
    breaks = c(0.05, 0.10, 0.25, 0.50, 0.75, 0.90, 0.95)
  ) +
  labs(title = "Predicted Probabilities (logit scale)",
       y = "Probability (logit scale)",
       x = "Husband's Income") +
  theme_ggeffects(base_size = 16) +
  theme(legend.position = "top")
```

These plots show something different, and simpler than what appears on the probability scale. For both the `not.work`
and `fulltime` response categories, the effect of children is essentially an additive one, with having small children
reducing the probability. The response category `parttime` shows an interactive pattern, with different shapes
for those with children present vs. absent.

### Plotting the dichotomies

Recent updates[^issue671] to `ggeffects` now make it possible to calculate and plot the predicted probabilities
for the  sub-models that comprise the nested logit --- for example, plotting
predicted values for the `work` and `full` dichotomies separately. This is done by
passing the argument `submodels = "dichotomies"` to `predict_response()`. 

[^issue671]: I'm grateful to Daniel Lüdecke for considering [this problem](https://github.com/strengejacke/ggeffects/issues/671) and revising the `ggeffects` package to more fully accommodate nestedLogit models.

In such plots,
it is sometimes useful to show explicitly the points on the grid of predictor values
used in estimation. Just add `geom_point()` for this. You can also achieve greater
resolution in the plot by moving the legend inside the plot as shown below.


```{r wlf-dichotomies}
#| out.width = "100%",
#| fig.height = 5,
#| fig.cap = "Predicted probabilities for the two dichotomies."
wlf.pred.dichot <- predict_response(wlf.nested, terms = c("hincome", "children"),
                             submodel = "dichotomies")

plot(wlf.pred.dichot) +
  geom_point() +
  theme(legend.position = "inside",
        legend.position.inside = c(.40, .85))
```


In this view, we see that the decision to work or not varies in a simple decreasing manner
with husband's income, and there is a large decrement in the probability of working
when there are young children in the home.

For those women who are working, the distinction between working fulltime vs. parttime is also
simple to describe.


## 🐊 Alligator food choice

As a simpler example with a single continuous predictor, we fit a nested logit model to the
`gators` data (originally from @Agresti:2002)
predicting primary food choice from alligator length.
The first dichotomy contrasts {Other} vs. {Fish, Invertebrates}, and the second
contrasts {Fish} vs. {Invertebrates}.

```{r gators-model}
data(gators)

# setup the dichotomies
gators.dichots = logits(
     other   = dichotomy("Other", c("Fish", "Invertebrates")),
     fish_inv = dichotomy("Fish", "Invertebrates"))

as.tree(gators.dichots)

# fit the model
gators.nested <- nestedLogit(food ~ length,
                             dichotomies = gators.dichots,
                             data = gators)
```

Now, get the predicted response probabilities, and plot them:
```{r gators-ggeffects}
#| out.width = "100%",
#| fig.height = 4,
#| fig.cap = "Predicted food choice probabilities for alligators by length."
predict_response(gators.nested, terms = "length") |>
  plot(line_size = 2)
```

As you can see, the main thing going on here is that larger alligators prefer fish, which smaller ones prefer invertebrates.
This makes perfect sense if you're an alligator!

For comparison, the basic `nestedLogit::plot()` method using default arguments gives a similar plot, with the three curves
overlaid in a single panel (it uses `graphics::matplot()`). A new feature of the plot method is
to dispense with a legend entirely, by using direct labels (`label = TRUE`) on the predicted curves.

```{r gators-plot, echo=-1}
#| out.width = "100%",
#| fig.height = 5,
#| fig.cap = "Predicted food choice probabilities for alligators by length."
par(mar = c(4, 4, 1, 1) + 0.5)
plot(gators.nested, x.var = "length",
     lty=1, lwd = 4,
     label = TRUE, label.col = "black", cex.lab = 1.3)

```

## Summary

The `ggeffects` package computes and plots predicted probabilities for the
_response categories_ of a nested logit model. As shown above, these can be
displayed on the logit scale using `ggplot2`'s built-in axis transformation.
You can now also display the predicted probabilities for the separate
dichotomies (e.g., `work` and `full`) as illustrated above.

<!--
However, `ggeffects` does not currently provide access to the individual
dichotomy sub-models that comprise the nested logit --- for example, plotting
predicted values for the `work` and `full` dichotomies separately.
(This is due to a conflict in the names of arguments (`model`), and may be resolved
in a future version of `nestedLogit`.)
-->

For  more specialized displays, see `vignette("plotting-ggplot", package = "nestedLogit")`,
which describes a manual workflow using `predict()` with `model = "dichotomies"` and
`as.data.frame()` to construct fully customized `ggplot2` plots.

```{r write-bib, echo=FALSE}
pkgs <- unique(c(.to.cite, .packages()))
knitr::write_bib(pkgs, file = here::here("vignettes", "packages.bib"))
```

## References

```{r, include = FALSE}
options(.opts)
```

---
title: "Plotting nestedLogit models with ggplot2"
author: "Michael Friendly and John Fox"
date: "`r Sys.Date()`"
package: nestedLogit
output:
  rmarkdown::html_vignette:
  fig_caption: yes
bibliography: ["references.bib", "packages.bib"]
csl: apa.csl
vignette: >
  %\VignetteIndexEntry{Plotting nestedLogit models with ggplot2}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  message = FALSE,
  warning = FALSE,
  fig.align = "center",
  fig.height = 6,
  fig.width = 7,
  fig.path = "fig/",
  dev = "png",
  comment = "#>"
)

# save some typing
knitr::set_alias(w = "fig.width",
                 h = "fig.height",
                 cap = "fig.cap")

# colorize text
colorize <- function(x, color) {
  if (knitr::is_latex_output()) {
    sprintf("\\textcolor{%s}{%s}", color, x)
  } else if (knitr::is_html_output()) {
    sprintf("<span style='color: %s;'>%s</span>", color,
      x)
  } else x
}


set.seed(47)
.opts <- options(digits = 4)

# packages to be cited here. Code at the end automatically updates packages.bib
#to.cite <- c("ggplot2", "geomtextpath", "equatiomatic")
```

Load the packages we'll use here:
```{r setup}
library(nestedLogit)    # Nested Dichotomy Logistic Regression Models
library(knitr)          # A General-Purpose Package for Dynamic Report Generation in R
library(dplyr)          # A Grammar of Data Manipulation
library(tidyr)          # Tidy Messy Data
library(ggplot2)        # Create Elegant Data Visualisations Using the Grammar of Graphics
library(geomtextpath)   # Curved Text in 'ggplot2'
```

The main vignette illustrated the basic plot method, `plot.nestedLogit()` in the package.
However, for better control of the details and possibly more pleasing graphs,
it is useful to describe how graphs can be constructed using `ggplot2` [@R-ggplot2]. We'll use the
same example of women's labor force participation, using the original dichotomies:

```{r wlf-model}
data(Womenlf, package = "carData")
comparisons <- logits(work=dichotomy("not.work", c("parttime", "fulltime")),
                      full=dichotomy("parttime", "fulltime"))

wlf.nested <- nestedLogit(partic ~ hincome + children,
                          dichotomies = comparisons,
                          data=Womenlf)
```

The advantages of this approach are that 

* you can handle more complicated designs with more predictors by faceting,
* it allows you to plot either the fitted probabilities or their transformed logits,
* you can also obtain plots for log odds corresponding to each of the dichotomies which comprise the nested logit model.

As we will illustrate, this provides a nice visual interpretation of the alternative
specification of dichotomies for the `Womenlf` data discussed in the section
"Alternative models for the `Womenlf` data" of the main vignette.


## Fitted probabilities
To draw a plot, it is sufficient to calculate predicted probabilities over a grid of
values of the predictor variables. Here, we select a range of 0 - 45 in steps of 5,
combined with the two values of `children`.

```{r pred.nested}
new <- expand.grid(hincome=seq(0, 45, by = 5),
                   children=c("absent", "present"))

pred.nested <- predict(wlf.nested, newdata = new)
names(pred.nested)
```

As explained in `help(predict.nestedLogit)`, the predict method returns a complicated structure --
a list of four data frames corresponding to the predicted probabilities for the response categories,
the corresponding logits, and each of their standard errors. 

```{r}
head(pred.nested[["p"]])
```

However, `ggplot` wants the data in long format. 
This is easily done using the `as.data.frame()` method, which also includes the values of the predictors in the `newdata` data set:

```{r}
plotdata <- as.data.frame(pred.nested, newdata=new)
head(plotdata)
```

## Plotting with `ggplot2`
Then we can plot probability against one predictor, use `color` to distinguish the levels of the response
(`partic`) and facet the plot by `children`. The point-wise standard errors are drawn
in a 68% confidence envelope using `geom_ribbon()`. We've also plotted the predicted values as points to
show where the predictions are obtained.
```{r wlf-ggplot-p1}
#| out.width = "100%",
#| fig.height = 4,
#| fig.cap = "**ggplot**: Predicted probabilities of working at all or working part time or full time versus husband's income, by `children`"

theme_set(theme_bw(base_size = 14))

gg1 <- ggplot(plotdata,
       aes(x=hincome, y=p, color=response)) +
  geom_line(linewidth = 2) +
  geom_point(size = 1.5, shape = 16, color = "black") +
  labs(x="Husband's Income", y= "Probability") +
  facet_wrap(~ children, labeller = label_both) +
  geom_ribbon(aes(ymin=p - se.p,
                  ymax=p + se.p,
                  fill = response), alpha = 0.3) 

gg1
```
It is noteworthy that the confidence envelopes are wider for not-working women at higher levels of husband's income, where there are fewer observations.

### Direct labels

Plot legends are somewhat hard to read and take up unnecessary space in the plot, so it is often better
to label the curves directly.  The `geomtextpath` package [@R-geomtextpath] produces a nicer plot.

```{r wlf-ggplot-p2}
#| out.width = "100%",
#| fig.height = 4,
#| fig.cap = "The same plot using direct labels on the curves rather than a legend."
gg1 + geom_textline(aes(label = response), 
                    hjust = -0.01, vjust=-0.5, size=5) +
  theme(legend.position = "none")
```

### Plotting log-odds
The advantage of the data structure returned by `as.data.frame()` is that you can just as easily plot the predicted probabilities on the scale of log-odds
($\text{logit}(p) = \log(p / (1-p))$),
using the `logit` and
`logit.se` components.

```{r wlf-ggplot-logit}
#| out.width = "100%",
#| fig.height = 4,
#| fig.cap = "Predicted log odds of working at all or working part time or full time versus husband's income, by `children`"

ggplot(plotdata,
       aes(x=hincome, y=logit, color=response)) +
  geom_line(linewidth = 2) +
  geom_point(size = 1.5, shape = 16, color = "black") +
  labs(x="Husband's Income", y= "Log Odds") +
  facet_wrap(~ children, labeller = label_both) +
  geom_ribbon(aes(ymin=logit - se.logit,
                  ymax=logit + se.logit,
                  fill = response), alpha = 0.3) +
  geom_textline(aes(label = response), 
                hjust = -0.01, vjust=-0.5, size=5) +
  theme(legend.position = "none")
```

## Predicted logit values for the dichotomies

The nested logit model `wlf.nested` comprises the two binary logistic regression
models for the `work` and `full` dichotomies. We can plot these as follows.

```{r}
names(models(wlf.nested))
```

The `predict()` method can also generate predicted values and their standard errors for the logits of these dichotomies, using the  `model = "dichotomies"` argument:
```{r}
pred.dichot <- predict(wlf.nested, newdata = new,
                       model = "dichotomies")
str(pred.dichot)
```

Transforming this to a data frame, we get an analogous result for plotting:

```{r}
plotlogit <- as.data.frame(pred.dichot, newdata = new)
head(plotlogit)
```

Then, plot the `logit` vs. husband's income, with separate curves for the two
sub-models:

```{r wlf-ggplot-dichot1}
#| out.width = "100%",
#| fig.height = 4,
#| fig.cap = "Predicted logits for the two dichotomies, `work` and `full` versus `hincome`, by `children`"
ggplot(plotlogit,
       aes(x=hincome, y=logit, color=response)) +
  geom_line(linewidth = 2) +
  geom_point(size = 1.5, shape = 16, color = "black") +
  labs(x="Husband's Income", y= "Log Odds") +
  facet_wrap(~ children, labeller = label_both) +
  geom_ribbon(aes(ymin=logit - se.logit,
                  ymax=logit + se.logit,
                  fill = response), alpha = 0.3) +
  geom_textline(aes(label = response),
                hjust = -0.01, vjust=-0.5, size=5) +
  theme(legend.position = "none")
```

Or, interchanging the roles of `children` and `response`, we can plot these the other way, faceting by `response`.

```{r wlf-ggplot-dichot2}
#| out.width = "100%",
#| fig.height = 4,
#| fig.cap = "Predicted logits for the two dichotomies, `work` and `full` versus `hincome`, by `response`"
ggplot(plotlogit,
       aes(x=hincome, y=logit, color=children)) +
  geom_line(linewidth = 2) +
  geom_point(size = 1.5, shape = 16, color = "black") +
  labs(x="Husband's Income", y= "Log Odds") +
  facet_wrap(~ response, labeller = label_both) +
  geom_ribbon(aes(ymin=logit - se.logit,
                  ymax=logit + se.logit,
                  fill = children), alpha = 0.3) +
  geom_textline(aes(label = children),
                hjust = -0.01, vjust=-0.5, size=5) +
  theme(legend.position = "none")
```

This nicely illustrates the nature of the fitted logit models: The lines in each panel
have the same slopes for the two levels of `children`, differing only in their intercepts.
The `full` distinction between working full-time vs. part-time decreases faster with husband's
income than for the `work` dichotomy between not working at all and working either
part-time or full-time.

## Alternative model

In the main vignette we mentioned that
an alternative set of nested dichotomies first contrasts full-time work with the other categories, {full-time} vs. {not working, part-time}, and then {not working} vs. {part-time}.

```{r alt-model}
wlf.nested.alt <- nestedLogit(partic ~ hincome + children,
                              logits(full=dichotomy(nonfulltime=c("not.work", "parttime"), "fulltime"),
                                     part=dichotomy("not.work", "parttime")),
                              data=Womenlf)
```

Proceeding in the same way as above, we get predicted logits and standard errors for each
of the dichotomies.

```{r}
pred.dichot.alt <- predict(wlf.nested.alt, newdata = new,
                       model = "dichotomies")
plotlogit.alt <- as.data.frame(pred.dichot.alt, newdata = new)
head(plotlogit.alt)
```

Plotting these as before:
```{r wlf-ggplot-alt1}
#| out.width = "100%",
#| fig.height = 4,
#| fig.cap = "Predicted logits for the two dichotomies, `work` and `full` versus `hincome`, by `response`"
ggplot(plotlogit.alt,
       aes(x=hincome, y=logit, color=children)) +
  geom_line(linewidth = 2) +
  geom_point(size = 1.5, shape = 16, color = "black") +
  labs(x="Husband's Income", y= "Log Odds") +
  facet_wrap(~ response, labeller = label_both) +
  geom_ribbon(aes(ymin=logit - se.logit,
                  ymax=logit + se.logit,
                  fill = children), alpha = 0.3) +
  geom_textline(aes(label = children),
                hjust = -0.01, vjust=-0.5, size=5) +
  theme(legend.position = "none")
```

It's apparent that the alternative model produces a simpler description of the data: The predictors husband's income and presence of children affect the decision to work full-time, but not the decision to work part-time among those who aren't engaged in full-time work.
In particular it is clear that neither husband's income nor having young children has any effect on the
decision to work part-time.

```{r, include = FALSE}
options(.opts)
```

## References

---
title: "Other Examples of Nested Logit Models"
author: "Michael Friendly"
date: "`r Sys.Date()`"
package: nestedLogit
output:
  rmarkdown::html_vignette:
    toc: true
    toc_depth: 2
    number_sections: true
fig_caption: yes
bibliography: ["references.bib", "packages.bib"]
csl: apa.csl
vignette: >
  %\VignetteIndexEntry{Other Examples of Nested Logit Models}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r setup, include=FALSE}
knitr::opts_chunk$set(
  collapse  = TRUE,
  message   = FALSE,
  warning   = FALSE,
  fig.align = "center",
  fig.height = 5,
  fig.width  = 7,
  fig.path   = "fig/",
  dev        = "png",
  comment    = "#>"
)

# packages to be cited in packages.bib
.to.cite <- c("nestedLogit", "nnet", "car", "ggplot2", "AER", "dplyr",
               "mlogit", "vcd", "MASS")
```

# Introduction

This document explores datasets beyond those already packaged with `nestedLogit`
(`Womenlf`, `GSS`, `gators`, `HealthInsurance`) that could profit from treatment
as nested logit models, either because:

- the response categories have a natural **hierarchical tree structure** and the
  independence-of-irrelevant-alternatives (IIA) assumption of plain multinomial
  logit is implausible across nests, or
  
- the response is **ordinal** and **continuation logits** (a special case of
  nested dichotomies) are a natural model, and the **proportional odds** model
  (which assumes equal slopes for the predictors) might not be likely to hold.

From a scan of suitable datasets in available packages, the best candidates identified for this vignette  are:

| Dataset | Package | Response levels | Structure | N |
|---------|---------|-----------------|-----------|---|
| `Chile` | carData | Abstain/No/Undec/Yes | two competing trees | 2700 |
| `TravelMode` | AER | air/train/bus/car | IIA violation | 210 |
| `Fishing` | mlogit | beach/pier/boat/charter | shore vs. vessel | 1182 |
| `Arthritis` | vcd | None/Some/Marked | ordinal continuation | 84 |
| `survey$Smoke` | MASS | Never/Occas/Regul/Heavy | ordinal continuation | 237 |
| `pneumo` | VGAM | normal/mild/severe | ordinal continuation | 8 |

Polytomous response data comes in different forms (e.g., wide vs. long)
so this vignette also provides an opportunity to illustrate transformations
among these to make the data suitable for analysis and for plotting.

## Load packages
Load the packages we'll use here:
```{r load-packages}
library(nestedLogit)
library(nnet)       # for: multinom()
library(car)        # for: Anova()
library(ggplot2)
```

---

# Chilean plebiscite voting intent (`carData::Chile`) {#chile}

## Data

The September, 1988 Chilean national plebiscite asked citizens whether the military government of
Augusto Pinochet should remain in power for another eight years. Six months before that,
the independent research center FLASCO/Chile conducted a national survey of 27000 randomly
selected voters. The main question concerned their intention to vote (`vote`) in the plebiscite, which was
recorded as:

- **Yes** (support Pinochet)
- **No** (oppose Pinochet)
- **Undec** (undecided)
- **Abstain** (abstain / not vote)

[The levels of `Chile$vote` are actually just the first letters: "Y", "N", "U", "A". For ease of interpretation, these
are recoded in a step below.]

<!-- DONE: Recoded vote factor with longer labels (Yes/No/Undec/Abstain) ordered by median statusquo (No < Abstain < Undec < Yes) in the chile-drop chunk. -->

`Chile` is in `carData`, which is already a Suggests: dependency of `nestedLogit`, so
no extra package is needed. One main predictor is a standardized scale of support for the
`statusquo`, where high values represent general support for the policies of the military regime.
Other predictors are: `sex`, `age`, `education` (a factor) and `income` of the respondents.

```{r chile-data}
data("Chile", package = "carData")
str(Chile)
```



The dataset contains missing values on a number of predictors, but there were 168 cases who did not answer the
voting intention question. These are dropped here, so that all models use the same cases.
```{r chile-drop}
colSums(is.na(Chile))

# Drop the 168 rows with missing vote so all models use the same cases
chile <- Chile[!is.na(Chile$vote), ]

# Recode vote with longer labels, ordered by median statusquo (No < Abstain < Undec < Yes)
chile$vote <- factor(chile$vote,
                     levels = c("N", "A", "U", "Y"),
                     labels = c("No", "Abstain", "Undec", "Yes"))
nrow(chile)
table(chile$vote)
```

The strong predictor `statusquo` is a scale measuring attitude toward the
political status quo (higher = more supportive of Pinochet's regime).
<!-- DONE: vote levels now ordered by median statusquo: No (-1.07) < Abstain (-0.26) < Undec (0.08) < Yes (1.18) -->


```{r chile-eda}
# statusquo distribution by vote
boxplot(statusquo ~ vote, data = chile,
        xlab = "Vote", ylab = "Attitude toward status quo",
        main  = "Chile 1988: status quo attitude by voting intention")
```

With this reordering of the response levels, a clear pattern emerges in the boxplot. But, also note how the shapes
of the distributions change, and the presence of outliers. But here, the goal is to explain voting intention using
the other predictors.

## Two competing tree structures

The four `vote` categories may seem to be unordered in the general case. However, we can think more productively about
the underlying process for a respondent that leads to their choice. 
A key theoretical question is the **ordering of the decision process**, which gives rise to
two different sets of nested comparisons.

### Engagement-first tree

The voter first decides whether to participate meaningfully (cast a Yes or No
vote) vs. disengage (Abstain or remain Undecided). Engagement is then governed
by demographic/civic variables; direction by political attitude.

```
            vote
           /    \
       engaged  disengaged
       /   \     /      \
     Yes    No Abstain  Undec
```

We specify this as three nested dichotomies:
```{r chile-dichots-eng}
dichots_eng <- logits(
  engage    = dichotomy(engaged    = c("Yes", "No"),
                        disengaged = c("Abstain", "Undec")),
  direction = dichotomy("Yes", "No"),
  disengage = dichotomy("Abstain", "Undec")
)

# print method
dichots_eng

# print as an ASCII tree
as.tree(dichots_eng, response = "vote")
```


### Direction-first tree

The voter first forms a political opinion (pro or anti status quo) and only
then decides whether to act on it. `statusquo` should dominate the direction
split; demographics should govern the two engagement splits.

```
          vote
         /    \
       pro    anti
      /   \   /      \
    Yes  Undec No  Abstain
```

These dichotomies are specified as follows:

```{r chile-dichots-dir}
dichots_dir <- logits(
  direction  = dichotomy(pro  = c("Yes", "Undec"),
                         anti = c("No", "Abstain")),
  engage_pro = dichotomy("Yes", "Undec"),
  engage_ant = dichotomy("No", "Abstain")
)
dichots_dir
```

## Fitting the models

For comparison, we can fit the standard multinomial logistic regression models (using `"N"` as the baseline category)
and the two nested dichotomy models. I use the main effects of the principal predictors (ignoring `region` and `population`)
in all three models.

```{r chile-fit}
mod.formula <- vote ~ statusquo + age + sex + education + income

# Multinomial logit baseline (reference = "No")
chile2 <- within(chile, vote <- relevel(vote, ref = "No"))
chile_multi <- multinom(mod.formula, data = chile2, trace = FALSE)

# Nested logit: engagement-first tree
chile_eng <- nestedLogit(mod.formula, dichotomies = dichots_eng, data = chile)

# Nested logit: direction-first tree
chile_dir <- nestedLogit(mod.formula, dichotomies = dichots_dir, data = chile)
```

Details of this analysis are provided by `summary()`, which provides tests of coefficients as well as overall measures of fit
(residual deviance) for each dichotomy as well as for the combined models comprising all four vote choices.
This output is suppressed here
```{r chile-summaries, eval=FALSE}
summary(chile_eng)
summary(chile_dir)
```

### Omnibus tests

`car::Anova()` gives $G^2$ tests for each term in the submodels for the dichotomies as well as tests for these terms in the combined model.
<!-- DONE: Side-by-side Anova display noted as a potential future package enhancement (e.g., a method that cbinds the Anova tables for two nested models). -->

```{r chile-anova}
Anova(chile_eng)
Anova(chile_dir)
```

## Which tree fits better?

Both nested models and the multinomial model have identical parameter counts (5 predictors × 3 sub-models
= 15 slopes plus 3 intercepts), so AIC is a fair comparison.

```{r chile-aic}
data.frame(
  model   = c("Multinomial", "Engagement tree", "Direction tree"),
  logLik  = c(logLik(chile_multi), logLik(chile_eng), logLik(chile_dir)),
  AIC     = c(AIC(chile_multi),    AIC(chile_eng),    AIC(chile_dir))
)
```

The direction-first tree should fit better (as it does) if `statusquo` is primarily a
predictor of political opinion (Y/N vs. U/A) rather than of engagement. We can
check this by inspecting the `statusquo` coefficients in each sub-model of the
engagement tree: they should be large in `direction` and small in `engage` and
`disengage`.

### Pseudo-R² measures

`RSQ()` provides a more detailed picture, showing how much of the variance in each
dichotomy is accounted for by the predictors.

```{r chile-rsq-eng}
RSQ(chile_eng, include = c("AIC", "n"))
```

```{r chile-rsq-dir}
RSQ(chile_dir, include = c("AIC", "n"))
```

The contrast between the two trees is striking.
In the engagement-first tree, the `direction` sub-model (Yes vs. No, given engagement)
has the highest R² by far, reflecting the dominant role of `statusquo` in
distinguishing supporters from opponents once engagement is decided.
The `engage` and `disengage` sub-models have much lower R², because the
predictors do a poorer job of explaining who opts out of the vote.

In the direction-first tree, the top-level `direction` split (pro vs. anti status quo)
achieves the highest R² of any single sub-model across both trees, consistent
with `statusquo` being primarily a political-orientation predictor.
The two engagement sub-models within each side (`engage_pro`, `engage_ant`) have
lower R², as expected.

The higher Combined R² for the direction-first tree confirms the AIC comparison above:
structuring the tree so that `statusquo` governs the first split yields a
better-fitting and more interpretable model.

## Predicted probabilities

To visualize these models, it is useful to calculate the predicted probabilities of `vote` choice over a range of predictor values,
holding constant (averaging over) the predictor we don't want to display in a given plot
```{r chile-newdata}
new_chile <- data.frame(
  statusquo = seq(-1.5, 1.5, by = 0.25),
  age       = median(chile$age,    na.rm = TRUE),
  sex       = factor("F",  levels = levels(chile$sex)),
  education = factor("S",  levels = levels(chile$education)),
  income    = median(chile$income, na.rm = TRUE)
)
```

We can get the predicted probabilities for these separate models for the dichotomies using `predict.nestedLogit()`
and turning these into data frames for plotting with `ggplot2`.

```{r chile-predict}
pred_eng <- as.data.frame(predict(chile_eng, newdata = new_chile))
pred_dir <- as.data.frame(predict(chile_dir, newdata = new_chile))
```

<!-- DONE: Added geom_ribbon() using se.p for 95% pointwise confidence bands (clamped to [0, 1]). -->

```{r chile-plot, fig.height=5, fig.width=9}
# as.data.frame() returns long format: one row per (observation × response level)
# with predictor columns, plus 'response', 'p', 'se.p', 'logit', 'se.logit'
pred_long <- rbind(
  transform(pred_eng, model = "Engagement-first tree"),
  transform(pred_dir, model = "Direction-first tree")
)

ggplot(pred_long, aes(x = statusquo, y = p, colour = response, fill = response)) +
  geom_ribbon(aes(ymin = pmax(0, p - 1.96 * se.p),
                  ymax = pmin(1, p + 1.96 * se.p)),
              alpha = 0.15, colour = NA) +
  geom_line(linewidth = 1.2) +
  geom_point(color = "blacK") +
  facet_wrap(~ model) +
  labs(x      = "Attitude toward status quo",
       y      = "Predicted probability",
       colour = "Vote", fill = "Vote",
       title  = "Chile 1988: predicted vote probabilities by tree structure") +
  theme_bw()
```

This is substantively interesting, because the two different structures for the dichotomy comparisons
give quite different views of the predictions from the models.

### Plotting logits

Using the `plot.nestedLogit()` method, we can also show these on the logit scale, using the `scale = "logit"` argument.
The `others` argument allows us to condition on (average over) values of the variables not shown in this plot.
There are other niceties, not shown here, such as using direct labels on the curves (`label = TRUE`, `label.x`) rather than a legend.

<!-- DONE: plot() with scale="logit" now works as of commit b523bc7. -->


```{r chile-plot-logit, fig.height=5, fig.width=6}
# Logit-scale plot using the new scale= argument
plot(chile_eng, "statusquo",
     others = list(age       = median(chile$age, na.rm = TRUE),
                   sex       = "F",
                   education = "S",
                   income    = median(chile$income, na.rm = TRUE)),
     scale  = "logit",
     xlab   = "Attitude toward status quo",
     main   = "Engagement-first tree")
```

---

# Travel mode choice (`AER::TravelMode`) {#travel}

The dataset `AER::TravelMode` is a classic in econometrics, originally from @Greene:2003.
It relates to the mode of travel chosen by individuals for travel between
Sydney and Melbourne, Australia.

* Do they choose to go by "car", "air", "train", or "bus"?
* How do variables like cost, waiting time, household income influence that choice?

```{r travel-pkg, include=FALSE}
have_AER <- requireNamespace("AER",   quietly = TRUE)
have_dplyr <- requireNamespace("dplyr", quietly = TRUE)
```

This section requires the `AER` and `dplyr` packages.
<!-- DONE: Added AER to Suggests: in DESCRIPTION. -->

```{r travel-eval, eval=have_AER && have_dplyr}
library(AER)
library(dplyr)
data("TravelMode", package = "AER")
str(TravelMode)
```

## Data structure

`TravelMode` is in **long format**: 840 rows = 210 individuals × 4 travel
alternatives (air, train, bus, car). Variables such as `wait` and `gcost` are
*alternative-specific* — they differ across modes for the same individual.

`nestedLogit()` requires **one row per individual** with the chosen mode as the
response. Only **individual-specific** predictors (`income`, `size`) work here
directly; alternative-specific costs would require a different approach.

```{r travel-reshape, eval=have_AER && have_dplyr}
tm <- TravelMode |>
  filter(choice == "yes") |>
  select(mode, income, size) |>
  mutate(mode = relevel(factor(mode), ref = "car"))

table(tm$mode)
```

## Nesting structure

The canonical nested structure separates *private* (car) from *public* transit,
then within public separates *air* from *ground*, then *train* from *bus*.
This reflects the IIA problem: adding a near-identical alternative (e.g., a
second bus service) should not equally reduce the probability of choosing car.

```{r travel-dichots, eval=have_AER && have_dplyr}
travel_dichots <- logits(
  pvt_pub  = dichotomy("car", public = c("air", "train", "bus")),
  air_grnd = dichotomy("air", ground = c("train", "bus")),
  tr_bus   = dichotomy("train", "bus")
)
travel_dichots
```

## Fitting

```{r travel-fit, eval=have_AER && have_dplyr}
tm_multi  <- multinom(mode ~ income + size, data = tm, trace = FALSE)
tm_nested <- nestedLogit(mode ~ income + size,
                         dichotomies = travel_dichots,
                         data = tm)
summary(tm_nested)
Anova(tm_nested)
```

## Predicted probabilities

```{r travel-predict, eval=have_AER && have_dplyr}
new_tm <- data.frame(income = seq(10, 60, by = 10), size = 1)

# as.data.frame() returns long format; reshape to wide for comparison
pred_tm_nested_long <- as.data.frame(predict(tm_nested, newdata = new_tm))
nested_wide <- reshape(
  pred_tm_nested_long[, c("income", "size", "response", "p")],
  idvar     = c("income", "size"),
  timevar   = "response",
  direction = "wide"
)
names(nested_wide) <- sub("^p\\.", "", names(nested_wide))

pred_tm_multi <- as.data.frame(predict(tm_multi, newdata = new_tm,
                                       type = "probs"))

cat("Nested logit:\n")
cbind(income = new_tm$income, round(nested_wide[, levels(tm$mode)], 3))

cat("Multinomial logit:\n")
cbind(income = new_tm$income, round(pred_tm_multi[, levels(tm$mode)], 3))
```

## Model comparison

```{r travel-aic, eval=have_AER && have_dplyr}
data.frame(
  model  = c("Multinomial", "Nested"),
  logLik = c(logLik(tm_multi), logLik(tm_nested)),
  AIC    = c(AIC(tm_multi),    AIC(tm_nested))
)
```

---

# Fishing mode choice (`mlogit::Fishing`) {#fishing}

Another classic econometric dataset amenable to nested dichotomies treatment concerns the choice by
recreational anglers among different modes of fishing: from a beach or pier, or on a boat, that
could be private or a charter. 

<!-- DONE: Fishing section developed below. `nestedLogit()` can only use the individual-specific
predictor `income`; the choice-specific `price.*` / `catch.*` variables require a full conditional
logit / nested logit framework such as `mlogit`. This section shows the simple nested-dichotomy
fit and notes the limitation. -->

The dataset is from @HerrgesKling:1999; see also @CameronTrivedi:2005 for an econometric treatment
that also considers the cost of fishing by these modes and also the expected catch
associated with each. Models for such response-specific predictors are interesting,
but outside the scope of our nested logit models.


```{r fishing-pkg, include=FALSE}
have_mlogit <- requireNamespace("mlogit", quietly = TRUE)
```

This section requires the `mlogit` package for the dataset.

```{r fishing-load, eval=have_mlogit}
data("Fishing", package = "mlogit")
str(Fishing)
```

## Overview

1182 US recreational anglers each chose one of four fishing modes:
`beach`, `pier`, `boat`, `charter`. The natural nesting separates
*shore-based* from *vessel-based* alternatives:

```
       fishing
      /        \
   shore      vessel
  /     \    /      \
beach  pier boat  charter
```

Shore-based modes share unobserved attributes (no boat required, lower cost
and commitment) that vessel-based modes do not, so IIA is plausible within
but not across groups.

Unlike `AER::TravelMode`, `mlogit::Fishing` is **already in wide format** — one row per
individual — with `mode` indicating the chosen alternative and `income` as the only
individual-specific predictor. The `price.*` and `catch.*` variables are
*alternative-specific* (one column per mode) and cannot be used directly with
`nestedLogit()`, which expects predictors that apply to the individual, not to each
alternative. A full conditional-logit model (e.g., via `mlogit::mlogit()`) would be
needed to exploit those variables.

```{r fishing-dichots, eval=have_mlogit}
fishing_dichots <- logits(
  shore_vessel = dichotomy(shore  = c("beach", "pier"),
                           vessel = c("boat",  "charter")),
  shore_type   = dichotomy("beach", "pier"),
  vessel_type  = dichotomy("boat",  "charter")
)
fishing_dichots
as.tree(fishing_dichots, response = "mode")
```

Fit the model and show the summaries for the dichotomies:
```{r fishing-fit, eval=have_mlogit}
fish_nested <- nestedLogit(mode ~ income,
                           dichotomies = fishing_dichots,
                           data = Fishing)
summary(fish_nested)
Anova(fish_nested)
```

## Plots
Again, the `plot.nestedLogit()` gives an interpretable plot of the response probabilities:
```{r fishing-plot, eval=have_mlogit}
plot(fish_nested, x.var = "income",
     xlab  = "Monthly income (USD)",
     main  = "Fishing mode choice vs. income",
     label=TRUE, label.col="black",
     cex.lab = 1.2)
```

* Fishing from the beach is a low-probability choice, which doesn't vary much with income.
* Fishing from a pier is next overall, followed by charter fishing. Both of these decline with income.
* The tendency to fish from a private boat (yours or friend's) increases dramatically with income.

---

# Arthritis treatment outcome (`vcd::Arthritis`) {#arthritis}

```{r arthritis-pkg, include=FALSE}
have_vcd <- requireNamespace("vcd", quietly = TRUE)
```

<!-- The dataset `vcd::Arthritis` is most commonly analysed  -->

```{r arthritis-load, eval=have_vcd}
data("Arthritis", package = "vcd")
str(Arthritis)
table(Arthritis$Improved)
```

## Overview

84 patients in a double-blind clinical trial received either an active treatment
or a placebo (`Treatment`). They are classified by `Sex` and `Age`.
The outcome `Improved` is *ordinal*, with levels "None", "Some" or "Marked" improvement.

Simpler analyses of these data consider only the dichotomy ("Improved") between 
"None" and the other categories.

```
      Improved
     /        \
  None      Any improvement
            /             \
         Some           Marked
```

Continuation logits are natural here: dichotomy 1 asks whether the patient
improved at all; dichotomy 2 asks, given improvement, whether it was marked
rather than merely some. Treated patients should have higher probabilities at
both transitions.

```{r arthritis-dichots, eval=have_vcd}
arth_dichots <- continuationLogits(c("None", "Some", "Marked"))
arth_dichots
```

```{r arthritis-fit, eval=have_vcd}
arth_nested <- nestedLogit(Improved ~ Treatment + Sex + Age,
                           dichotomies = arth_dichots,
                           data = Arthritis)
summary(arth_nested)
Anova(arth_nested)
```

## Plotting
```{r arthritis-plot, eval=have_vcd}
plot(arth_nested, x.var = "Age",
     others = list(Treatment = "Treated", Sex = "Female"),
     xlab   = "Age",
     main   = "Arthritis: Treatment = Treated, Sex = Female")
```

---

# Smoking status (`MASS::survey`) {#smoking}

```{r survey-load}
data("survey", package = "MASS")
# Drop NAs on Smoke
sv <- survey[!is.na(survey$Smoke), ]
table(sv$Smoke)
```

## Overview

237 Australian university students. `Smoke` is ordinal:
`Never` < `Occas` < `Regul` < `Heavy`.

Continuation logits:
- Dichotomy 1: Never vs. ever smoked
- Dichotomy 2: Occasional vs. regular+
- Dichotomy 3: Regular vs. heavy

```{r smoke-dichots}
smoke_dichots <- continuationLogits(c("Never", "Occas", "Regul", "Heavy"))
smoke_dichots
```

```{r smoke-fit}
smoke_nested <- nestedLogit(Smoke ~ Sex + Age + Exer,
                            dichotomies = smoke_dichots,
                            data = sv)
summary(smoke_nested)
Anova(smoke_nested)
```

```{r smoke-plot}
plot(smoke_nested, x.var = "Age",
     others = list(Sex = "Female", Exer = "Some"),
     xlab   = "Age",
     main   = "Smoking: Female students, some exercise")
```

**Note**: With only 237 observations and modest predictors, results are
illustrative rather than definitive.

---

# Ordinal examples — brief sketches

## Pneumoconiosis severity (`VGAM::pneumo`)

```{r pneumo-pkg, include=FALSE}
have_VGAM <- requireNamespace("VGAM", quietly = TRUE)
```

```{r pneumo-show, eval=have_VGAM}
data("pneumo", package = "VGAM")
pneumo
```

8 grouped observations on coal miners. Response: `normal` / `mild` / `severe`.
Predictor: `duration` (years of dust exposure).  Very small data but the story
is clean — continuation logits on an ordinal severity scale vs. a single
continuous dose predictor.

```{r pneumo-fit, eval=have_VGAM}
# pneumo is aggregated; nestedLogit() needs individual-level data or
# frequency weights — requires expansion or a weighted interface.
# Placeholder: would use continuationLogits(c("normal", "mild", "severe"))
```

## Mental health status (Agresti)

Mental health status (`Well`, `Mild`, `Moderate`, `Impaired`) vs. SES and
life events — Table 9.1 in @Agresti:2002.
May be available via `vcdExtra::Mental` or must be entered from the book.
Classic illustration of continuation logits for an ordinal psychiatric outcome.

---

# Session info

```{r session-info}
sessionInfo()
```

```{r write-bib, echo=FALSE}
pkgs <- unique(c(.to.cite, .packages()))
knitr::write_bib(pkgs, file = here::here("vignettes", "packages.bib"))
```

## References

---
title: "Nested-dichotomies logistic regression models"
author: "Michael Friendly and John Fox"
date: "`r Sys.Date()`"
package: nestedLogit
output: 
  rmarkdown::html_vignette:
    toc: true
    toc_depth: 3
    number_sections: true
fig_caption: yes
bibliography: ["references.bib", "packages.bib"]
csl: apa.csl
vignette: >
  %\VignetteIndexEntry{Nested-dichotomies logistic regression models}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  message = FALSE,
  warning = FALSE,
  fig.align = "center",
  fig.height = 6,
  fig.width = 7,
  fig.path = "fig/",
  dev = "png",
  comment = "#>"
)

# save some typing
knitr::set_alias(w = "fig.width",
                 h = "fig.height",
                 cap = "fig.cap")

# colorize text: use inline as `r colorize(text, color)`
colorize <- function(x, color) {
  if (knitr::is_latex_output()) {
    sprintf("\\textcolor{%s}{%s}", color, x)
  } else if (knitr::is_html_output()) {
    sprintf("<span style='color: %s;'>%s</span>", color,
      x)
  } else x
}


set.seed(47)
.opts <- options(digits = 4)

# packages to be cited here. Code at the end automatically updates packages.bib
.to.cite <- c("nnet", "car", "broom", "ggplot2", "geomtextpath", "ggeffects")

# removed: "equatiomatic" -- now in references.bib
```

Load the packages we'll use here:
```{r setup}
library(nestedLogit)    # Nested Dichotomy Logistic Regression Models
library(knitr)          # A General-Purpose Package for Dynamic Report Generation in R
library(car)            # Companion to Applied Regression
library(nnet)           # Feed-Forward Neural Networks and Multinomial Log-Linear Models
library(broom)          # Convert Statistical Objects into Tidy Tibbles
library(dplyr)          # A Grammar of Data Manipulation
library(effects)        # Effect Displays for Linear, Generalized Linear, and Other Models
```

## Models for polytomous responses

The familiar logistic-regression model applies when there is a binary ("_dichotomous_")
response, such as "survived" vs. "died", or voted "yes" vs. "no" on a referendum.
Often, however, the response variable is multi-category ("_polytomous_"),
taking on $m > 2$ discrete values.
For example, 

* Respondents to a social survey are classified
by their highest completed level of education, taking on the values (1) less than highschool, (2) highschool graduate, (3) some post-secondary, or (4) post-secondary degree.

* Women's labor-force participation is classified as (1) not working outside the home, (2) working part-time, or (3) working full-time.

* Voters in Quebec in a Canadian national election choose one of the (1) Liberal Party, (2) Conservative Party, (3) New Democratic Party, or (4) Bloc Quebecois.

The numbers in these examples, (1), (2), etc., are category labels, and the categories may be ordered (as in the first two examples) or may not (as in the third).

There are several different ways to model the category probabilities for a polytomous response.
Let \(\phi_{ij} \equiv \phi_j \,  ( \vec{x}_i )\) 
be the probability of response $j$ for case
$i$, given the predictors $\vec{x}_i$.
Because \(\sum_j \,  \phi_{ij} = 1\), any \(m - 1\) of
these probabilities imply the last; for example, \(\phi_{im} = 1 - \sum_{j = 1}^{m - 1} \,  \phi_{ij}\).  

The essential idea is to
construct a model for the polytomous 
response composed of $m-1$
logit comparisons among the response categories in a manner analogous to
to the treatment of factors in the predictor variables. There are also more restrictive models specifically for ordered categorical responses, but we will not consider them here.

### Multinomial logit model

One natural generalization of the the standard logistic-regression (or _logit_) model is the _multinomial logit_ (or _generalized logit_) model.
When the polytomous response has $m$ levels, the multinomial logit model comprises
$m-1$ log-odds comparisons with a reference level, typically the first or last, as described in
@Fox:2016:ARA [sec. 14.2.1] and @FriendlyMeyer:2016:DDAR [sec. 8.3]. This is an inessential choice, in that the likelihood under the model and the fitted response probabilities that it produces are unaffected by choice of reference level, much as choice of reference level for dummy regressors created from a factor predictor doesn't affect the fit of a regression model.
The standard implementation of this model in R is `multinom()` in the
**nnet** package [@nnet2002], which takes the first level of the response as the omitted reference category.

### Nested-dichotomies logit model

Because it uses the familiar dichotomous logit model, fitting separate models for each of a hierarchically nested set of binary comparisons among the
response categories, the _nested-dichotomies_ logit model can be a simpler alternative to the multinomial logit model. Standard methods
for model summaries, tests and graphs can then be employed for each of the constituent binary logit models, and
taken together, the set of $m-1$ models comprises a complete
model for the polytomous response, just as the multinomial logit model does. This approach is described by  @Fienberg:80 and
is also discussed by @Fox:2016:ARA [sec. 14.2.2] and @FriendlyMeyer:2016:DDAR [sec. 8.2]. 

For an $m$-category response and a model-matrix with $p$ regressors, both the nested-dichotomies logit model and the multinomial logit model have $p \times (m - 1)$ parameters. The models are not equivalent, however, in that they generally produce different sets of fitted category probabilities and hence different likelihoods. 


By the construction of nested dichotomies, the submodels are 
statistically independent
(because the likelihood for the polytomous response is the product of the likelihoods for
the dichotomies),
so test statistics, such as likelihood ratio ($G^2$) and Wald chi-square tests for
regression coefficients can be summed to give overall tests for the full polytomy.
In this way, the $m-1$ dichotomies are analogous to $m-1$
orthogonal contrasts for an $m$-level factor in a balanced ANOVA design.

Alternative sets of nested dichotomies are illustrated in the figure below, for a four-category polytomous response $Y = \{1, 2, 3, 4\}$. In the case shown at the left of the figure, the response categories
are divided first as $\{1, 2\}$ vs. $\{3, 4\}$. Then these compound categories are subdivided
 as the dichotomies $\{1\}$ vs. $\{2\}$ and as $\{3\}$ vs. $\{4\}$.
Alternatively, as shown at the right of the figure, the response categories
are divided progressively:
first as $\{1\}$ vs. $\{2, 3, 4\}$; 
next as $\{2\}$ vs. $\{3, 4\}$; and
and finally $\{3\}$ vs. $\{4\}$.


```{r}
#| nested,
#| echo = FALSE,
#| out.width="80%",
#| fig.cap = "**Nested dichotomies**: The boxes show two different ways a four-category response can be represented as three nested dichotomies."
knitr::include_graphics("nested.jpg")
```
<!-- knitr::include_graphics(here::here("vignettes", "fig", "nested.jpg")) -->


This example makes clear that nested dichotomies are not unique and that alternative sets of nested dichotomies are
not equivalent: Different choices have different interpretations. Moreover, and more fundamentally, fitted probabilities and hence the likelihood for the nested-dichotomies model depend on how the nested dichotomies are defined. 

The nested-dichotomies model is consequently most compelling when there is a natural and unique way to define the dichotomies, such as a process that proceeds through an orderly sequence of stages. Consider the set of nested dichotomies at the right of the figure above, and the previously mentioned four-level educational response variable with categories (1) less than highschool, (2) highschool graduate, (3) some post-secondary, and (4) post-secondary degree. In the vast majority of cases, individuals proceed through these educational stages in sequence. The first dichotomy, $\{1\}$ vs. $\{2, 3, 4\}$, therefore represents highschool graduation; the second, $\{2\}$ vs. $\{3, 4\}$, enrollment in post-secondary education; and the third, $\{3\}$ vs. $\{4\}$, completion of a post-secondary degree. This scheme for generating the  logits for the nested-dichotomies model is termed _continuation logits_.

To take another example, the figure below shows the classification of psychiatric patients
into four diagnostic categories.  These might be naturally dichotomized by contrasting the normal individuals to the groups of patients, and then dividing the patient groups into
a comparison of depressed and manic patients vs. schizophrenics, followed by
a comparison of depressed vs. manic patients. A model predicting
diagnosis can be interpreted in terms of the probabilities of classification
into each of the response categories. 

```{r}
#| nested-psychiatric,
#| echo = FALSE,
#| out.width="70%",
#| fig.cap = "**Psychiatric classification**: The figure shows how four diagnostic categories might be represented by nested dichotomies."
knitr::include_graphics("nested-psychiatric.png")
```
<!-- knitr::include_graphics(here::here("vignettes", "fig", "nested-psychiatric.png")) -->

## Example: Women's labor-force participation

For a principal example, we consider the data set `Womenlf` from the **carData** package [@R-carData].
The data give the responses of 263 young married women, 21--30 years old, drawn from a 1977 survey carried out by
the York University Institute for Social Research
[@Atkinson-etal:1984]. This example was originally developed by @Fox:1984:LSM [sec. 5.1.5].
The variables in the model are:

* `partic`: labor force participation, the response, with levels:
  + `"fulltime"`: working full-time
  + `"not.work"`: not working outside the home
  + `"parttime"` : working part-time.

* `hincome`: Husband's income, in \$1,000s.
* `children`: Presence of children in the home, `"absent"` or `"present"`.
* `region`: Region of Canada (`r paste(paste0('"', levels(Womenlf$region), '"'), collapse=", ")`).

The response, `partic` is a factor, but the levels are ordered
alphabetically. To facilitate interpretation, we reorder the levels of `partic`:


```{r order-partic}
data(Womenlf, package = "carData")
Womenlf$partic <- with(Womenlf, 
                       factor(partic, levels = c("not.work", "parttime", "fulltime")))
```
In 1977, the majority
of the 263 women in the sample were not working outside the home:
```{r response-distribution}
xtabs(~ partic, data=Womenlf)
```



### Defining nested dichotomies

How can we understand these womens' labor-force participation choices in terms of the explanatory variables? We'll consider three polytomous logit models for the `Womenlf` data, two of which entail different choices of nested dichotomies, and the multinomial logit model. The example will illustrate the potential pitfalls and advantages of the nested-dichotomies approach in a context where there isn't a compelling choice of nested dichotomies.

It is at least arguable to construe a woman's labor-force choice as first involving
a dichotomy (let's call it `work`) between women who are not working outside the home vs. those who are working
(either part-time or full-time). A second dichotomy (`full`) contrasts those who work
full-time time vs. part-time, but among only those who work.

The two binary variables for the nested dichotomies can be created by recoding `partic` 
as follows.  
```{r recode}
Womenlf <- within(Womenlf, {
  work = ifelse(partic == "not.work", 0, 1)
  full = ifelse(partic == "fulltime",  1,
                ifelse(partic == "parttime", 0, NA))
})
```

Note that the complete sample of `r nrow(Womenlf)` cases is available for the `work` dichotomy,
while only `r sum(Womenlf$partic != "not.work")` cases---excluding those not working outside the home---are available for the `full` dichotomy:
```{r dichots-distributions}
xtabs(~ work, data=Womenlf)
xtabs(~ full, data=Womenlf, addNA=TRUE)
```
The relationship of the response variable `partic` to the two nested dichotomies is as follows:

```{r xtabs}
xtabs(~ partic + work, data=Womenlf)
xtabs(~ partic + full, addNA=TRUE, data=Womenlf)
```


We can then fit separate binary logit models to the two nested dichotomies directly:

```{r submodels}
mod.work <- glm(work ~ hincome + children, family=binomial, data=Womenlf)
mod.full <- glm(full ~ hincome + children, family=binomial, data=Womenlf)
```

In equation form, the two log-odds models are shown below.
(Model equations are conveniently rendered in markdown/LaTeX using the `equatiomatic` package [@R-equatiomatic].)
<!-- equatiomatic::extract_eq(mod.work) -->
<!-- equatiomatic::extract_eq(mod.work) -->

$$
L_1 =\log\left[ \frac { P( \operatorname{work} = \operatorname{1} ) }{ 1 - P( \operatorname{work} = \operatorname{1} ) } \right] = \alpha_1 + \beta_{11}(\operatorname{hincome}) + \beta_{12}(\operatorname{children}_{\operatorname{present}})
$$
$$
L_2 = \log\left[ \frac { P( \operatorname{full} ) }{ 1 - P( \operatorname{full} ) } \right] = \alpha_2 + \beta_{21}(\operatorname{hincome}) + \beta_{22}(\operatorname{children}_{\operatorname{present}})
$$
The estimated regression coefficients for the two binary logit models are
```{r direct-estimates}
mod.work
mod.full
```

A disadvantage of this approach is that it is tedious to obtain tests for the combined model, to compute and plot predicted probabilities, and so forth.

### Using `dichotomy()` and `logits()` to define the response

Instead, the **nestedLogit** package provides tools to specify and manipulate the nested-logit model. The `dichotomy()` function
defines a single dichotomy, and the `logits()` function
uses $m-1$ calls to `dichotomy()` to create a `"dichotomies"`
object representing the nested dichotomies. For example:
```{r comparisons}
comparisons <- logits(work=dichotomy("not.work", working=c("parttime", "fulltime")),
                      full=dichotomy("parttime", "fulltime"))

comparisons
```
It is mandatory to name the dichotomies (here `work` and `full`), and we can optionally name the elements of each dichotomy, an option that is particularly useful for a compound category, such as `working=c("parttime", "fulltime")` in the example.

There are coercion functions to convert the set of nested dichotomies to a matrix or to
a character string, representing the tree structure of the dichotomies:

```{r coerce-comparisons}
as.matrix(comparisons)

cat(as.character(comparisons))
```

### Using `nestedLogit()` to fit the model

To fit the model, we supply `comparisons` as the `dichotomies` argument to the `nestedLogit()` function.
The model `formula` argument, `partic ~ hincome + children` specifies a main-effects model
for husband's income and presence of young children; aside from the `dichotomies` argument, the general format of the function call is typical for an R statistical modeling function (with optional `subset` and `contrasts` arguments not shown in this example). An atypical feature of `nestedLogit()` is that the `data` argument is required.
```{r wlf.nested}
wlf.nested <- nestedLogit(partic ~ hincome + children, 
                          dichotomies = comparisons,
                          data=Womenlf)
```

The result, `wlf.nested`, is a an object of class `"nestedLogit"`, encapsulating the details of the model for the nested
dichotomies. The `models` component of the object contains essentially the same `"glm"` model objects as we constructed directly as `mod.work` and `mod.full` above, here named `work` and `full`.

```{r names-etc}
names(wlf.nested)

names(wlf.nested$models) # equivalent: names(models(wlf.models))

# view the separate models
models(wlf.nested, 1) 

models(wlf.nested, 2)
```


### Methods for `"nestedLogit"` objects

```{r child="../man/partials/methods.Rmd"}
```


We illustrate the application of some of these methods:

**Coefficients**: By default, `coef()` returns a matrix whose rows are the regressors in the model and
whose columns represent the nested dichotomies.
In the `Womenlf` example, the coefficients $\widehat{\beta}_{j1}$ give the estimated change in the
log-odds of working vs. not working associated with a \$1,000 increase
in husband's income and with having children present vs. absent, each holding the other constant.
The coefficients $\widehat{\beta}_{j2}$ are similar for the log-odds of working full-time vs. part-time
among those who are working outside the home. The exponentiated coefficients
$e^{\widehat{\beta}_{jk}}$ give multiplicative effects on the odds for these comparisons.
```{r coef}
coef(wlf.nested)

# show as odds ratios
exp(coef(wlf.nested))
```
Thus, the odds of both working and working full-time decrease with husband's income, by about 4% and 10% respectively per $1000.
Having young children also decreases the odds of both working and working full-time, by about 79% and 93% respectively.

**Analysis of deviance**: A method for the `Anova()` function from the **car** package [@R-car]
computes Type II or III likelihood-ratio or Wald tests for each term in the model. Note that the likelihood-ratio or Wald $\chi^2$ and degrees of freedom for the `Combined Responses` is the sum of their values for the separate dichotomies:
```{r Anova}
Anova(wlf.nested)
```

**Linear hypotheses**: The `linearHypothesis()` function in the **car** package provides a very general method for
computing Wald tests of specific hypotheses about individual coefficients in a model or their linear combinations.

For example, the following command tests the hypothesis that the coefficients for `hincome` and
`children` are simultaneously all equal to zero. This is equivalent to the test of the global
null model, $H_0$: all $\beta_{jk} = 0$ for $j = 1,2; k=1,2$ against an alternative that one or more coefficients $\beta_{jk} \ne 0$.

`linearHypothesis()` reports this test for each of the submodels for the dichotomies `work` and `full`, as well as for the combined model:

```{r linearHyp}
linearHypothesis(wlf.nested, c("hincome", "childrenpresent"))
```

**Tidy summaries**: The **broom** package [@R-broom]
provides functions for compact and tidy summaries of fitted models. The `glance()` method for a `"nestedLogit"` model produces a one-line summary of the statistics
for each dichotomy. The `tidy()` method combines the coefficients for the sub-models, together with test statistics:

```{r}
glance(wlf.nested)   # summarize the sub-models

tidy(wlf.nested)     # summarize the coefficients
```


These functions facilitate the construction of custom tables. For example, to extract the likelihood-ratio difference-in-deviance ($G^2$)  tests and compute ($G^2 / df$):
```{r}
gl <- glance(wlf.nested)
gl |> 
  select(response, deviance, df.residual) |> 
  add_row(response = "Combined", deviance = sum(gl$deviance), df.residual = sum(gl$df.residual)) |>
  mutate(
    `P-value` = pchisq(deviance, df.residual, lower.tail = FALSE),
    `$G^2$/df` = deviance / df.residual) |>
  rename(`$G^2$` = deviance,
         df = df.residual) |>
  knitr::kable(digits = 3)
```


**Pseudo-R² measures**: While `Anova()` and `linearHypothesis()` test the statistical significance of individual predictors,
pseudo-R² statistics summarize the overall explanatory power of the model.
The `RSQ()` function computes several such measures for each dichotomy sub-model and for the combined polytomous model.
By default it reports three widely used statistics:

* **McFadden's R²**: $1 - L / L_0$, where $L$ is the fitted model log-likelihood and $L_0$ that of the null (intercept-only) model.
  Values of 0.1–0.3 are generally considered indicative of a reasonable fit in logistic regression [@statisticalhorizons-r2].
* **Cox-Snell R²**: $1 - \exp\!\bigl(2(L_0 - L)/n\bigr)$, a rescaling based on the likelihood ratio; it is bounded strictly below 1.
* **Nagelkerke R²**: Cox-Snell divided by its theoretical maximum, so that the scale runs from 0 to 1.

The Combined row aggregates the sub-model log-likelihoods (valid because the nested dichotomies are statistically independent)
and uses the full sample size $n = `r nrow(Womenlf)`$ rather than the sum of the per-dichotomy observation counts,
which would double-count observations.

```{r wlf-rsq}
RSQ(wlf.nested)
```

The results reveal an interesting asymmetry between the two dichotomies.
The `full` sub-model — contrasting full-time with part-time work, among those who work outside the home — has
substantially higher pseudo-R² values than the `work` sub-model, which contrasts working with not working.
This reflects what we saw in the `Anova()` output: husband's income and presence of young children are very strong
predictors of work _intensity_ among women who are already in the labor force, but are relatively weaker predictors
of the initial decision to work outside the home at all.
The Combined row gives an overall characterization of how well the predictors account for the three-category
response.

**Model updating**: The `update()` function makes it easy to create a new model from an old one, by adding or subtracting terms from the
model formula, specifying a new formula, or changing the observations used or contrasts for factors.

For example, you might ask, "_Does it make sense to include region of Canada in the model for the `Womenlf` data?_"  This question can be answered
by adding `region` to the model formula, and comparing the new model to the original one using
`anova()`. The tests here are for the additional contribution of `region` over and above
the main effects of `hincome` and `children`:

```{r update}
wlf.nested.1 <- update(wlf.nested, formula = . ~ . + region)

anova(wlf.nested, wlf.nested.1)
```

Recall that `anova()` with two or models tests the models sequentially against one another, in the order specified.
This assumes that the models compared are _nested_ (an unintentional pun), in the sense that the
terms in the smaller model in each sequential pair are a subset of those in the larger model.

In a similar manner, we could fit and test a _wider_ scope of models. For example to add
an interaction between husband's income and children and then test the interaction term:

```{r more-models}
#| eval = TRUE
wlf.nested.2 <- update(wlf.nested, formula = . ~ .^2)

anova(wlf.nested, wlf.nested.2)
```

We can see that neither `region` nor the `hincome` $\times$ `children` interaction make a difference to the fit 
either of the sub-models or of the combined model for the three response categories.


### Obtaining fitted values: `predict()` 

By default, `predict()` for a `"nestedLogit"` model object returns a `"predictNestedLogit"`
object, which is a named list of four data
frames whose columns are the response categories:

* `p`: predicted response-category probabilities.
* `logit`: the predicted probabilities transformed to the log-odds (logit) scale, `logit = log(p / (1 - p))`.
* `se.p`: standard errors of the predicted probabilities, computed by the delta method (see the corresponding vignette).
* `se.logit`: standard errors of the logits.

The computation is a bit tricky, because the probabilities of working full-time
or part-time are conditional on working outside the home, but `predict()` takes care of the details.  See `vignette("standard-errors")` for how these are calculated.

```{r}
wlf.pred <- predict(wlf.nested)
print(wlf.pred, n=5)
```
By default, fitted values and standard errors are computed for _all_ the observations in the data set. You can
provide a `newdata` data.frame containing arbitrary combinations of predicted values,
for example to obtain a grid of predicted values for custom plots or other purposes.

```{r}
new <- expand.grid(hincome=seq(0, 45, length=4),
                   children=c("absent", "present"))

wlf.new <- predict(wlf.nested, new)
```


For greater flexibility, the `as.data.frame()` method for `"predictNestedLogit"`
converts the components to long format.

```{r}
as.data.frame(wlf.new) 
```


### Plotting `"nestedLogit"` objects

The **nestedLogit** package includes a basic `plot()` method for `"nestedLogit"` models,
which calculates fitted probabilities and standard errors for the response categories and plots the probabilities and point-wise confidence limits
against a single explanatory variable
on the horizontal axis, while `other` explanatory variables are fixed to particular values.
To produce multi-panel plots, it is necessary to call `plot()` repeatedly for
various levels of the other predictors, and to compose these into a single figure, for example using `par("mfcol")`:

```{r wlf-plot}
#| out.width = "100%",
#| fig.height = 4,
#| fig.cap = "**plot method**: Predicted probabilities and 95 percent pointwise confidence envelopes of not working, working part-time, and working full-time",
#| fig.show = "hold"
op <- par(mfcol=c(1, 2), mar=c(4, 4, 3, 1) + 0.1)
col <- scales::hue_pal()(3)                  # ggplot discrete colors
plot(wlf.nested, "hincome",                  # left panel
     other = list(children="absent"),
     xlab = "Husband's Income", 
     legend.location="top", col = col)
plot(wlf.nested, "hincome",                  # right panel
     other = list(children="present"),
     xlab = "Husband's Income", 
     legend=FALSE, col = col)
par(op)
```

### Effect plots for `"nestedLogit"` models

We provide a `"nestedLogit"` method for the `Effect()` function in the **effects** package [@effects2003; @FoxWeisberg:2019:CAR]. Because `Effect()` is the basic building block for other functions in the **effects** package, such as `predictorEffects()` [@effects2018], the full range of capabilities of the **effects** package is available; in particular, it's possible to produce effect plots similar to those for multinomial logistic regression models [@effects2009].

We illustrate with the nested-logit model fit to the `Womenlf` data set:

```{r wlf-effect-plot-1}
#| out.width = "100%",
#| fig.height = 6,
#| fig.cap = "Default predictor effect plots for the nested-logit model fit to the `Womenlf` data"
plot(predictorEffects(wlf.nested))
```

The panel at the left shows the predictor effect plot for husband's income, that at the right the predictor effect plot for presence of children. 

Because the model is additive, there are no conditioning variables in either plot. Effect plots are generally more interesting in models with interactions, in which case the effect plot for a predictor is conditioned on the other predictors with which the focal predictor interacts (see `?Effect.nestedLogit` for an example). 

In the predictor effect plot for `hincome`, the factor `children` is by default fixed to its distribution in the data; in the predictor effect plot for `children`, the numeric predictor `hincome` is by default fixed to its mean. The band in the graph at the left and vertical bars in the graph at the right give 95\% confidence limits around the fit.

An alternative style of effect plot shows the fit as a stacked-area graph (in this case without confidence limits):
```{r wlf-effect-plot-2}
#| out.width = "100%",
#| fig.height = 4,
#| fig.cap = "Stacked-area predictor effect plots for the nested-logit model fit to the `Womenlf` data"
plot(predictorEffects(wlf.nested),
     axes=list(y=list(style="stacked")),
     lines=list(col=scales::hue_pal()(3)))
```

The computation and display of effect graphs are highly customizable. For details, see the documentation for the **effects** package.

Effect plots for nestedLogit models are also supported in the
**ggeffects** package [@R-ggeffects].  For example, the following
(not run) produces plots of predicted probabilities for the same model with separate panels for the levels of `partic`.

```{r wlf-ggeff}
#| eval=FALSE
ggpredict(wlf.nested, c("hincome[all]", "children")) |> plot()
```


### Alternative models for the `Womenlf` data 

We've mentioned that the nested dichotomies {not working} vs. {part-time, full-time} and {part-time} vs. {full-time} for the `Womenlf` data are largely arbitrary. An alternative set of nested dichotomies first contrasts full-time work with the other categories, {full-time} vs. {not working, part-time}, and then {not working} vs. {part-time}. The rationale is that the real hurdle for young married women to enter the paid labor force is to combine full-time work outside the home with housework. This alternative nested-dichotomies model proves enlightening:
```{r alt-model}
wlf.nested.alt <- nestedLogit(partic ~ hincome + children,
                              logits(full=dichotomy(nonfulltime=c("not.work", "parttime"), "fulltime"),
                                     part=dichotomy("not.work", "parttime")),
                              data=Womenlf)
```

The `Anova()` and `summary()` for this model show that the effects of husband's income and children make a substantial contribution to the `full` model but not to the `part` model:
```{r alt-anova}
Anova(wlf.nested.alt)

summary(wlf.nested.alt)
```

It's apparent that the alternative model produces a simpler description of the data: The predictors husband's income and presence of children affect the decision to work full-time, but not the decision to work part-time among those who aren't engaged in full-time work.

But do the fits of the two models to the data differ? We can compare fitted probabilities under the two specifications:
```{r fit-correlations}
fit1 <- predict(wlf.nested)$p
fit2 <- predict(wlf.nested.alt)$p
diag(cor(fit1, fit2))
mean(as.matrix(abs(fit1 - fit2)))
max(abs(fit1 - fit2))
```
The fitted probabilities are similar but far from the same. 

The two models have the same number of parameters and neither is nested within the other, so a conventional likelihood-ratio test is inappropriate, but we can still compare maximized log-likelihoods under the two models:
```{r compare-logLik}
logLik(wlf.nested)
logLik(wlf.nested.alt)
```
Because the sample sizes and numbers of parameters are the same for the two models, differences in AIC and BIC are just twice the differences in the log-likelihoods; for example:
```{r compare-AIC}
AIC(wlf.nested, wlf.nested.alt)
```
The comparison slightly favors the alternative nested-dichotomies model.

Here's a graph of the alternative model:
```{r wlf-alt-plot}
#| out.width = "100%",
#| fig.height = 4,
#| fig.cap = "Predicted probabilities for the alternative nested-dichotomies model",
#| fig.show = "hold"
op <- par(mfcol=c(1, 2), mar=c(4, 4, 3, 1) + 0.1)
col <- scales::hue_pal()(3)
plot(wlf.nested.alt, "hincome",           # left panel
     others = list(children="absent"), 
     xlab="Husband's Income", 
     legend.location="top", col = col)
plot(wlf.nested.alt, "hincome",           # right panel
     others = list(children="present"),
     xlab="Husband's Income", 
     legend=FALSE, col = col)
par(op)
```
Compare this to the previous graph for the original specification.

It's also of interest to compare the nested-dichotomies models to the multinomial logit model, which, as we explained, treats the response categories symmetrically:
```{r}
wlf.multinom <- multinom(partic ~ hincome + children, data = Womenlf)
summary(wlf.multinom)
logLik(wlf.multinom)
```

Check the relationship between fitted probabilities:
```{r check-corr}
fit3 <- predict(wlf.multinom, type="probs")[, c("not.work", "parttime", "fulltime")]
diag(cor(fit2, fit3))
mean(as.matrix(abs(fit2 - fit3)))
max(abs(fit2 - fit3))
```
As it turns out, the multinomial logit model and the alternative nested-dichotomies model produce nearly identical fits with similar simple interpretations.

```{r write-bib, echo = FALSE}
# write a packages.bib file of the packages (.packages()) that have been used here
pkgs <- unique(c(.to.cite, .packages()))
knitr::write_bib(pkgs, file = here::here("vignettes", "packages.bib"))
```

```{r, include = FALSE}
options(.opts)
```

## References

---
title: "Standard Errors of Fitted Category Probabilities by the Delta Method for the Nested Logit Model"
author: "John Fox"
date: "2023-05-20"
package: nestedLogit
output:
  rmarkdown::html_vignette:
  fig_caption: yes
bibliography: ["references.bib", "packages.bib"]
csl: apa.csl
vignette: >
  %\VignetteIndexEntry{Standard Errors of Fitted Category Probabilities by the Delta Method for the Nested Logit Model}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

<!-- \usepackage{amsmath} -->

This document uses the delta method [@Fox:2021:Primer, sec. 6.3.5] to derive approximations to the variances of estimated probabilities for dichotomous logit models, and from these, for the nested logit model. The standard errors of these estimated probabilities are the square-roots of their respective variances.

## Notation

Let $\psi_j$ represent the probability that the dichotomous response in the $j$th nested dichotomous logit model is $Y_j = 1$ (i.e., a "success"),  $j = 1, \ldots, m - 1$, where $m$ is the number of response categories for the polytomy. Then $1 - \psi_j$ is the probability that $Y_j  = 0$ (i.e., a "failure"). I assume that the regression coefficients and their covariance matrix for each dichotomous logit model are estimated in the usual manner. 

Let $\lambda_j = \log[\psi_j/(1 - \psi_j]$ represent the (estimated) logit (log-odds) for the $j$th dichotomous logit model, with variance $V(\lambda_j)$ (see below).

Let $\phi_k$, $k = 1, \ldots, m$ represent the probability that the polytomous response is $Y = k$. 

Let $\widehat{\psi}_{j}$ and $\widehat{\phi}_k$ represent the estimates of these probabilities. 

In the sequel, which involves only the estimates of these and other parameters,  I'll omit the hats so as to simplify the notation.

In the nested logit model, the polytomous probabilities $\phi_k$ are each products of probabilities  $\psi_j$ or $1 - \psi_j$ for $j \in \mathcal{M}_k \subseteq \left\{ 1, \ldots, m - 1 \right\}$; that is $\mathcal{M}_k$ is the subset of the dichotomous logit models that enter into $\phi_k$. Let $\psi_{j, k_j}$ represent either $\psi_j$ or $1 - \psi_j$, as appropriate for category $k$ of the polytomous response. Then
\begin{equation*}
 \phi_k = \prod_{j \in \mathcal{M}_k} \psi_{j,k_j}
\end{equation*}
for $k = 1, \ldots, m$.

Finally, the individual-category probabilities $\phi_k$ can be converted into logits, $\Lambda_k = \log[\phi_k/(1 - \phi_k)]$. The estimates of these logits should approach asymptotic normality more rapidly than the estimates of the corresponding probabilities.

### An Example

I'll use the following example to illustrate the results in this document: Suppose that we have a three-category response variable $Y$ with categories $A$, $B$, and $C$, and define the two nested dichotomies $Y_1$ coded 0 or 1 for categories $\{A\}$ and $\{B, C\}$, respectively, and $Y_2$ coded 0 and 1 for categories $\{B\}$ and $\{C\}$. Then $\psi_1 = \Pr(Y_1 = 1) = \Pr(\{B, C\})$; $1 - \psi_1 = \Pr(Y_1 = 0) = \Pr(\{A\})$. As well, $\mathcal{M}_A = \{1\}$ and $\mathcal{M}_B = \mathcal{M}_C = \{1, 2\}$. Consequently,
\begin{align*}
	\phi_A &= \psi_{1, A_1} = 1 - \psi_{1}\\
	\phi_B &= \psi_{1, B_1}\psi_{2, B_2} = \psi_{1}(1 - \psi_{2}) \\
	\phi_C &= \psi_{1, C_1}\psi_{2, C_2} = \psi_{1}\psi_{2}
\end{align*}
Here, I abuse the notation slightly in the interest of clarity, using letters rather than numbers for the response categories, so the index of response categories, $k$, takes on the values $A$, $B$, and $C$, rather than 1, 2, and 3.

## Variances of the Estimated Probabilities

### For the Dichotomous Logit Models

The estimated probability of success $\psi_{j}$ for the $j$th dichotomous logit model is
\begin{equation*}
 \psi_{j} = \frac{1}{1 + e^{-\lambda_{j}}}
\end{equation*}
Then
\begin{align*}
	\lambda_{j} &= \alpha^{(j)} + \beta_1^{(j)} x_1 + \cdots + \beta_p^{(j)} x_p \\
	&= \mathbf{x}^T \boldsymbol{\beta}^{(j)}
\end{align*}
is a function of the regression coefficients, where $\mathbf{x}^T = [1, x_1, \ldots, x_p]$ (an arbitrary vector of values of the regressors) and $\boldsymbol{\beta}^{(j)} = [\alpha^{(j)}, \beta_1^{(j)}, \ldots, \beta_p^{(j)}]^T$. The probability of failure is
\begin{equation*}
 1 - \psi_{j} = \frac{1}{1 + e^{\lambda_{j}}}
\end{equation*}
The variance of the logit is $V(\lambda_{j}) = \mathbf{x}^T V(\boldsymbol{\beta}^{(j)}) \mathbf{x}$,


The derivatives of $\psi_{j}$ and $1 - \psi_{j}$ with respect to $\lambda_{j}$ are
\begin{align*}
 \frac{d\psi_{j}}{d\lambda_{j}} &= \frac{e^{-\lambda_{j}}}{\left( 1 + e^{-\lambda_{j}} \right)^2} \\
 \frac{d \left(1 - \psi_{j} \right)}{d\lambda_{j}} &= - \frac{e^{\lambda_{j}}}{\left( 1 + e^{\lambda_{j}} \right)^2}
\end{align*}

By the univariate delta method,
\begin{align*}
 V(\psi_{j}) &\approx \left( \frac{d\psi_{j}}{d\lambda_{j}} \right)^2 V(\lambda_{j}) \\
 &= \left[ \frac{e^{-\lambda_{j}}}{\left( 1 + e^{-\lambda_{j}} \right)^2} \right]^2 V(\lambda_{j})\\
 V(1 - \psi_{j}) &\approx \left[ \frac{d \left(1 - \psi_{j} \right)}{d\lambda_{j}} \right]^2 V(\lambda_{j}) \\
 &= \left[ \frac{e^{\lambda_{j}}}{\left( 1 + e^{\lambda_{j}} \right)^2}\right]^2 V(\lambda_{j}) \\
 &= V(\psi_{j})
\end{align*}

### For the Nested Logit Model

The variances of the estimated response-category probabilities for the polytomous response can be obtained similarly by the multivariate delta method, recognizing that these probabilities are products of the dichotomous probabilities. The result is greatly simplified because the dichotomies are independent, and so the covariance matrix of the estimated dichotomous probabilities is diagonal.

The required derivatives are
\begin{equation*}
 \frac{\partial \phi_k}{\partial \psi_{j, k_j} } = \prod_{j' \in \mathcal{M}_k - \{j\}} \psi_{j', k_j}
\end{equation*}
for $j \in \mathcal{M}_k$ and $k = 1, \ldots, m$. Here, $-$ denotes set difference. Because $V(\psi_{j}) =  V(1 - \psi_{j})$, it's always the case that $V(\psi_{j, k_j}) =  V(\psi_{j})$, and so
\begin{align*}
 V(\phi_k) &\approx \sum_{j \in \mathcal{M}_k} \left( \frac{\partial \phi_k}{\partial \psi_{j, k_j} }  \right)^2 V\left( \psi_{j} \right) \\
 &= \sum_{j \in \mathcal{M}_k} \left( \prod_{j' \in \mathcal{M}_k - \{j\}} \psi_{j', k_j} \right)^2 V\left( \psi_{j} \right)
\end{align*}
for $k = 1, \ldots, m$.

Applying these results to the example, recall, first, that $\mathcal{M}_A = \{1\}$ and so the set for the product $\prod_{j' \in \mathcal{M}_{A} - \{j\}} \phi^{(j', A_j)}$, that is, $j' \in \{1\}-\{1\}$, is empty. In this case, the product is taken = 1, and $V(\phi_A) = V(\psi_{1})$. That makes intuitive sense, because, as noted previously, $\phi_A = 1 - \psi_{1}$.

Proceeding with $B$ and $C$, $\mathcal{M}_B = \mathcal{M}_C = \{1, 2\}$. Consequently, each product $\prod_{j' \in \mathcal{M}_{B} - \{j\}} \psi_{j', B_j}$ and $\prod_{j' \in \mathcal{M}_{C} - \{j\}} \psi_{j', C_j}$ has only one term, for $j' = 2$ in the case of $B$ or $j' = 1$ in the case of $C$:
\begin{align*}
	V(\phi_B) &= \psi_{2, B_2} V(\psi_{1}) + \psi_{1, B_1}  V(\psi_{2}) \\
	&= (1 - \psi_{2}) V(\psi_{1}) + \psi_{1} V(\psi_{2}) \\
	V(\phi_C) &= \psi_{2, C_2} V(\psi_{1}) + \psi_{1, C_1}  V(\psi_{2}) \\
	&= \psi_{2} V(\psi_{1}) + \psi_{1} V(\psi_{2}) \\
\end{align*}

Yet another application of the delta method produces approximate variances for the individual-category logits. The relevant derivative is
\begin{equation*}
 \frac{d\Lambda_k}{d\phi_k} = \frac{1}{\phi_k(1 - \phi_k)}
\end{equation*}
for $k = 1, \ldots, m$, and so
\begin{align*}
 V(\Lambda_k) &\approx \left( \frac{d\Lambda_k}{d\phi_k} \right)^2 V(\phi_k) \\
 &= \left[ \frac{1}{\phi_k(1 - \phi_k)} \right]^2 V(\phi_k)
\end{align*}

## Acknowledgment

I'm grateful to Georges Monette of York University for a close reading of an earlier version of this document, and in particular for his suggested simplification of the notation employed.

## Reference

As befits a model-fitting function, the package defines a nearly complete set of methods for  `"nestedLogit"` objects:

* `print()` and `summary()` print the results for each of the submodels.
* `update()` re-fits the model, allowing changes to the model `formula`, `data`, `subset`, and `contrasts` arguments.
* `coef()` returns the coefficients for the predictors in each dichotomy.
* `vcov()` returns the variance-covariance matrix of the coefficients
* `predict()` computes predicted probabilities for the response categories, either for the cases in the data, which is equivalent to `fitted()`, or for arbitrary combinations of the predictors; the latter is useful for producing plots to aid interpretation.
* `confint()` calculates confidence intervals for the predicted probabilities or predicted logits.
* `as.data.frame()` method for predicted probabilities and logits converts these to long format for use with `ggplot2`.
* `glance()` and `tidy()` are extensions of `broom::glance.glm()` and `broom::tidy.glm()` to obtain compact summaries of a `"nestedLogit"` model object.
* `plot()` provides basic plots of the predicted probabilities over a range of values of the predictor variables.
* `models()` is an extractor function for the binary logit models in the `"nestedLogit"` object
* `Effect()` calculates marginal effects collapsed over some variable(s) for the purpose of making effect plots.

These functions are supplemented by various methods for testing hypotheses about and comparing nested-logit models:

* `anova()` provides analysis-of-deviance Type I (sequential) tests for each dichotomy and for the combined model. When given a sequence of model objects, `anova()` tests the models against one another in the order specified.
* `Anova()` uses `car::Anova()` to provide analysis-of-deviance Type II or III (partial) tests for each dichotomy and for the combined model.
* `linearHypothesis()` uses `car::linearHypothesis()` to compute Wald tests for hypotheses about coefficients or their linear combinations.
* `logLik()` returns the log-likelihood and degrees of freedom for the nested-dichotomies logit model.
* Through `logLik()`, the `AIC()` and `BIC()` functions compute the Akaike and Bayesian information criteria model-comparison statistics.
* `RSQ()` computes a variety of pseudo $R^2$ (McFadden, CoxSnell, Nagelkerke, ...) measures for nested logit models.

---
title: "Displaying nestedLogit Models as LaTeX Equations"
author: "Michael Friendly"
date: "`r Sys.Date()`"
package: nestedLogit
output:
  rmarkdown::html_vignette:
    toc: true
    toc_depth: 2
    number_sections: true
  fig_caption: yes
bibliography: ["references.bib", "packages.bib"]
csl: apa.csl
vignette: >
  %\VignetteIndexEntry{Displaying nestedLogit Models as LaTeX Equations}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  message  = FALSE,
  warning  = FALSE,
  comment  = "#>"
)

.opts <- options(digits = 4)

# packages to be cited in packages.bib
.to.cite <- c("nestedLogit", "equatiomatic", "carData")
```

## Overview

The `equatiomatic` package [@R-equatiomatic] provides a general mechanism for
converting fitted statistical models into LaTeX equations, via the function
`extract_eq()`.  For any model class that has a `broom::tidy()` method,
`extract_eq()` can generate both the symbolic form of the model equation and
a version with fitted coefficient values substituted in.

The `nestedLogit` package [@R-nestedLogit] supports `extract_eq()` for objects
of class `"nestedLogit"` through a dedicated S3 method, `extract_eq.nestedLogit`.
Because a nested logit model is represented internally as a collection of binary
logit sub-models — one for each dichotomy — `extract_eq()` generates a separate
equation for each dichotomy and returns them as a named list.

To use these features, load both packages:

```{r setup}
library(nestedLogit)
library(equatiomatic)
```

## Fitting a nested logit model

We use the `Womenlf` data [@R-carData], which records the labor-force
participation of married women and is included in the `carData` package.
The three-category response `partic` (not working, part-time, full-time) is
decomposed into two nested binary dichotomies:

- **work**: not working vs. working (part-time or full-time)
- **full**: part-time vs. full-time, *among those who work*

```{r model}
data(Womenlf, package = "carData")

comparisons <- logits(
  work = dichotomy("not.work", working = c("parttime", "fulltime")),
  full = dichotomy("parttime", "fulltime")
)

wlf.nested <- nestedLogit(partic ~ hincome + children,
                           dichotomies = comparisons,
                           data = Womenlf)
```

## Symbolic equations

Calling `extract_eq()` on a `"nestedLogit"` object with `submodel = "name"`
returns the equation for a single dichotomy in symbolic (Greek-letter) form,
labeled by its name.  The equation renders automatically in R Markdown and
Quarto documents.

### Work dichotomy (not working vs. working)

```{r eqn-work}
extract_eq(wlf.nested, submodel = "work")
```

### Full-time dichotomy (part-time vs. full-time)

```{r eqn-full}
extract_eq(wlf.nested, submodel = "full")
```

### Logit notation

A new version of `equatiomatic` (v. 0.4.6) implements a `logit_notation` argument to simply the display of the LHS of these
equations as `logit [P()]` rather than `log [P() / 1 - P()]`.

```{r eqn-full-logit}
extract_eq(wlf.nested, submodel = "full", logit_notation = TRUE)
```


## Using `extract_eq()` options

There are a wide variety of options you can pass to `extract_eq()` to control the details of how the equations are rendered
in LaTeX. These include:

* `use_coefs`: Use the model estimates in the equations instead of symbols, a nice way to display a fitted model
* Options for coloring symbols in the equations: `greek_colors`, `var_colors`, `subscript_colors` and others.
* `ital_vars`: 	Logical, defaults to `FALSE`. Should the variable names not be wrapped in the `\operatorname{}` command so they appear in Roman text?


### Equations with fitted coefficients

Passing `use_coefs = TRUE` substitutes the fitted coefficient values into the
equations.

```{r eqn-coef-work}
extract_eq(wlf.nested, use_coefs = TRUE, submodel = "work")
```

```{r eqn-coef-full}
extract_eq(wlf.nested, use_coefs = TRUE, submodel = "full")
```

### Coloring symbols

The `greek_colors` and `var_colors` arguments control the color of the Greek
coefficient symbols and the variable names, respectively.  This can help
distinguish the structural parameters from the predictors when displaying
equations in presentations or documents.

```{r eqn-colors}
extract_eq(wlf.nested,
           greek_colors = "blue",
           submodel = "work")
```

The color arguments accept any R color name or hex code, and can be a vector
to color each symbol differently. `var_colors` needs a named vector of the variable names.

```{r eqn-colors-vec}
extract_eq(wlf.nested,
           greek_colors = c("black", "blue", "blue"),
           var_colors   = c(hincome = "red", children="darkgreen"),
           submodel     = "work")
```

### Equations for individual sub-models

The individual binary logit sub-models (objects of class `"glm"`) can also be
passed directly to `extract_eq()`.  This can be useful when you want to work
with a single dichotomy in isolation.

```{r submodel}
mod.work <- models(wlf.nested, "work")
extract_eq(mod.work)
```

Note that the response is rendered as `..y` — the internal variable name used when fitting the sub-model — rather than as a meaningful label. The `extract_eq()` method for a `"nestedLogit"` model fixes this infelicity.

## Using the raw LaTeX

Each equation returned by `extract_eq()` is an object of class `"equation"` (from
`equatiomatic`), which is a character string containing the LaTeX source.
This renders automatically in R Markdown and Quarto documents.  To access
the raw LaTeX — for example to paste it into a paper or to render it with
another tool such as `katex` — use `as.character()`:

```{r raw-latex}
cat(as.character(extract_eq(wlf.nested, submodel = "work")), "\n\n")

cat(as.character(extract_eq(wlf.nested, submodel = "full")), "\n")
```



```{r write-bib, echo=FALSE}
pkgs <- unique(c(.to.cite, .packages()))
knitr::write_bib(pkgs, file = here::here("vignettes", "packages.bib"))
```

## Alligator food choice: `gators` data

The `gators` data (built into `nestedLogit`) records the primary food choice of
alligators — Other, Fish, or Invertebrates — as a function of body length.
The three-category response is decomposed into two dichotomies using `logits()`:

- **other**: {Other} vs. {Fish, Invertebrates}
- **fish_inv**: {Fish} vs. {Invertebrates}, *among those not eating Other*

```{r gators-model}
data(gators)
gators.dichots <- logits(
  other    = dichotomy("Other", c("Fish", "Invertebrates")),
  fish_inv = dichotomy("Fish", "Invertebrates")
)
gators.dichots

gators.nested <- nestedLogit(food ~ length,
                             dichotomies = gators.dichots,
                             data = gators)
```

Note that the dichotomy name `fish_inv` contains an underscore. Because `_` is
the subscript operator in LaTeX, `extract_eq()` replaces it with `.` in the
displayed equation (the `submodel` argument still uses the original R name).

```{r gators-eqns}
extract_eq(gators.nested, submodel = "other")
extract_eq(gators.nested, submodel = "fish_inv")
```

With fitted coefficients:

```{r gators-eqns-coef}
extract_eq(gators.nested, use_coefs = TRUE, submodel = "other")
extract_eq(gators.nested, use_coefs = TRUE, submodel = "fish_inv")
```

## References

```{r, include=FALSE}
options(.opts)
```

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/Effect.nestedLogit.R
\name{Effect.nestedLogit}
\alias{Effect.nestedLogit}
\title{Effect Displays for Nested Logit Models}
\usage{
\method{Effect}{nestedLogit}(
  focal.predictors,
  mod,
  confidence.level = 0.95,
  fixed.predictors = NULL,
  ...
)
}
\arguments{
\item{focal.predictors}{a character vector of the names of one or more of
the predictors in the model, for which the effect display should be computed.}

\item{mod}{a \code{"nestedLogit"} model object.}

\item{confidence.level}{for point-wise confidence bands around the effects
(the default is \code{0.95}).}

\item{fixed.predictors}{controls the values at which other predictors are fixed;
see \code{\link[effects]{Effect}} for details; if \code{NULL} (the default),
numeric predictors are set to their means, factors to their distribution in the data.}

\item{...}{optional arguments to be passed to the \code{\link[effects]{Effect}} method for
binary logit models (fit by the \code{\link{glm}} function).}
}
\value{
an object of class \code{"effpoly"} (see \code{\link[effects]{Effect}}).
}
\description{
Computes effects (in the sense of the \pkg{effects} package---see, in
particular, \code{\link[effects]{Effect}})---for \code{"nestedLogit"} models, which then
can be used with other functions in the \pkg{effects} package, for example,
\code{\link[effects]{predictorEffects}} and to produce effect plots.
}
\examples{
data("Womenlf", package = "carData")
comparisons <- logits(work=dichotomy("not.work",
                                     working=c("parttime", "fulltime")),
                      full=dichotomy("parttime", "fulltime"))
m <- nestedLogit(partic ~ hincome + children,
                   dichotomies = comparisons,
                   data=Womenlf)
peff.women <- effects::predictorEffects(m)
plot(peff.women)
plot(peff.women, axes=list(y=list(style="stacked")))
summary(peff.women)

dichots <- logits(AB_CD = dichotomy(c("A", "B"), c("C", "D")),
                  A_B   = dichotomy("A", "B"),
                  C_D   = dichotomy("C", "D"))
m.health <- nestedLogit(product4 ~ age + gender*household + position_level,
                        dichotomies = dichots, data = HealthInsurance)
eff.gen.hh <- effects::Effect(c("gender", "household"), m.health,
                              xlevels=list(household=0:7))
eff.gen.hh
plot(eff.gen.hh, axes=list(x=list(rug=FALSE)))
plot(eff.gen.hh, axes=list(x=list(rug=FALSE), 
                           y=list(style="stacked")))
}
\references{
John Fox and Sanford Weisberg (2019). \emph{An R Companion to Applied Regression},
3rd Edition. Sage, Thousand Oaks, CA.

John Fox, Sanford Weisberg (2018). Visualizing Fit and Lack of Fit in Complex
Regression Models with Predictor Effect Plots and Partial Residuals.
\emph{Journal of Statistical Software}, 87(9), 1-27.
}
\seealso{
\code{\link[effects]{Effect}}, \code{\link[effects]{plot.effpoly}},
\code{\link[effects]{predictorEffects}}
}
\author{
John Fox
}
\keyword{regression}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/equatiomatic.R
\name{extract_eq.nestedLogit}
\alias{extract_eq.nestedLogit}
\title{Extract Equations for a \code{"nestedLogit"} Model}
\usage{
extract_eq.nestedLogit(model, submodel = NULL, ...)
}
\arguments{
\item{model}{a \code{"nestedLogit"} object.}

\item{submodel}{character; name of a single dichotomy (e.g. \code{"work"}).
When supplied, a single \code{"equation"} object is returned and renders
directly in a knitr chunk.  When omitted, equations for all dichotomies
are returned as a named list of class \code{"nestedLogit_equations"}.}

\item{\dots}{additional arguments passed to \code{\link[equatiomatic:extract_eq]{equatiomatic::extract_eq()}}.
For example, use \code{use_coefs = TRUE} to display fitted coefficients rather
than symbols; \code{wrap = TRUE} to split the RHS across multiple lines;
\code{greek_colors}, \code{var_colors}, etc. to color the symbols.}
}
\value{
When \code{submodel} is \code{NULL}, an object of class
\code{c("nestedLogit_equations", "list")} containing one \code{"equation"} per
dichotomy, named by the dichotomy.  Individual equations can be extracted
with \verb{$name} or by passing \code{submodel}.  When \code{submodel} is a dichotomy
name, the single corresponding \code{"equation"} object is returned and renders
directly in a knitr chunk.
}
\description{
Provides an \code{\link[equatiomatic:extract_eq]{equatiomatic::extract_eq()}} method for \code{"nestedLogit"} objects
to render the models as LaTeX equations in documents.

It extracts the LaTeX equation for each of the binary logit sub-models
comprising a nested logit model, substituting the dichotomy name for the
internal \code{..y} placeholder used by \code{\link[=nestedLogit]{nestedLogit()}}.
}
\details{
Internally, \code{\link[=nestedLogit]{nestedLogit()}} fits each sub-model on a temporary response
column named \code{..y}, so \code{\link[equatiomatic:extract_eq]{equatiomatic::extract_eq()}} would render the
response as \code{..y} rather than the dichotomy name.  This method replaces
\code{..y} with the name of each dichotomy.

Because \verb{_} is the subscript operator in LaTeX, any underscores in
dichotomy names are replaced by \code{.} in the displayed equation
(e.g., a dichotomy named \code{fish_inv} appears as \code{fish.inv}).
This does not affect the names of the returned list, which retain the
original R names.
}
\examples{
data("Womenlf", package = "carData")
wlf.nested <- nestedLogit(partic ~ hincome + children,
                 dichotomies = logits(work = dichotomy("not.work",
                                                       working = c("parttime", "fulltime")),
                                      full = dichotomy("parttime", "fulltime")),
                 data = Womenlf)
if (requireNamespace("equatiomatic", quietly = TRUE)) {
  equatiomatic::extract_eq(wlf.nested)
}

}
\seealso{
\code{\link[equatiomatic:extract_eq]{equatiomatic::extract_eq()}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/as.tree.R
\name{as.tree}
\alias{as.tree}
\alias{as.tree.dichotomies}
\alias{as.tree.continuationDichotomies}
\title{Display the Tree Structure of Nested Dichotomies}
\usage{
as.tree(x, ...)

\method{as.tree}{dichotomies}(x, response = NULL, lobstr = FALSE, ...)

\method{as.tree}{continuationDichotomies}(x, response = NULL, lobstr = FALSE, ...)
}
\arguments{
\item{x}{A \code{"dichotomies"} or \code{"continuationDichotomies"} object.}

\item{\dots}{additional arguments (currently unused).}

\item{response}{Optional character string giving the name of the response
variable, used as the root label of the tree. If \code{NULL} (default),
the root is labeled \code{"(response)"}.}

\item{lobstr}{Logical. If \code{FALSE} (default), renders a 2-D ASCII tree
with \code{/} and \code{\\} branch connectors, with each node centered
above its two children. If \code{TRUE}, builds a nested list
representation of the tree and renders it via \code{\link[lobstr]{tree}},
which must be installed.}
}
\value{
Invisibly returns \code{x}; called for its side effect of printing.
}
\description{
Display the nested structure of a \code{"dichotomies"} or
\code{"continuationDichotomies"} object as a 2-D ASCII tree diagram showing
how the response categories are split at each level of the nesting.
}
\details{
The flat list of dichotomies is reconstructed into a binary tree by matching
each dichotomy's domain (the union of its two sides) to the multi-level
groups produced by earlier splits.  Branch labels are taken from the named
arguments to \code{\link{dichotomy}} when present, and are otherwise
generated automatically as \code{\{level1, level2, ...\}}.
}
\examples{
## Womenlf: named group on one branch
comparisons <- logits(work = dichotomy("not.work",
                                       working = c("parttime", "fulltime")),
                      full = dichotomy("parttime", "fulltime"))
as.tree(comparisons, response = "partic")

## GSS: continuation logits for ordered education levels
cont <- continuationLogits(c("<highschool", "highschool",
                             "college", "graduate"))
as.tree(cont, response = "degree")

## gators data: Food choice
# create dichotomies
gators.dichots <- logits(d1=dichotomy("Other", c("Fish", "Invertebrates")),
                         d2=dichotomy("Fish", "Invertebrates"))
as.tree(gators.dichots, response = "Food")
as.tree(gators.dichots, response = "Food", lobstr = TRUE)

}
\seealso{
\code{\link{logits}}, \code{\link{continuationLogits}},
\code{\link{print.dichotomies}}
Other conversions:
\code{\link{as.matrix.dichotomies}}, \code{\link{as.character.dichotomies}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/RSQ.R
\name{RSQ}
\alias{RSQ}
\alias{RSQ.nestedLogit}
\alias{print.RSQ.nestedLogit}
\alias{RSQ.multinom}
\alias{print.RSQ.multinom}
\alias{RSQ.polr}
\alias{print.RSQ.polr}
\title{Pseudo-R² Measures for Nested Logit and Related Models}
\usage{
RSQ(x, ...)

\method{RSQ}{nestedLogit}(
  x,
  which = c("McFadden", "CoxSnell", "Nagelkerke"),
  include = "AIC",
  digits = 3L,
  ...
)

\method{print}{RSQ.nestedLogit}(x, digits = attr(x, "digits"), ...)

\method{RSQ}{multinom}(
  x,
  which = c("McFadden", "CoxSnell", "Nagelkerke"),
  include = "AIC",
  digits = 3L,
  ...
)

\method{print}{RSQ.multinom}(x, digits = attr(x, "digits"), ...)

\method{RSQ}{polr}(
  x,
  which = c("McFadden", "CoxSnell", "Nagelkerke"),
  include = "AIC",
  digits = 3L,
  ...
)

\method{print}{RSQ.polr}(x, digits = attr(x, "digits"), ...)
}
\arguments{
\item{x}{a \code{"nestedLogit"} object.}

\item{...}{currently unused.}

\item{which}{character vector naming the pseudo-R² measures to compute.
Any subset of \code{c("McFadden", "McFaddenAdj", "CoxSnell", "Nagelkerke", "Tjur")},
or \code{"ALL"} to include all of them.
Default: \code{c("McFadden", "CoxSnell", "Nagelkerke")}.}

\item{include}{character vector of additional columns to append to the result.
Any subset of \code{c("AIC", "BIC", "n")}, where \code{"n"} adds the
number of observations used for each row, or \code{"ALL"} to include all of them.
Default: \code{"AIC"}.}

\item{digits}{integer; number of decimal places used when printing
(default \code{3L}).}
}
\value{
An object of class \code{c("RSQ.nestedLogit", "data.frame")} with one
row per dichotomy plus a final \code{"Combined"} row, and columns
\code{response} (the sub-model name), the requested pseudo-R² measures,
and any additional statistics requested via \code{include}.
The \code{formula}, object name, and \code{digits} are stored as attributes
and used by the \code{print} method.
}
\description{
Computes pseudo-R² and related fit measures for a \code{"nestedLogit"} object
and related models for a polytomous response. For the \code{"nestedLogit"} case,
the result shows
one row per binary logit sub-model (dichotomy) and an additional
\code{"Combined"} row for the overall polytomous model.
}
\details{
\code{RSQ} is implemented as an S3 generic with methods for \code{"nestedLogit"}, as well as
\code{nnet::multinom()}, and \code{MASS::polr()} objects, which are other methods for
modeling a polytomous response variable.

In contrast to standard, Gaussian linear models, where \eqn{R^2} has a uniformly simple interpretation as
"variance accounted for" by the model, and with different, yet \emph{equivalent} computational formulas,
there is no single commonly accepted measure for logistic regression models for a binary response or
a dichotomy among outcomes.

The following measures are available via the \code{which} argument:
\describe{
\item{\code{"McFadden"}}{1 - L/L\eqn{_0}, where L is the fitted model
log-likelihood and L\eqn{_0} that of the null (intercept-only) model
(McFadden, 1979).  Values of 0.1--0.3 indicate a reasonable fit in
logistic regression.}
\item{\code{"McFaddenAdj"}}{1 - (L - k)/L\eqn{_0}, where k is the number
of non-intercept parameters; penalises model complexity (Hosmer &
Lemeshow, 2000).}
\item{\code{"CoxSnell"}}{1 - exp(2(L\eqn{_0} - L)/n); bounded strictly
below 1 for discrete outcomes (Cox & Snell, 1989).}
\item{\code{"Nagelkerke"}}{Cox-Snell divided by its theoretical maximum,
rescaling to [0, \1] (Nagelkerke, 1991).}
\item{\code{"Tjur"}}{Mean fitted value for \eqn{y = 1} minus mean fitted
value for \eqn{y = 0}; the coefficient of discrimination (Tjur, 2009).
Per-dichotomy only (\code{NA} in the Combined row).}
}

For the \strong{Combined} row the log-likelihood is the sum of the sub-model
log-likelihoods (exploiting the independence of the nested dichotomies), and
\eqn{n} is \code{nrow(x$data)} --- the full sample size of the polytomous model ---
not the sum of per-dichotomy observation counts, which would double-count
observations that appear in more than one sub-model.

A wider range of pseudo-R² measures for logistic-type models (\code{glm},
\code{polr}, \code{multinom}, \code{vglm}) is available in
\code{\link[DescTools:PseudoR2]{DescTools::PseudoR2()}}, including the Efron (1978) and
McKelvey & Zavoina (1975) measures not implemented here.
For an accessible overview see \url{https://statisticalhorizons.com/r2logistic/}.
}
\examples{
data("Womenlf", package = "carData")
wlf.nested <- nestedLogit(partic ~ hincome + children,
  logits(work = dichotomy("not.work", c("parttime", "fulltime")),
         full = dichotomy("parttime", "fulltime")),
  data = Womenlf)

# Default: McFadden, CoxSnell, Nagelkerke + AIC
RSQ(wlf.nested)

# All measures and all extra columns
RSQ(wlf.nested, which = "ALL", include = "ALL")

# Multinomial logit for comparison
if (requireNamespace("nnet", quietly = TRUE)) {
  wlf.multi <- nnet::multinom(partic ~ hincome + children, data = Womenlf,
                              trace = FALSE)
  RSQ(wlf.multi)
}

# Proportional-odds model for comparison
if (requireNamespace("MASS", quietly = TRUE)) {
  wlf.polr <- MASS::polr(partic ~ hincome + children, data = Womenlf)
  RSQ(wlf.polr)
}

}
\references{
Cox, D. R., & Snell, E. J. (1989). \emph{The Analysis of Binary Data} (2nd ed.).
Chapman and Hall.

Efron, B. (1978). Regression and ANOVA with zero-one data: Measures of
residual variation. \emph{Journal of the American Statistical Association},
\emph{73}(361), 113--121. \doi{10.2307/2286498}

Hosmer, D. W., & Lemeshow, S. (2000). \emph{Applied Logistic Regression}
(2nd ed.). Wiley. \doi{10.1002/0471722146}

McFadden, D. (1979). Quantitative methods for analysing travel behaviour of
individuals: Some recent developments. In D. A. Hensher & P. R. Stopher
(Eds.), \emph{Behavioural Travel Modelling} (pp. 279--318). Croom Helm.

McKelvey, R. D., & Zavoina, W. (1975). A statistical model for the analysis
of ordinal level dependent variables. \emph{Journal of Mathematical Sociology},
\emph{4}(1), 103--120. \doi{10.1080/0022250X.1975.9989847}

Nagelkerke, N. J. D. (1991). A note on a general definition of the
coefficient of determination. \emph{Biometrika}, \emph{78}(3), 691--692.
\doi{10.1093/biomet/78.3.691}

Tjur, T. (2009). Coefficients of determination in logistic regression
models --- a new proposal: The coefficient of discrimination.
\emph{The American Statistician}, \emph{63}(4), 366--372.
\doi{10.1198/tast.2009.08210}
}
\seealso{
\code{\link[=nestedLogit]{nestedLogit()}}, \code{\link[broom:glance]{broom::glance()}},
\code{\link[DescTools:PseudoR2]{DescTools::PseudoR2()}},
\code{\link[nnet:multinom]{nnet::multinom()}}, \code{\link[MASS:polr]{MASS::polr()}}
}
\author{
Michael Friendly
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/nested.R
\name{nestedLogit}
\alias{nestedLogit}
\alias{logits}
\alias{dichotomy}
\alias{continuationLogits}
\title{Binary Logit Models for Nested Dichotomies}
\usage{
nestedLogit(formula, dichotomies, data, subset = NULL, contrasts = NULL, ...)

logits(...)

dichotomy(...)

continuationLogits(levels, names, prefix = "above_")
}
\arguments{
\item{formula}{a model formula with the polytomous response on the left-hand side
and the usual linear-model-like specification on the right-hand side.}

\item{dichotomies}{specification of the logits for the nested dichotomies,
constructed by the \code{logits} and \code{dichotomy} functions,
or \code{continuationLogits}. Alternatively, the \code{dichotomies}
can be specified as a nested (i.e., recursive) list, the elements of
which can be given optional names. See Details.}

\item{data}{a data frame with the data for the model; unlike in most statistical
modeling functions, the \code{data} argument is required. Cases with \code{NA}s
in any of the variables appearing in the model formula will be removed
with a Note message.}

\item{subset}{a character string specifying an expression to fit the model
to a subset of the data; the default, \code{NULL}, uses the full data set.}

\item{contrasts}{an optional list of contrast specification for specific factors in the
model; see \code{\link{lm}} for details.}

\item{\dots}{for \code{nestedLogit}, optional named arguments to be passed to \code{\link{glm}};
for \code{logits}, definitions of the nested logits---with each named argument specifying
a dichotomy; for \code{dichotomy}, two character vectors giving the levels
defining the dichotomy; the vectors can optionally be named.}

\item{levels}{A character vector of set of levels of the variables or a number
specifying the numbers of levels (in which case, uppercase letters will be
use for the levels).}

\item{names}{Names to be assigned to the dichotomies; if absent, names
will be generated from the levels.}

\item{prefix}{a character string (default: \code{"above_"}) used as a prefix to the names of the continuation dichotomies.}
}
\value{
\code{nestedLogit} returns an object of class \code{"nestedLogit"} containing
the following elements:
\itemize{
\item \code{models}, a named list of (normally) \eqn{m - 1} \code{"glm"} objects,
each a binary logit model for one of the \eqn{m - 1} nested dichotomies representing
the \eqn{m}-level response.
\item \code{formula}, the model formula for the nested logit models.
\item \code{dichotomies}, the \code{"dichotomies"} object defining the nested dichotomies
for the model.
\item \code{data.name}, the name of the data set to which the model is fit, of class \code{"name"}.
\item \code{data}, the data set to which the model is fit.
\item \code{subset}, a character representation of the \code{subset} argument or
\code{"NULL"} if the argument isn't specified.
\item \code{contrasts}, the \code{contrasts} argument or \code{NULL} if the argument
isn't specified.
\item \code{contrasts.print} a character representation of the \code{contrasts} argument or
\code{"NULL"} if the argument isn't specified.
}
\code{logits} and \code{continuationLogits} return objects of class \code{"dichotomies"}
and \code{c("continuationDichotomies" "dichotomies")}, respectively, which are two-elements lists,
each element containing a list of two character vectors representing a dichotomy.
\code{dichotomy} returns a list of two character vectors representing a dichotomy.
}
\description{
Fit a related set of binary logit models via the \code{\link{glm}}
function to nested dichotomies, comprising a model for the polytomy.
A polytomous response with \eqn{m} categories can be analyzed using
\eqn{m-1} binary logit comparisons.  When these comparisons are nested,
the \eqn{m-1} sub-models are statistically independent. Therefore,
the likelihood chi-square statistics for the sub-models are additive
and give overall tests for a model for the polytomy.
This method was introduced by Fienberg (1980),and subsequently illustrated by
Fox(2016) and Friendly & Meyer (2016).

\code{dichotomy} and \code{logits} are helper functions to construct the dichotomies.

\code{continuationLogits} constructs a set of \eqn{m-1} logit comparisons, called
continuation logits,
for an ordered response. With \eqn{m=4} levels, say, \code{A, B, C, D},
considered low to high:
The first contrasts \code{B, C, D} against \code{A}.
The second ignores \code{A} and contrasts \code{C, D} against \code{B}.
The second ignores \code{A, B} and contrasts \code{D} against \code{C}.
}
\details{
A \emph{dichotomy} for a categorical variable is a comparison of one subset
of levels against another subset. A set of dichotomies is \emph{nested}, if after
an initial dichotomy, all subsequent ones are \emph{within} the groups of levels
lumped together in earlier ones. Nested dichotomies correspond to a binary tree
of the successive divisions.

For example, for a 3-level response, a first
dichotomy could be \code{ {A}, {B, C}} and then the second one would be
just \code{{B}, {C}}. Note that in the second dichotomy, observations
with response \code{A} are treated as \code{NA}.

The function \code{dichotomy} constructs a \emph{single} dichotomy in the required form,
which is a list of length 2 containing two character vectors giving the levels
defining the dichotomy. The function \code{logits} is used to create the
set of dichotomies for a response factor. Alternatively, the nested dichotomies can be
specified more compactly as a nested (i.e., recursive) list with optionally named
elements; for example,
\code{list(air="plane", ground=list(public=list("train", "bus"), private="car"))}.

The function \code{continuationLogits} provides a
convenient way to generate all dichotomies for an ordered response.
For an ordered response with \eqn{m=4} levels, say, \code{A, B, C, D},
considered low to high:
The dichotomy first contrasts \code{B, C, D} against \code{A}.
The second ignores \code{A} and contrasts \code{C, D} against \code{B}.
The second ignores \code{A, B} and contrasts \code{D} against \code{C}.
}
\examples{
data("Womenlf", package = "carData")

  #' Use `logits()` and `dichotomy()` to specify the comparisons of interest
  comparisons <- logits(work=dichotomy("not.work",
                                       working=c("parttime", "fulltime")),
                        full=dichotomy("parttime", "fulltime"))
  print(comparisons)

  m <- nestedLogit(partic ~ hincome + children,
                   dichotomies = comparisons,
                   data=Womenlf)
  print(summary(m))
  print(car::Anova(m))
  coef(m)

  # equivalent;
  nestedLogit(partic ~ hincome + children,
              dichotomies = list("not.work",
                                 working=list("parttime", "fulltime")),
              data=Womenlf)

  # get predicted values
  new <- expand.grid(hincome=seq(0, 45, length=10),
                     children=c("absent", "present"))
  pred.nested <- predict(m, new)

  # plot
  op <- par(mfcol=c(1, 2), mar=c(4, 4, 3, 1) + 0.1)
  plot(m, "hincome", list(children="absent"),
       xlab="Husband's Income", legend=FALSE)
  plot(m, "hincome", list(children="present"),
       xlab="Husband's Income")
  par(op)


  continuationLogits(c("none", "gradeschool", "highschool", "college"))
  continuationLogits(4)


}
\references{
S. Fienberg (1980). \emph{The Analysis of Cross-Classified Categorical Data},
2nd Edition, MIT Press, Section 6.6.

J. Fox (2016), \emph{Applied Linear Regression and Generalized Linear Models}, 3rd Edition, Sage,
Section 14.2.2.

J. Fox and S. Weisberg (2011), \emph{An R Companion to Applied Regression}, 2nd Edition, Sage, Section 5.8.

M. Friendly and D. Meyers (2016), \emph{Discrete Data Analysis with R}, CRC Press,
Section 8.2.
}
\seealso{
\code{\link{nestedMethods}}
}
\author{
John Fox
}
\keyword{regression}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/gators.R
\docType{data}
\name{gators}
\alias{gators}
\title{Alligator Food Choice}
\format{
A data frame with 59 rows and 2 columns.
\describe{
\item{food}{Primary food type found in the alligator's stomach, a factor with
levels \code{"Other"}, \code{"Fish"}, and \code{"Invertebrates"}.}
\item{length}{Length of the alligator in meters, a numeric vector.}
}
}
\source{
Agresti, A. (1996). \emph{An Introduction to Categorical Data Analysis}. Wiley.
}
\usage{
data("gators", package = "nestedLogit")
}
\description{
Agresti (1996, p. 207) gives this data on 59 alligators sampled
from a lake in Florida. It has the length of the alligator in meters and the
primary food type found in the alligator's stomach. The food type was classified
into three categories: "Fish", "Invertebrates", and "Other".

Of interest is whether or not the length of an alligator is associated with the
primary food type. Does knowing the length of an alligator give us some indication
about its primary food type? If so, how is length associated with the choice of
food type?
}
\examples{
data(gators)
table(gators$food)
# average length of gators by food
with(gators, tapply(length, food, mean))

# create dichotomies
dichot <- logits(d1=dichotomy("Other", c("Fish", "Invertebrates")),
                 d2=dichotomy("Fish", "Invertebrates"))

gators.nested <- nestedLogit(food ~ length,
                            dichotomies = dichot,
                            data = gators)
car::Anova(gators.nested)

# use the plot method
plot(gators.nested, x.var = "length")
}
\references{
An example using this from \url{https://data.library.virginia.edu/getting-started-with-multinomial-logit-models/}.
}
\seealso{
\code{\link{nestedLogit}}.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/HealthInsurance.R
\docType{data}
\name{HealthInsurance}
\alias{HealthInsurance}
\title{Choice of Health Insurance Product}
\format{
A data frame with 1448 rows and 7 columns.
\describe{
\item{product}{Choice among three products, a factor with levels \code{"A"}, \code{"B"},
and \code{"C"}.}
\item{product4}{Choice among four products, a factor with levels \code{"A"}, \code{"B"},
\code{"C"}, and \code{"D"}.}
\item{age}{The age of the individual, in years.}
\item{household}{The number of people living with the individual in the
same household.}
\item{position_level}{Position level in the company at the time the choice was made,
where 1 is is the lowest level and 5 is the highest, a numeric vector.}
\item{gender}{The gender of the individual, a factor with levels \code{"Female"}
and \code{"Male"}.}
\item{absent}{The number of days the individual was absent from work in the year
prior to the choice,}
}
}
\source{
Originally taken from McNulty, K. (2022).
\emph{Handbook of Regression Modeling in People Analytics},
\url{https://peopleanalytics-regression-book.org/data/health_insurance.csv}.
}
\usage{
data("HealthInsurance", package = "nestedLogit")
}
\description{
A company recently introduced a new health insurance provider for
its employees. At the beginning of the year the employees had to choose one of
three (or four) different health plan products from this provider to best suit
their needs.

This dataset was modified from its original source (McNulty, 2022) for the
present purposes by adding a fourth choice, sampled randomly from the original three.
}
\examples{
lbinary <- logits(AB_CD = dichotomy(c("A", "B"), c("C", "D")),
                  A_B   = dichotomy("A", "B"),
                  C_D   = dichotomy("C", "D"))
as.matrix(lbinary)
health.nested <- nestedLogit(product4 ~ age  + gender * household + position_level,
                             dichotomies = lbinary, data = HealthInsurance)
                             car::Anova(health.nested)
coef(health.nested)
}
\seealso{
\code{\link{nestedLogit}}.
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/nestedHypotheses.R
\name{nestedHypotheses}
\alias{nestedHypotheses}
\alias{Anova.nestedLogit}
\alias{print.Anova.nestedLogit}
\alias{anova.nestedLogit}
\alias{print.anova.nestedLogit}
\alias{logLik.nestedLogit}
\alias{linearHypothesis.nestedLogit}
\title{Hypothesis-Testing and Related Methods for \code{"nestedLogit"} Objects}
\usage{
\method{Anova}{nestedLogit}(mod, ...)

\method{print}{Anova.nestedLogit}(x, ...)

\method{linearHypothesis}{nestedLogit}(model, ...)

\method{anova}{nestedLogit}(object, object2, ...)

\method{print}{anova.nestedLogit}(x, ...)

\method{logLik}{nestedLogit}(object, ...)
}
\arguments{
\item{\dots}{arguments to be passed down. In the case of \code{linearHypothesis},
the second argument is typically the \code{hypothesis.matrix}. See the
Details section of \code{\link[car]{linearHypothesis}}. In the case of \code{anova},
additional sequential \code{"nestedLogit"} models.}

\item{x, object, object2, mod, model}{in most cases, an object of class \code{"nestedLogit"}.}
}
\value{
\itemize{
\item The \code{Anova} and \code{anova} methods return objects
of class \code{"Anova.nestedLogit"} and \code{"anova.nestedLogit"}, respectively,
each of which contains a list of \code{"anova"} objects (see \code{\link{anova}})
and is usually printed.
\item The \code{linearHypothesis} method is called for its side effect, printing
the result of linear hypothesis tests, and invisibly returns \code{NULL}.
\item The \code{logLik} method returns an object of class \code{"logLik"} (see \code{\link{logLik}}).
}
}
\description{
Various methods for testing hypotheses about nested logit models.
\describe{
\item{\code{Anova}}{Calculates type-II or type-III analysis-of-variance tables for \code{"nestedLogit"} objects;
see \code{\link[car]{Anova}} in the \pkg{car} package.}
\item{\code{anova}}{Computes sequential analysis of variance (or deviance) tables for one or more fitted
\code{"nestedLogit"}  objects; see \code{\link{anova}}.}
\item{\code{linearHypothesis}}{Computes Wald tests for linear hypotheses;
see \code{\link[car]{linearHypothesis}} in the \pkg{car} package.}
\item{\code{logLik}}{Returns the log-likelihood and degrees of freedom for the nested-dichotomies model.
(and through it \code{\link{AIC}} and \code{\link{BIC}} model-comparison statistics).}
}
}
\examples{
# define continuation dichotomies for level of education
cont.dichots <- continuationLogits(c("<highschool",
                                     "highschool",
                                     "college",
                                     "graduate"))
# fit a nested model for the GSS data examining education degree in relation to parent & year
m <- nestedLogit(degree ~ parentdeg + year,
                 cont.dichots,
                 data=GSS)

# Anova and anova tests
car::Anova(m) # type-II (partial) tests

anova(update(m, . ~ . - year), m) # model comparison

# Wald test
car::linearHypothesis(m, c("parentdeghighschool", "parentdegcollege",
                           "parentdeggraduate"))

# log-liklihood, AIC, and BIC
logLik(m)
AIC(m)
BIC(m)
}
\seealso{
\code{\link[car]{Anova}}, \code{\link{anova}},
\code{\link[car]{linearHypothesis}}, \code{\link{logLik}}, \code{\link{AIC}},
\code{\link{BIC}}
}
\author{
John Fox
}
\keyword{regression}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/GSS.R
\docType{data}
\name{GSS}
\alias{GSS}
\title{Data From the U.S. General Social Survey 1972-2016}
\format{
A data frame with 44091 rows and 3 columns.
\describe{
\item{parentdeg}{A factor representing parents' attained level of education
(highest "degree" obtained), recording
the higher of mother's and father's education, with levels \code{"<highschool"},
\code{"highschool"}, \code{"college"}, and \code{"graduate"}.}
\item{degree}{The respondent's level of education, a factor with the same levels
as \code{parentdeg}.}
\item{year}{The year of the survey, between \code{1972} and
\code{2016}.}
}
}
\source{
General Social Survey, NORC, The University of Chicago
https://www.norc.org/Research/Projects/Pages/general-social-survey.aspx.
}
\usage{
data("GSS", package = "nestedLogit")
}
\description{
This data set is drawn from the U.S. General Social Survey (GSS) for years
between 1972 and 2016.
}
\examples{
round(100*with(GSS, prop.table(table(degree, parentdeg), 2)))
m.GSS <- nestedLogit(degree ~ parentdeg*year,
                     continuationLogits(c("<highschool",  "highschool",
                                           "college", "graduate")),
                     data=GSS)
car::Anova(m.GSS)
summary(m.GSS)

# plot fitted probabilities
plot(m.GSS, x.var = "year",
            others = list(parentdeg = "<highschool"),
            lty = 1,
            label = TRUE)
plot(m.GSS, x.var = "year",
            others = list(parentdeg = "graduate"),
            lty = 1,
            label = TRUE)
}
\seealso{
\code{\link{nestedLogit}}.
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/as.data.frame.predict.R
\name{as.data.frame.predictNestedLogit}
\alias{as.data.frame.predictNestedLogit}
\title{Convert a Predicted Objects to a data.frame}
\usage{
\method{as.data.frame}{predictNestedLogit}(x, row.names = NULL, optional = FALSE, ...)
}
\arguments{
\item{x}{a \code{"predictNestedLogit"} object}

\item{row.names}{row.names for result (for conformity with generic; not currently used)}

\item{optional}{logical. If TRUE, setting row names and converting column names
(to syntactic names: see \code{\link[base]{make.names}} is optional}

\item{...}{other arguments (unused)}
}
\value{
\itemize{
\item For \code{predict(\dots, model="nested")} (the default), returns
a data frame containing the values of predictors along with the columns
\code{response}, \code{p}, \code{se.p}, \code{logit}, \code{se.logit}.
\item For \code{predict(\dots, model="dichotomies")}, returns
a data frame containing the values of predictors along with the columns
\code{response}, \code{logit}, and \code{se.logit}.
}
}
\description{
These functions provide simple ways to convert the results of \code{\link{predict.nestedLogit}}
to a data frame in a consistent format for plotting and other actions.
}
\examples{
data("Womenlf", package = "carData")
comparisons <- logits(work=dichotomy("not.work", c("parttime", "fulltime")),
                     full=dichotomy("parttime", "fulltime"))

wlf.nested <- nestedLogit(partic ~ hincome + children,
                          dichotomies = comparisons,
                          data=Womenlf)
# get predicted values for a grid of `hincome` and `children`
new <- expand.grid(hincome=seq(0, 45, length=10),
                   children=c("absent", "present"))

pred.nested <- predict(wlf.nested, new)
plotdata <- as.data.frame(pred.nested)
str(plotdata)

# Predicted logit values for the dichotomies
pred.dichot <- predict(wlf.nested, new, model = "dichotomies")
plotlogit <- as.data.frame(pred.dichot)
str(plotlogit)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plot.nested-ci.R
\name{plot.nestedLogit}
\alias{plot.nestedLogit}
\title{Plotting Nested Logit Models}
\usage{
\method{plot}{nestedLogit}(
  x,
  x.var,
  others,
  n.x.values = 100L,
  scale = c("prob", "logit"),
  xlab = x.var,
  ylab = NULL,
  main,
  cex.main = 1,
  digits.main = getOption("digits") - 2L,
  font.main = 1L,
  pch = 1L:length(response.levels),
  lwd = 3,
  lty = 1L:length(response.levels),
  col = (scales::hue_pal())(length(response.levels)),
  legend = TRUE,
  legend.inset = 0.01,
  legend.location = "topleft",
  legend.bty = "n",
  conf.level = 0.95,
  conf.alpha = 0.25,
  label = FALSE,
  label.x = "max",
  label.cex = 1.25,
  label.col = col,
  ...
)
}
\arguments{
\item{x}{an object of \code{"nestedLogit"} produced by \code{\link{nestedLogit}}.}

\item{x.var}{quoted name of the variable to appear on the x-axis; if omitted, the first
predictor in the model is used.}

\item{others}{a named list of values for the other variables in the model,
that is, other than \code{x.var}; if any other predictor is omitted, it is set
to an arbitrary value---the mean for a numeric predictor or the first level or
value of a factor, character, or logical predictor; only one value may be
specified for each variable in \code{others}.}

\item{n.x.values}{the number of evenly spaced values of \code{x.var} at which
to evaluate fitted probabilities to be plotted (default \code{100}).}

\item{scale}{character string; \code{"prob"} (the default) plots fitted probabilities
on the y-axis; \code{"logit"} plots fitted log odds (logits), i.e., \eqn{\log(p/(1-p))}.}

\item{xlab}{label for the x-axis (defaults to the value of \code{x.var}).}

\item{ylab}{label for the y-axis (defaults to \code{"Fitted Probability"} when
\code{scale = "prob"} and \code{"Fitted Log Odds"} when \code{scale = "logit"}).}

\item{main}{main title for the graph (if missing, constructed from the variables and
values in \code{others}).}

\item{cex.main}{size of main title (see \code{\link{par}}).}

\item{digits.main}{number of digits to retain when rounding values for the main title.}

\item{font.main}{font for main title (see \code{\link{par}}).}

\item{pch}{plotting characters (see \code{\link{par}}).}

\item{lwd}{line width (see \code{\link{par}}).}

\item{lty}{line types (see \code{\link{par}}).}

\item{col}{line colors for the response levels (see \code{\link{par}}).}

\item{legend}{if \code{TRUE} (the default), add a legend for the
response levels to the graph. Ignored when \code{label = TRUE}.}

\item{legend.inset}{default \code{0.01} (see \code{\link{legend}}).}

\item{legend.location}{position of the legend (default \code{"topleft"},
see \code{\link{legend}}).}

\item{legend.bty}{the type of box to be drawn around the legend. The allowed values are "o" (the default) and "n".}

\item{conf.level}{the level for pointwise confidence envelopes around the predicted values;
the default is \code{0.95}. If \code{NULL}, the confidence envelopes are suppressed.}

\item{conf.alpha}{the opacity of the confidence envelopes; the default is \code{0.3}.}

\item{label}{if \code{TRUE}, label the curves directly instead of using a legend.
Default is \code{FALSE}.}

\item{label.x}{where to place the label for each curve. Either a single string,
\code{"min"} or \code{"max"} (applied to all curves), or a character vector of
length equal to the number of response categories with each element being
\code{"min"} or \code{"max"}, e.g. \code{label.x = c("min", "max", "max")}.
\code{"min"} places the label at the left end of the curve;
\code{"max"} (the default) places it at the right end.}

\item{label.cex}{character expansion factor for direct labels; default \code{1.25}.}

\item{label.col}{colors for direct labels; defaults to \code{col}, so labels match
their curves. Supply a vector of length equal to the number of response
categories to use different colors for the labels.}

\item{\dots}{arguments to be passed to \code{\link{matplot}}.}
}
\value{
NULL Used for its side-effect of producing a plot
}
\description{
A \code{\link{plot}} method for \code{"nestedLogit"} objects produced by the
\code{\link{nestedLogit}} function. Fitted probabilities under the model,
or the corresponding logits are plotted
for each level of the polytomous response variable, with one of the explanatory variables
on the horizontal axis and other explanatory variables fixed to particular values.
By default, a 95\% pointwise confidence envelope is added to the plot.
}
\examples{
data("Womenlf", package = "carData")
m <- nestedLogit(partic ~ hincome + children,
                 logits(work=dichotomy("not.work", c("parttime", "fulltime")),
                        full=dichotomy("parttime", "fulltime")),
                        data=Womenlf)
plot(m, legend.location="top")

op <- par(mfcol=c(1, 2), mar=c(4, 4, 3, 1) + 0.1)
plot(m, "hincome", list(children="absent"),
     xlab="Husband's Income", legend=FALSE)
plot(m, "hincome", list(children="present"),
     xlab="Husband's Income")
par(op)

# Plot on the logit (log-odds) scale
plot(m, "hincome", list(children="absent"), scale = "logit",
     xlab = "Husband's Income")

# Gators example: direct curve labels instead of a legend
data("gators", package = "nestedLogit")
gators.nested <- nestedLogit(food ~ length,
  logits(d1 = dichotomy("Other", c("Fish", "Invertebrates")),
         d2 = dichotomy("Fish", "Invertebrates")),
  data = gators)

# All labels at the right end (default)
plot(gators.nested, x.var = "length", label = TRUE,
     xlab = "Alligator length (m)")

# Mixed placement: Other and Invertebrates labeled at left, Fish at right
# (food levels are: "Other", "Fish", "Invertebrates")
plot(gators.nested, x.var = "length", label = TRUE,
     label.x = c("min", "max", "min"),
     xlab = "Alligator length (m)")

}
\seealso{
\code{\link{nestedLogit}}, \code{\link[graphics]{matplot}}
}
\author{
John Fox, Michael Friendly
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/nestedMethods.R
\name{nestedMethods}
\alias{nestedMethods}
\alias{print.nestedLogit}
\alias{summary.nestedLogit}
\alias{print.summary.nestedLogit}
\alias{update.nestedLogit}
\alias{coef.nestedLogit}
\alias{vcov.nestedLogit}
\alias{print.dichotomies}
\alias{as.dichotomies.matrix}
\alias{as.matrix.continuationDichotomies}
\alias{as.character.dichotomies}
\alias{as.matrix.dichotomies}
\alias{as.dichotomies}
\title{Methods for \code{"nestedLogit"} and Related Objects}
\usage{
\method{print}{nestedLogit}(x, ...)

\method{summary}{nestedLogit}(object, ...)

\method{print}{summary.nestedLogit}(x, ...)

\method{print}{dichotomies}(x, ...)

\method{coef}{nestedLogit}(object, as.matrix = TRUE, ...)

\method{vcov}{nestedLogit}(object, as.matrix = FALSE, ...)

\method{update}{nestedLogit}(object, formula, dichotomies, data, subset, contrasts, ...)

\method{as.matrix}{dichotomies}(x, ...)

\method{as.character}{dichotomies}(x, ...)

\method{as.matrix}{continuationDichotomies}(x, ...)

as.dichotomies(x, ...)

\method{as.dichotomies}{matrix}(x, ...)
}
\arguments{
\item{x, object}{in most cases, an object of class \code{"nestedLogit"}.}

\item{\dots}{arguments to be passed down.}

\item{as.matrix}{if \code{TRUE} (the default for \code{coef}) return coefficients
as a matrix with one column for each nested dichotomy,
or coefficient covariances as a matrix with one row and column for each
combination of dichotomies and coefficients; if \code{FALSE} (the default for
\code{vcov}), return a list of coefficients or coefficient covariances
with one element for each dichotomy.}

\item{formula}{optional updated model formula.}

\item{dichotomies}{optional updated dichotomies object.}

\item{data}{optional updated data argument}

\item{subset}{optional updated subset argument.}

\item{contrasts}{optional updated contrasts argument.}
}
\value{
\itemize{
\item The \code{coef} and \code{vcov} methods return either matrices or lists of regression
coefficients and their covariances, respectively.
\item The \code{update} method returns an object of class \code{"nestedLogit"} (see \code{\link{nestedLogit}})
derived from the original nested-logit model.
\item The \code{summary} method returns an object of class \code{"summary.nestedLogit"}, which is
a list of summaries of the \code{\link{glm}} objects that comprise the nested-dichotomies model; the
object is normally printed.
\item The methods for \code{as.matrix}, \code{as.character}, and \code{as.dichotomies} coerce
various objects to matrices, character vectors, and dichotomies objects.
\item The various \code{print} methods invisibly return their \code{x} arguments.
}
}
\description{
Various methods for processing \code{"nestedLogit"} and related objects.
Most of these are the standard methods for a model-fitting function.
\describe{
\item{\code{coef}, \code{vcov}}{Return the coefficients and their variance-covariance matrix respectively.}
\item{\code{update}}{Re-fit a \code{"nestedLogit"} model with a change in any of the \code{formula}, \code{dichotomies},
\code{data}, \code{subset}, or \code{contrasts}, arguments.}
\item{\code{summary}}{Summarize a \code{"nestedLogit"} model, giving the summary for each binary logit model in the
nested dichotomies.}
\item{\code{print}}{Print the model or a summary of the model.}
\item{\code{as.matrix}, \code{as.character}, \code{as.dichotomies}}{Coerce
dichotomy-related objects to matrices, character vectors, and dichotomies objects.}
}
}
\examples{
# define continuation dichotomies for level of education
cont.dichots <- continuationLogits(c("<highschool",
                                     "highschool",
                                     "college",
                                     "graduate"))

# Show dichotomies in various forms
print(cont.dichots)
as.matrix(cont.dichots)
as.character(cont.dichots)

# fit a nested model for the GSS data examining education degree in relation to parent & year
m <- nestedLogit(degree ~ parentdeg + year,
                 cont.dichots,
                 data=GSS)

coef(m)                             # coefficient estimates
sqrt(diag(vcov(m, as.matrix=TRUE))) # standard errors
print(m)
summary(m)

}
\seealso{
\code{\link{nestedLogit}}, \code{\link{predict.nestedLogit}},
\code{\link{plot.nestedLogit}},
\code{\link{glance.nestedLogit}}, \code{\link{tidy.nestedLogit}}
}
\author{
John Fox and Michael Friendly
}
\keyword{regression}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/models.R
\name{models}
\alias{models}
\alias{models.nestedLogit}
\title{Extract Binary Logit Models from a \code{nestedLogit} Object}
\usage{
models(model, select, as.list = FALSE)

\method{models}{nestedLogit}(model, select, as.list = FALSE)
}
\arguments{
\item{model}{a \code{"nestedLogit"} model.}

\item{select}{a numeric or character vector giving the number(s) or names(s)
of one or more
binary logit models to be extracted from \code{model}; if absent, a list of
all of the binary logits models in \code{model} is returned.}

\item{as.list}{if \code{TRUE} (the default is \code{FALSE}) and one binary logit
model is selected, return the \code{"glm"} object in a one-element named list;
otherwise a single model is returned directly as a \code{"glm"} object;
when more than one binary
logit model is selected, the corresponding \code{"glm"} objects are \emph{always}
returned as a named list.}
}
\value{
\code{model} returns either a single \code{"glm"} object (see \code{\link{glm}}) or a
list of \code{"glm"} objects, each representing a binary logit model.
}
\description{
Nested logit models represent an overall models for a polytomous response (>2 categories)
by a set of binary logit models corresponding to nested dichotomies among the response
categories.
\code{models} is used to extract \code{"glm"} objects representing binary logit
models from a \code{"nestedLogit"} object.
}
\examples{
  data("Womenlf", package = "carData")
  comparisons <- logits(work=dichotomy("not.work",
                                       working=c("parttime", "fulltime")),
                        full=dichotomy("parttime", "fulltime"))
  m <- nestedLogit(partic ~ hincome + children,
                   dichotomies = comparisons,
                   data=Womenlf)

  # extract both submodels, as a list
  models(m, c("work", "full"))

  # extract the binomial logit model for working vs. non-working
  m_work <- models(m, "work")

  # use that to plot residuals
  plot(density(residuals(m_work)))

  # or plot that model -- gives the 'regression quartet' for a glm()
  op <- par(mfrow = c(2,2))
  plot(m_work)
  par(op)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/broomMethods.R
\name{broomMethods}
\alias{broomMethods}
\alias{glance.nestedLogit}
\alias{tidy.nestedLogit}
\title{Broom Related Methods}
\usage{
\method{glance}{nestedLogit}(x, ...)

\method{tidy}{nestedLogit}(x, ...)
}
\arguments{
\item{x}{an object of class \code{"nestedLogit"}.}

\item{\dots}{arguments to be passed down.}
}
\value{
\itemize{
\item \code{glance} returns a \code{tibble} containing one row of fit statistics for each dichotomy,
labeled \code{response}. See \code{\link[broom]{glance}} for details.
\item \code{tidy} returns a \code{tibble} containing coefficient estimates and test statistics for
the combinations of \code{response} and \code{term}. See \code{\link[broom]{tidy}} for details.
}
}
\description{
These functions give compact summaries of a \code{"nestedLogit"} object
\describe{
\item{\code{glance}}{Construct a single row summaries for the dichotomies \code{"nestedLogit"} model.}
\item{\code{tidy}}{Summarizes the terms in \code{"nestedLogit"} model.}
}
}
\examples{
data("Womenlf", package = "carData")
m <-  nestedLogit(partic ~ hincome + children,
                  dichotomies = logits(work=dichotomy("not.work",
                                                      working=c("parttime", "fulltime")),
                                       full=dichotomy("parttime", "fulltime")),
                  data=Womenlf)

# one-line summaries
broom::glance(m)
# coefficients and tests
broom::tidy(m)

}
\seealso{
\code{\link{nestedMethods}}, \code{\link[broom]{glance}}, \code{\link[broom]{tidy}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/predict.nestedLogit.R
\name{predict.nestedLogit}
\alias{predict.nestedLogit}
\alias{fitted.nestedLogit}
\alias{print.predictNestedLogit}
\alias{confint.predictNestedLogit}
\alias{print.predictDichotomies}
\title{Predicted Probabilities and Logits for \code{"nestedLogit"} Models}
\usage{
\method{predict}{nestedLogit}(object, newdata, model = c("nested", "dichotomies"), ...)

\method{print}{predictNestedLogit}(x, n = min(10L, nrow(x$p)), ...)

\method{confint}{predictNestedLogit}(
  object,
  parm = c("prob", "logit"),
  level = 0.95,
  conf.limits.logit = TRUE,
  ...
)

\method{print}{predictDichotomies}(x, n = 10L, ...)

\method{fitted}{nestedLogit}(object, model = c("nested", "dichotomies"), ...)
}
\arguments{
\item{object}{a fitted object of class \code{"nestedLogit"};
for \code{confint}, an object of class \code{"predictNestedLogit"}.}

\item{newdata}{a data frame containing combinations of values of the predictors
at which fitted probabilities (or other quantities) are to be computed.
If missing, the original data are used.}

\item{model}{either \code{"nested"} (the default), in which case fitted probabilities
under the nested logit model are returned, or \code{"dichotomies"}, in which case
\code{\link{predict.glm}} is invoked for each binary logit model fit to the nested
dichotomies and a named list of the results is returned.}

\item{\dots}{arguments to be passed down.}

\item{x}{an object of class \code{"predictNestedLogit"} or \code{"predictDichotomies"}.}

\item{n}{an integer or \code{"all"} to control how many rows are printed for each of the
predicted values.}

\item{parm}{one of \code{"prob"} or \code{"logit"},
indicating whether to generate confidence intervals for probabilities or logits
of the responses.}

\item{level}{confidence level, a number between 0 and 1; default is \code{0.95}.}

\item{conf.limits.logit}{logical; when \code{parm = "prob"}, if \code{TRUE} (the default),
confidence limits are computed on the logit scale and back-transformed to probabilities;
if \code{FALSE}, Wald-type limits are computed directly on the probability scale.}
}
\value{
\itemize{
\item The \code{predict} and \code{fitted} methods return an object of class \code{"predictNestedLogit"}
(when \code{model = "nested"}) or \code{"predictDichotomies"} (when \code{model = "dichotomies"}).

A \code{"predictNestedLogit"} object is a list containing:
\describe{
\item{\code{p}}{a data frame of predicted probabilities, with one column per response category.}
\item{\code{logit}}{a data frame of predicted logits.}
\item{\code{se.p}}{a data frame of standard errors of predicted probabilities.}
\item{\code{se.logit}}{a data frame of standard errors of predicted logits.}
\item{\code{.data}}{the \code{newdata} data frame, if supplied.}
}

A \code{"predictDichotomies"} object is a named list of data frames, one per dichotomy,
each produced by \code{\link{predict.glm}}.

\item The \code{confint} method returns a data frame of point estimates and confidence limits.
\item The various \code{print} methods invisibly return their \code{x} arguments.
}
}
\description{
The \code{predict} and \code{fitted} methods compute predicted values from a fitted
\code{"nestedLogit"} model.
The \code{confint} method computes point-wise confidence limits for predicted response-category
probabilities or logits.

\describe{
\item{\code{predict}, \code{fitted}}{Compute predicted response-category probabilities
(or predicted logits for each binary logit model in the dichotomies) from a fitted
\code{"nestedLogit"} model.}
\item{\code{confint}}{Compute point-wise confidence limits for predicted response-category
probabilities or logits.}
\item{\code{print} methods}{Print predicted probabilities, logits, and their standard errors,
with control over how many rows to display.}
}
}
\details{
The \code{predict} method provides predicted values for two representations of the model.
\code{model = "nested"} (the default) gives the fitted probabilities for each of the response categories,
along with the corresponding logits and standard errors of each.
\code{model = "dichotomies"} gives the fitted log odds for each of the binary logit models in the
nested dichotomies.

The \code{fitted} method (with no \code{newdata}) is equivalent to \code{predict} applied to the
original data used to fit the model.

For the \code{confint} method with \code{parm = "prob"}, setting
\code{conf.limits.logit = TRUE} (the default) computes confidence limits on the logit scale
and back-transforms them to probabilities, which ensures that the limits lie in \eqn{[0, 1]}.
Setting \code{conf.limits.logit = FALSE} computes Wald-type confidence intervals directly on the
probability scale, which may extend outside \eqn{[0, 1]}.
}
\examples{
# define continuation dichotomies for level of education
cont.dichots <- continuationLogits(c("<highschool",
                                     "highschool",
                                     "college",
                                     "graduate"))

# fit a nested model for the GSS data
m <- nestedLogit(degree ~ parentdeg + year,
                 cont.dichots,
                 data=GSS)

# predicted probabilities for first few cases
predict(m)

# predicted probabilities at specific values of predictors
new <- expand.grid(parentdeg=c("<highschool",  "highschool",
                               "college", "graduate"),
                   year=c(1972, 2016))

fit <- predict(m, newdata=new)
cbind(new, fit)

# use fitted() -- equivalent to predict() on the original data
f <- fitted(m)

# predicted logits for each dichotomy
pred.dichot <- predict(m, newdata=new, model="dichotomies")
pred.dichot

# confidence intervals for predicted probabilities
fit.ci <- confint(fit)
head(fit.ci)

# confidence intervals for predicted logits
fit.ci.logit <- confint(fit, parm="logit")
head(fit.ci.logit)
}
\seealso{
\code{\link{nestedLogit}}, \code{\link{nestedMethods}},
\code{\link{plot.nestedLogit}},
\code{\link{as.data.frame.predictNestedLogit}}
}
\author{
John Fox and Michael Friendly
}
\keyword{regression}
